@@ -1,51 +1,10 @@
-from __future__ import (
-    absolute_import,
-    division,
-    print_function,
-    unicode_literals,
-)
-
-import sys
-
 from pcs import (
     prop,
-    usage,
     utils,
 )
-from pcs.lib.pacemaker_values import is_true
-from pcs.cli.common.console_report import indent
 from pcs.cli.common.errors import CmdLineInputError
-from pcs.lib.errors import LibraryError
-
-def acl_cmd(lib, argv, modifiers):
-    if len(argv) < 1:
-        sub_cmd, argv_next = "show", []
-    else:
-        sub_cmd, argv_next = argv[0], argv[1:]
-
-    try:
-        if sub_cmd == "help":
-            usage.acl(argv_next)
-        elif sub_cmd == "show":
-            show_acl_config(lib, argv_next, modifiers)
-        elif sub_cmd == "enable":
-            acl_enable(argv_next)
-        elif sub_cmd == "disable":
-            acl_disable(argv_next)
-        elif sub_cmd == "role":
-            acl_role(lib, argv_next, modifiers)
-        elif sub_cmd in ["target", "user"]:
-            acl_user(lib, argv_next, modifiers)
-        elif sub_cmd == "group":
-            acl_group(lib, argv_next, modifiers)
-        elif sub_cmd == "permission":
-            acl_permission(lib, argv_next, modifiers)
-        else:
-            raise CmdLineInputError()
-    except LibraryError as e:
-        utils.process_library_reports(e.args)
-    except CmdLineInputError as e:
-        utils.exit_on_cmdline_input_errror(e, "acl", sub_cmd)
+from pcs.common.tools import indent
+from pcs.lib.pacemaker.values import is_true
 
 
 def _print_list_of_objects(obj_list, transformation_fn):
@@ -57,10 +16,19 @@ def _print_list_of_objects(obj_list, transformation_fn):
 
 
 def show_acl_config(lib, argv, modifiers):
+    """
+    Options:
+      * -f - CIB file
+    """
     # TODO move to lib once lib supports cluster properties
     # enabled/disabled should be part of the structure returned
     # by lib.acl.get_config
-    properties = utils.get_set_properties(defaults=prop.get_default_properties())
+    modifiers.ensure_only_supported("-f")
+    if argv:
+        raise CmdLineInputError()
+    properties = utils.get_set_properties(
+        defaults=prop.get_default_properties()
+    )
     acl_enabled = properties.get("enable-acl", "").lower()
     if is_true(acl_enabled):
         print("ACLs are enabled")
@@ -74,118 +42,79 @@ def show_acl_config(lib, argv, modifiers):
     _print_list_of_objects(data.get("role_list", []), role_to_str)
 
 
-def acl_enable(argv):
+def acl_enable(lib, argv, modifiers):
+    """
+    Options:
+      * -f - CIB file
+    """
     # TODO move to lib once lib supports cluster properties
-    prop.set_property(["enable-acl=true"])
+    modifiers.ensure_only_supported("-f")
+    if argv:
+        raise CmdLineInputError()
+    prop.set_property(lib, ["enable-acl=true"], modifiers.get_subset("-f"))
 
-def acl_disable(argv):
+def acl_disable(lib, argv, modifiers):
+    """
+    Options:
+      * -f - CIB file
+    """
     # TODO move to lib once lib supports cluster properties
-    prop.set_property(["enable-acl=false"])
-
-
-def acl_role(lib, argv, modifiers):
-    if len(argv) < 1:
+    modifiers.ensure_only_supported("-f")
+    if argv:
         raise CmdLineInputError()
-
-    sub_cmd, argv_next = argv[0], argv[1:]
-    try:
-        if sub_cmd == "create":
-            role_create(lib, argv_next, modifiers)
-        elif sub_cmd == "delete":
-            role_delete(lib, argv_next, modifiers)
-        elif sub_cmd == "assign":
-            role_assign(lib, argv_next, modifiers)
-        elif sub_cmd == "unassign":
-            role_unassign(lib, argv_next, modifiers)
-        else:
-            usage.show("acl", ["role"])
-            sys.exit(1)
-    except CmdLineInputError as e:
-        utils.exit_on_cmdline_input_errror(e, "acl", "role {0}".format(sub_cmd))
+    prop.set_property(lib, ["enable-acl=false"], modifiers.get_subset("-f"))
 
 
-def acl_user(lib, argv, modifiers):
-    if len(argv) < 1:
-        raise CmdLineInputError()
-
-    sub_cmd, argv_next = argv[0], argv[1:]
-    try:
-        if sub_cmd == "create":
-            user_create(lib, argv_next, modifiers)
-        elif sub_cmd == "delete":
-            user_delete(lib, argv_next, modifiers)
-        else:
-            usage.show("acl", ["user"])
-            sys.exit(1)
-    except CmdLineInputError as e:
-        utils.exit_on_cmdline_input_errror(e, "acl", "user {0}".format(sub_cmd))
-
-
-def user_create(lib, argv, dummy_modifiers):
-    if len(argv) < 1:
+def user_create(lib, argv, modifiers):
+    """
+    Options:
+      * -f - CIB file
+    """
+    modifiers.ensure_only_supported("-f")
+    if not argv:
         raise CmdLineInputError()
     user_name, role_list = argv[0], argv[1:]
     lib.acl.create_target(user_name, role_list)
 
 
-def user_delete(lib, argv, dummy_modifiers):
+def user_delete(lib, argv, modifiers):
+    """
+    Options:
+      * -f - CIB file
+    """
+    modifiers.ensure_only_supported("-f")
     if len(argv) != 1:
         raise CmdLineInputError()
     lib.acl.remove_target(argv[0])
 
 
-def acl_group(lib, argv, modifiers):
-    if len(argv) < 1:
-        raise CmdLineInputError()
-
-    sub_cmd, argv_next = argv[0], argv[1:]
-    try:
-        if sub_cmd == "create":
-            group_create(lib, argv_next, modifiers)
-        elif sub_cmd == "delete":
-            group_delete(lib, argv_next, modifiers)
-        else:
-            usage.show("acl", ["group"])
-            sys.exit(1)
-    except CmdLineInputError as e:
-        utils.exit_on_cmdline_input_errror(
-            e, "acl", "group {0}".format(sub_cmd)
-        )
-
-
-def group_create(lib, argv, dummy_modifiers):
-    if len(argv) < 1:
+def group_create(lib, argv, modifiers):
+    """
+    Options:
+      * -f - CIB file
+    """
+    modifiers.ensure_only_supported("-f")
+    if not argv:
         raise CmdLineInputError()
     group_name, role_list = argv[0], argv[1:]
     lib.acl.create_group(group_name, role_list)
 
 
-def group_delete(lib, argv, dummy_modifiers):
+def group_delete(lib, argv, modifiers):
+    """
+    Options:
+      * -f - CIB file
+    """
+    modifiers.ensure_only_supported("-f")
     if len(argv) != 1:
         raise CmdLineInputError()
     lib.acl.remove_group(argv[0])
 
 
-def acl_permission(lib, argv, modifiers):
-    if len(argv) < 1:
-        raise CmdLineInputError()
-
-    sub_cmd, argv_next = argv[0], argv[1:]
-    try:
-        if sub_cmd == "add":
-            permission_add(lib, argv_next, modifiers)
-        elif sub_cmd == "delete":
-            run_permission_delete(lib, argv_next, modifiers)
-        else:
-            usage.show("acl", ["permission"])
-            sys.exit(1)
-    except CmdLineInputError as e:
-        utils.exit_on_cmdline_input_errror(
-            e, "acl", "permission {0}".format(sub_cmd)
-        )
-
-
 def argv_to_permission_info_list(argv):
+    """
+    Commandline options: no options
+    """
     if len(argv) % 3 != 0:
         raise CmdLineInputError()
 
@@ -210,7 +139,12 @@ def argv_to_permission_info_list(argv):
 
 
 def role_create(lib, argv, modifiers):
-    if len(argv) < 1:
+    """
+    Options:
+      * -f - CIB file
+    """
+    modifiers.ensure_only_supported("-f")
+    if not argv:
         raise CmdLineInputError()
 
     role_id = argv.pop(0)
@@ -224,13 +158,25 @@ def role_create(lib, argv, modifiers):
 
 
 def role_delete(lib, argv, modifiers):
+    """
+    Options:
+      * -f - CIB file
+      * --autodelete - autodelete empty targets, groups
+    """
+    modifiers.ensure_only_supported("-f", "--autodelete")
     if len(argv) != 1:
         raise CmdLineInputError()
 
-    lib.acl.remove_role(argv[0], autodelete_users_groups=True)
+    lib.acl.remove_role(
+        argv[0],
+        autodelete_users_groups=modifiers.get("--autodelete")
+    )
 
 
 def _role_assign_unassign(argv, keyword, not_specific_fn, user_fn, group_fn):
+    """
+    Commandline options: no options
+    """
     argv_len = len(argv)
     if argv_len < 2:
         raise CmdLineInputError()
@@ -257,7 +203,12 @@ def _role_assign_unassign(argv, keyword, not_specific_fn, user_fn, group_fn):
         raise CmdLineInputError()
 
 
-def role_assign(lib, argv, dummy_modifiers):
+def role_assign(lib, argv, modifiers):
+    """
+    Options:
+      * -f - CIB file
+    """
+    modifiers.ensure_only_supported("-f")
     _role_assign_unassign(
         argv,
         "to",
@@ -268,29 +219,45 @@ def role_assign(lib, argv, dummy_modifiers):
 
 
 def role_unassign(lib, argv, modifiers):
+    """
+    Options:
+      * -f - CIB file
+      * --autodelete - autodelete empty targets, groups
+    """
+    modifiers.ensure_only_supported("-f", "--autodelete")
     _role_assign_unassign(
         argv,
         "from",
         lambda role_id, ug_id: lib.acl.unassign_role_not_specific(
-            role_id, ug_id, modifiers.get("autodelete", False)
+            role_id, ug_id, modifiers.get("--autodelete")
         ),
         lambda role_id, ug_id: lib.acl.unassign_role_from_target(
-            role_id, ug_id, modifiers.get("autodelete", False)
+            role_id, ug_id, modifiers.get("--autodelete")
         ),
         lambda role_id, ug_id: lib.acl.unassign_role_from_group(
-            role_id, ug_id, modifiers.get("autodelete", False)
+            role_id, ug_id, modifiers.get("--autodelete")
         )
     )
 
 
-def permission_add(lib, argv, dummy_modifiers):
+def permission_add(lib, argv, modifiers):
+    """
+    Options:
+      * -f - CIB file
+    """
+    modifiers.ensure_only_supported("-f")
     if len(argv) < 4:
         raise CmdLineInputError()
     role_id, argv_next = argv[0], argv[1:]
     lib.acl.add_permission(role_id, argv_to_permission_info_list(argv_next))
 
 
-def run_permission_delete(lib, argv, dummy_modifiers):
+def run_permission_delete(lib, argv, modifiers):
+    """
+    Options:
+      * -f - CIB file
+    """
+    modifiers.ensure_only_supported("-f")
     if len(argv) != 1:
         raise CmdLineInputError()
     lib.acl.remove_permission(argv[0])
@@ -325,4 +292,3 @@ def _permission_to_str(permission):
         out += ["id", permission.get("reference")]
     out.append("({0})".format(permission.get("id")))
     return " ".join(out)
-
