@@ -1,42 +1,12 @@
 import json
 from functools import partial
 
-from pcs import usage
 from pcs.cli.common.errors import CmdLineInputError
 from pcs.cli.common.parse_args import prepare_options, group_by_keywords
-from pcs.cli.common.console_report import indent
-from pcs.cli.common.routing import create_router
+from pcs.common.tools import indent
 
 parse_cmd_sections = partial(group_by_keywords, implicit_first_group_key="main")
 
-def alert_cmd(lib, argv, modifiers):
-    create_router(
-        {
-            "help": lambda _lib, _argv, _modifiers: usage.alert(_argv),
-            "create": alert_add,
-            "update": alert_update,
-            "delete": alert_remove,
-            "remove": alert_remove,
-            "config": print_alert_config,
-            "show": print_alert_config,
-            "recipient": create_router(
-                {
-                    "help": lambda _lib, _argv, _modifiers: usage.alert(
-                        ["recipient"]
-                    ),
-                    "add": recipient_add,
-                    "update": recipient_update,
-                    "delete": recipient_remove,
-                    "remove": recipient_remove,
-                },
-                ["alert", "recipient"],
-            ),
-            "get_all_alerts": print_alerts_in_json,
-        },
-        ["alert"],
-        default_cmd="config",
-    )(lib, argv, modifiers)
-
 
 def ensure_only_allowed_options(parameter_dict, allowed_list):
     for arg, value in parameter_dict.items():
@@ -170,11 +140,16 @@ def recipient_remove(lib, argv, modifiers):
 
 
 def _nvset_to_str(nvset_obj):
+    # TODO duplicite to pcs.resource._nvpairs_strings
+    key_val = {
+        nvpair_obj["name"]: nvpair_obj["value"]
+        for nvpair_obj in nvset_obj
+    }
     output = []
-    for nvpair_obj in nvset_obj:
-        output.append("{key}={value}".format(
-            key=nvpair_obj["name"], value=nvpair_obj["value"]
-        ))
+    for name, value in sorted(key_val.items()):
+        if " " in value:
+            value = f'"{value}"'
+        output.append(f"{name}={value}")
     return " ".join(output)
 
 
@@ -224,15 +199,17 @@ def print_alert_config(lib, argv, modifiers):
     modifiers.ensure_only_supported("-f")
     if argv:
         raise CmdLineInputError()
+    print("\n".join(alert_config_lines(lib)))
 
-    print("Alerts:")
+def alert_config_lines(lib):
+    lines = ["Alerts:"]
     alert_list = lib.alert.get_all_alerts()
     if alert_list:
         for alert in alert_list:
-            print("\n".join(indent(_alert_to_str(alert), 1)))
+            lines.extend(indent(_alert_to_str(alert), 1))
     else:
-        print(" No alerts defined")
-
+        lines.append(" No alerts defined")
+    return lines
 
 def print_alerts_in_json(lib, argv, modifiers):
     """
