@@ -23,18 +23,11 @@ def config_setup(lib, arg_list, modifiers):
     if "sites" not in peers or not peers["sites"]:
         raise CmdLineInputError()
 
-    booth_config = []
-    for site in peers["sites"]:
-        booth_config.append({"key": "site", "value": site, "details": []})
-    for arbitrator in peers["arbitrators"]:
-        booth_config.append({
-            "key": "arbitrator",
-            "value": arbitrator,
-            "details": [],
-        })
-
     lib.booth.config_setup(
-        booth_config, overwrite_existing=modifiers.get("--force")
+        peers["sites"],
+        peers["arbitrators"],
+        instance_name=modifiers.get("--name"),
+        overwrite_existing=modifiers.get("--force"),
     )
 
 def config_destroy(lib, arg_list, modifiers):
@@ -49,6 +42,7 @@ def config_destroy(lib, arg_list, modifiers):
     if arg_list:
         raise CmdLineInputError()
     lib.booth.config_destroy(
+        instance_name=modifiers.get("--name"),
         ignore_config_load_problems=modifiers.get("--force")
     )
 
@@ -59,20 +53,19 @@ def config_show(lib, arg_list, modifiers):
 
     Options:
       * --name - name of a booth instace
-      * --booth-conf - booth config file, effective only when no node is
-        specified
-      * --booth-key - booth auth key file, not required by the command (just
-        from middleware)
       * --request-timeout - HTTP timeout for getting config from remote host
     """
-    modifiers.ensure_only_supported(
-        "--name", "--booth-conf", "--request-timeout", "--booth-key"
-    )
+    modifiers.ensure_only_supported("--name", "--request-timeout")
     if len(arg_list) > 1:
         raise CmdLineInputError()
     node = None if not arg_list else arg_list[0]
 
-    print(lib.booth.config_text(node_name=node).rstrip())
+    print(
+        lib.booth.config_text(
+            instance_name=modifiers.get("--name"),
+            node_name=node
+        ).decode("utf-8").rstrip()
+    )
 
 
 def config_ticket_add(lib, arg_list, modifiers):
@@ -80,10 +73,9 @@ def config_ticket_add(lib, arg_list, modifiers):
     add ticket to current configuration
 
     Options:
-      * --force - ignore config load issues
+      * --force
       * --booth-conf - booth config file
-      * --booth-key - booth auth key file, not required by the command (just
-        from middleware)
+      * --booth-key - booth auth key file
       * --name - name of a booth instace
     """
     modifiers.ensure_only_supported(
@@ -94,6 +86,7 @@ def config_ticket_add(lib, arg_list, modifiers):
     lib.booth.config_ticket_add(
         arg_list[0],
         prepare_options(arg_list[1:]),
+        instance_name=modifiers.get("--name"),
         allow_unknown_options=modifiers.get("--force")
     )
 
@@ -103,20 +96,21 @@ def config_ticket_remove(lib, arg_list, modifiers):
 
     Options:
       * --booth-conf - booth config file
-      * --booth-key - booth auth key file, not required by the command (just
-        from middleware)
+      * --booth-key - booth auth key file
       * --name - name of a booth instace
     """
     modifiers.ensure_only_supported("--booth-conf", "--name", "--booth-key")
     if len(arg_list) != 1:
         raise CmdLineInputError
-    lib.booth.config_ticket_remove(arg_list[0])
+    lib.booth.config_ticket_remove(
+        arg_list[0],
+        instance_name=modifiers.get("--name"),
+    )
 
-def _ticket_operation(lib_call, arg_list):
+def _ticket_operation(lib_call, arg_list, booth_name):
     """
     Commandline options:
       * --name - name of a booth instance
-      * -f - CIB file
     """
     site_ip = None
     if len(arg_list) == 2:
@@ -125,25 +119,27 @@ def _ticket_operation(lib_call, arg_list):
         raise CmdLineInputError()
 
     ticket = arg_list[0]
-    lib_call(ticket, site_ip)
+    lib_call(ticket, site_ip=site_ip, instance_name=booth_name)
 
 def ticket_revoke(lib, arg_list, modifiers):
     """
     Options:
       * --name - name of a booth instance
-      * -f - CIB file
     """
-    modifiers.ensure_only_supported("-f", "--name")
-    _ticket_operation(lib.booth.ticket_revoke, arg_list)
+    modifiers.ensure_only_supported("--name")
+    _ticket_operation(
+        lib.booth.ticket_revoke, arg_list, modifiers.get("--name")
+    )
 
 def ticket_grant(lib, arg_list, modifiers):
     """
     Options:
       * --name - name of a booth instance
-      * -f - CIB file
     """
-    modifiers.ensure_only_supported("-f", "--name")
-    _ticket_operation(lib.booth.ticket_grant, arg_list)
+    modifiers.ensure_only_supported("--name")
+    _ticket_operation(
+        lib.booth.ticket_grant, arg_list, modifiers.get("--name")
+    )
 
 def create_in_cluster(lib, arg_list, modifiers):
     """
@@ -157,7 +153,8 @@ def create_in_cluster(lib, arg_list, modifiers):
     if len(arg_list) != 2 or arg_list[0] != "ip":
         raise CmdLineInputError()
     lib.booth.create_in_cluster(
-        ip=arg_list[1],
+        arg_list[1],
+        instance_name=modifiers.get("--name"),
         allow_absent_resource_agent=modifiers.get("--force")
     )
 
@@ -177,6 +174,7 @@ def get_remove_from_cluster(resource_remove):
 
         lib.booth.remove_from_cluster(
             resource_remove,
+            instance_name=modifiers.get("--name"),
             allow_remove_multiple=modifiers.get("--force"),
         )
 
@@ -189,15 +187,17 @@ def get_restart(resource_restart):
         """
         Options:
           * --force - allow multiple
-          * -f - CIB file
           * --name - name of a booth instance
         """
-        modifiers.ensure_only_supported("--force", "-f", "--name")
+        modifiers.ensure_only_supported("--force", "--name")
         if arg_list:
             raise CmdLineInputError()
 
         lib.booth.restart(
-            resource_restart,
+            lambda resource_id_list: resource_restart(
+                lib, resource_id_list, modifiers.get_subset("--force")
+            ),
+            instance_name=modifiers.get("--name"),
             allow_multiple=modifiers.get("--force"),
         )
 
@@ -218,7 +218,10 @@ def sync(lib, arg_list, modifiers):
     )
     if arg_list:
         raise CmdLineInputError()
-    lib.booth.config_sync(skip_offline_nodes=modifiers.get("--skip-offline"))
+    lib.booth.config_sync(
+        instance_name=modifiers.get("--name"),
+        skip_offline_nodes=modifiers.get("--skip-offline")
+    )
 
 
 def enable(lib, arg_list, modifiers):
@@ -229,7 +232,7 @@ def enable(lib, arg_list, modifiers):
     modifiers.ensure_only_supported("--name")
     if arg_list:
         raise CmdLineInputError()
-    lib.booth.enable()
+    lib.booth.enable_booth(instance_name=modifiers.get("--name"))
 
 
 def disable(lib, arg_list, modifiers):
@@ -240,7 +243,7 @@ def disable(lib, arg_list, modifiers):
     modifiers.ensure_only_supported("--name")
     if arg_list:
         raise CmdLineInputError()
-    lib.booth.disable()
+    lib.booth.disable_booth(instance_name=modifiers.get("--name"))
 
 
 def start(lib, arg_list, modifiers):
@@ -251,7 +254,7 @@ def start(lib, arg_list, modifiers):
     modifiers.ensure_only_supported("--name")
     if arg_list:
         raise CmdLineInputError()
-    lib.booth.start()
+    lib.booth.start_booth(instance_name=modifiers.get("--name"))
 
 
 def stop(lib, arg_list, modifiers):
@@ -262,7 +265,7 @@ def stop(lib, arg_list, modifiers):
     modifiers.ensure_only_supported("--name")
     if arg_list:
         raise CmdLineInputError()
-    lib.booth.stop()
+    lib.booth.stop_booth(instance_name=modifiers.get("--name"))
 
 
 def pull(lib, arg_list, modifiers):
@@ -274,7 +277,10 @@ def pull(lib, arg_list, modifiers):
     modifiers.ensure_only_supported("--name", "--request-timeout")
     if len(arg_list) != 1:
         raise CmdLineInputError()
-    lib.booth.pull(arg_list[0])
+    lib.booth.pull_config(
+        arg_list[0],
+        instance_name=modifiers.get("--name"),
+    )
 
 
 def status(lib, arg_list, modifiers):
@@ -285,7 +291,7 @@ def status(lib, arg_list, modifiers):
     modifiers.ensure_only_supported("--name")
     if arg_list:
         raise CmdLineInputError()
-    booth_status = lib.booth.status()
+    booth_status = lib.booth.get_status(instance_name=modifiers.get("--name"))
     if booth_status.get("ticket"):
         print("TICKETS:")
         print(booth_status["ticket"])
