@@ -1,5 +1,9 @@
-from pcs.cli.common.console_report import format_list
-from pcs.cli.common.errors import CmdLineInputError
+from pcs.cli.common.console_report import format_plural
+from pcs.cli.common.errors import (
+    CmdLineInputError,
+    HINT_SYNTAX_CHANGE,
+)
+from pcs.common.tools import format_list
 
 
 ARG_TYPE_DELIMITER = "%"
@@ -9,16 +13,17 @@ ARG_TYPE_DELIMITER = "%"
 PCS_SHORT_OPTIONS = "hf:p:u:"
 PCS_LONG_OPTIONS = [
     "debug", "version", "help", "fullhelp",
-    "force", "skip-offline", "interactive", "autodelete",
+    "force", "skip-offline", "interactive", "autodelete", "simulate",
     "all", "full", "local", "wait", "config",
     "start", "enable", "disabled", "off", "request-timeout=",
+    "safe", "no-strict",
     "pacemaker", "corosync",
     "no-default-ops", "defaults", "nodesc",
     "master", "name=", "group=", "node=",
     "from=", "to=", "after=", "before=",
     "corosync_conf=",
     "booth-conf=", "booth-key=", "no-watchdog-validation", "no-keys-sync",
-    #in pcs status - do not display resorce status on inactive node
+    #in pcs status - do not display resource status on inactive node
     "hide-inactive",
     # pcs resource (un)manage - enable or disable monitor operations
     "monitor",
@@ -66,7 +71,7 @@ def split_list_by_any_keywords(arg_list, keyword_label):
             groups[current_keyword] = []
     return groups
 
-def split_option(arg):
+def split_option(arg, allow_empty_value=True):
     """
     Get (key, value) from a key=value commandline argument.
 
@@ -74,6 +79,8 @@ def split_option(arg):
     CmdLineInputError if the argument cannot be splitted.
 
     string arg -- commandline argument
+    allow_empty_value -- if True, empty value is allowed. Otherwise,
+        CmdLineInputError exception is raised
 
     Commandline options: no options
     """
@@ -81,7 +88,10 @@ def split_option(arg):
         raise CmdLineInputError("missing value of '{0}' option".format(arg))
     if arg.startswith("="):
         raise CmdLineInputError("missing key in '{0}' option".format(arg))
-    return arg.split("=", 1)
+    key, value = arg.split("=", 1)
+    if not (value or allow_empty_value):
+        raise CmdLineInputError("value of '{0}' option is empty".format(key))
+    return key, value
 
 def prepare_options(cmdline_args, allowed_repeatable_options=()):
     """
@@ -366,9 +376,12 @@ class InputModifiers():
             "--no-default-ops": "--no-default-ops" in options,
             "--nodesc": "--nodesc" in options,
             "--no-keys-sync": "--no-keys-sync" in options,
+            "--no-strict": "--no-strict" in options,
             "--no-watchdog-validation": "--no-watchdog-validation" in options,
             "--off": "--off" in options,
             "--pacemaker": "--pacemaker" in options,
+            "--safe": "--safe" in options,
+            "--simulate": "--simulate" in options,
             "--skip-offline": "--skip-offline" in options,
             "--start": "--start" in options,
             # string values
@@ -389,20 +402,53 @@ class InputModifiers():
             "-u": options.get("-u", None),
         })
 
-    def get_subset(self, *options):
-        return InputModifiers(
-            {opt: self.get(opt) for opt in options if self.is_specified(opt)}
-        )
+    def get_subset(self, *options, **custom_options):
+        opt_dict = {
+            opt: self.get(opt) for opt in options if self.is_specified(opt)
+        }
+        opt_dict.update(custom_options)
+        return InputModifiers(opt_dict)
 
-    def ensure_only_supported(self, *supported_options):
+    def ensure_only_supported(
+        self, *supported_options, hint_syntax_changed=False
+    ):
         unsupported_options = (
             # --debug is supported in all commands
             self._defined_options - set(supported_options) - set(["--debug"])
         )
         if unsupported_options:
+            pluralize = lambda word: format_plural(unsupported_options, word)
             raise CmdLineInputError(
-                "Specified options {} are not supported in this command".format(
-                    format_list(list(unsupported_options))
+                "Specified {option} {option_list} {_is} not supported in this "
+                "command".format(
+                    option=pluralize("option"),
+                    option_list=format_list(sorted(unsupported_options)),
+                    _is=pluralize("is"),
+                ),
+                # Print error messages which point users to the changes section
+                # in pcs manpage.
+                # To be removed in the next significant version.
+                hint=(HINT_SYNTAX_CHANGE if hint_syntax_changed else None)
+            )
+
+    def ensure_not_mutually_exclusive(self, *mutually_exclusive):
+        options_to_report = self._defined_options & set(mutually_exclusive)
+        if len(options_to_report) > 1:
+            raise CmdLineInputError(
+                "Only one of {} can be used".format(
+                    format_list(sorted(options_to_report))
+                )
+            )
+
+    def ensure_not_incompatible(self, checked, incompatible):
+        if not checked in self._defined_options:
+            return
+        disallowed = self._defined_options & set(incompatible)
+        if disallowed:
+            raise CmdLineInputError(
+                "'{}' cannot be used with {}".format(
+                    checked,
+                    format_list(sorted(disallowed))
                 )
             )
 
