@@ -1,10 +1,3 @@
-from __future__ import (
-    absolute_import,
-    division,
-    print_function,
-    unicode_literals,
-)
-
 from pcs.cli.common import parse_args
 from pcs.cli.common.errors import CmdLineInputError
 
