@@ -1,10 +1,3 @@
-from __future__ import (
-    absolute_import,
-    division,
-    print_function,
-    unicode_literals,
-)
-
 from pcs.cli.constraint.console_report import (
     constraint_plain as constraint_plain_default,
     constraint_with_sets,
@@ -43,12 +36,14 @@ def constraint_plain(constraint_type, options_dict, with_id=False):
 
     return type_report_map[constraint_type](options_dict, with_id)
 
-#Each value (callable taking report_item.info) returns string template.
-#Optionaly the template can contain placehodler {force} for next processing.
-#Placeholder {force} will be appended if is necessary and if is not presset
+#Each value (a callable taking report_item.info) returns a message.
+#Force text will be appended if necessary.
+#If it is necessary to put the force text inside the string then the callable
+#must take the force_text parameter.
 CODE_TO_MESSAGE_BUILDER_MAP = {
-    codes.DUPLICATE_CONSTRAINTS_EXIST: lambda info:
-        "duplicate constraint already exists{force}\n" + "\n".join([
+    codes.DUPLICATE_CONSTRAINTS_EXIST: lambda info, force_text:
+        "duplicate constraint already exists{0}\n".format(force_text)
+        + "\n".join([
             "  " + constraint(info["constraint_type"], constraint_info)
             for constraint_info in info["constraint_info_list"]
         ])
@@ -56,10 +51,9 @@ CODE_TO_MESSAGE_BUILDER_MAP = {
 
     codes.RESOURCE_FOR_CONSTRAINT_IS_MULTIINSTANCE: lambda info:
         (
-            "{resource_id} is a {mode} resource, you should use the"
+            "{resource_id} is a {parent_type} resource, you should use the"
             " {parent_type} id: {parent_id} when adding constraints"
         ).format(
-            mode="master/slave" if info["parent_type"] == "master" else "clone",
             **info
         )
     ,
