@@ -1,9 +1,3 @@
-from __future__ import (
-    absolute_import,
-    division,
-    print_function,
-)
-
 from pcs.cli.common.errors import CmdLineInputError
 from pcs.cli.constraint import command
 from pcs.cli.constraint_ticket import parse_args, console_report
@@ -16,8 +10,13 @@ def create_with_set(lib, argv, modifiers):
     list argv see usage for "constraint colocation set"
     dict like object modifiers can contain
         "force" allows resource in clone/master and constraint duplicity
-        "autocorrect" allows correct resource to its clone/master parent
+
+    Options:
+      * --force - allow resource inside clone (or master), allow duplicate
+        element
+      * -f - CIB file
     """
+    modifiers.ensure_only_supported("--force", "-f")
     command.create_with_set(
         lib.constraint_ticket.set,
         argv,
@@ -31,8 +30,13 @@ def add(lib, argv, modifiers):
     list argv see usage for "constraint colocation add"
     dict like object modifiers can contain
         "force" allows resource in clone/master and constraint duplicity
-        "autocorrect" allows correct resource to its clone/master parent
+
+    Options:
+      * --force - allow resource inside clone (or master), allow duplicate
+        element
+      * -f - CIB file
     """
+    modifiers.ensure_only_supported("--force", "-f")
     ticket, resource_id, resource_role, options = parse_args.parse_add(argv)
     if "rsc-role" in options:
         raise CmdLineInputError(
@@ -47,12 +51,16 @@ def add(lib, argv, modifiers):
         ticket,
         resource_id,
         options,
-        autocorrection_allowed=modifiers["autocorrect"],
-        resource_in_clone_alowed=modifiers["force"],
-        duplication_alowed=modifiers["force"],
+        resource_in_clone_alowed=modifiers.get("--force"),
+        duplication_alowed=modifiers.get("--force"),
     )
 
 def remove(lib, argv, modifiers):
+    """
+    Options:
+      * -f - CIB file
+    """
+    modifiers.ensure_only_supported("-f")
     if len(argv) != 2:
         raise CmdLineInputError()
     ticket, resource_id = argv
@@ -65,7 +73,14 @@ def show(lib, argv, modifiers):
     object lib exposes library
     list argv see usage for "constraint colocation show"
     dict like object modifiers can contain "full"
+
+    Options:
+      * --full - print more details
+      * -f - CIB file
     """
+    modifiers.ensure_only_supported("-f", "--full")
+    if argv:
+        raise CmdLineInputError()
     print("\n".join(command.show(
         "Ticket Constraints:",
         lib.constraint_ticket.show,
