@@ -1,10 +1,5 @@
-from __future__ import (
-    absolute_import,
-    division,
-    print_function,
-)
 from pcs.cli.common.parse_args import group_by_keywords, prepare_options
-from pcs.cli.common.errors import CmdLineInputError
+from pcs.cli.common.errors import CmdLineInputError, HINT_SYNTAX_CHANGE
 
 
 def parse_create_simple(arg_list):
@@ -29,33 +24,42 @@ def parse_create_simple(arg_list):
 def parse_create(arg_list):
     groups = group_by_keywords(
         arg_list,
-        set(["op", "meta", "clone", "master", "bundle"]),
+        set(["op", "meta", "clone", "promotable", "bundle"]),
         implicit_first_group_key="options",
         group_repeated_keywords=["op"],
         only_found_keywords=True,
     )
 
-    parts = {
-        "meta":  prepare_options(groups.get("meta", [])),
-        "options":  prepare_options(groups.get("options", [])),
-        "op": [
-            prepare_options(op)
-            for op in build_operations(groups.get("op", []))
-        ],
-    }
+    try:
+        parts = {
+            "meta":  prepare_options(groups.get("meta", [])),
+            "options":  prepare_options(groups.get("options", [])),
+            "op": [
+                prepare_options(op)
+                for op in build_operations(groups.get("op", []))
+            ],
+        }
 
-    if "clone" in groups:
-        parts["clone"] = prepare_options(groups["clone"])
-
-    if "master" in groups:
-        parts["master"] = prepare_options(groups["master"])
-
-    if "bundle" in groups:
-        parts["bundle"] = groups["bundle"]
+        if "clone" in groups:
+            parts["clone"] = prepare_options(groups["clone"])
+        if "promotable" in groups:
+            parts["promotable"] = prepare_options(groups["promotable"])
+        if "bundle" in groups:
+            parts["bundle"] = groups["bundle"]
+    except CmdLineInputError as e:
+        # Print error messages which point users to the changes section in pcs
+        # manpage.
+        # To be removed in the next significant version.
+        if e.message == "missing value of 'master' option":
+            raise CmdLineInputError(message=e.message, hint=HINT_SYNTAX_CHANGE)
+        raise e
 
     return parts
 
 def _parse_bundle_groups(arg_list):
+    """
+    Commandline options: no options
+    """
     repeatable_keyword_list = ["port-map", "storage-map"]
     keyword_list = ["meta", "container", "network"] + repeatable_keyword_list
     groups = group_by_keywords(
@@ -69,18 +73,21 @@ def _parse_bundle_groups(arg_list):
             continue
         if keyword in repeatable_keyword_list:
             for repeated_section in groups[keyword]:
-                if len(repeated_section) == 0:
+                if not repeated_section:
                     raise CmdLineInputError(
                         "No {0} options specified".format(keyword)
                     )
         else:
-            if len(groups[keyword]) == 0:
+            if not groups[keyword]:
                 raise CmdLineInputError(
                     "No {0} options specified".format(keyword)
                 )
     return groups
 
 def parse_bundle_create_options(arg_list):
+    """
+    Commandline options: no options
+    """
     groups = _parse_bundle_groups(arg_list)
     container_options = groups.get("container", [])
     container_type = ""
@@ -102,28 +109,58 @@ def parse_bundle_create_options(arg_list):
     }
     return parts
 
+def parse_bundle_reset_options(arg_list):
+    """
+    Commandline options: no options
+    """
+    groups = _parse_bundle_groups(arg_list)
+    container_options = groups.get("container", [])
+    parts = {
+        "container": prepare_options(container_options),
+        "network": prepare_options(groups.get("network", [])),
+        "port_map": [
+            prepare_options(port_map)
+            for port_map in groups.get("port-map", [])
+        ],
+        "storage_map": [
+            prepare_options(storage_map)
+            for storage_map in groups.get("storage-map", [])
+        ],
+        "meta": prepare_options(groups.get("meta", []))
+    }
+    return parts
+
 def _split_bundle_map_update_op_and_options(
     map_arg_list, result_parts, map_name
 ):
+    """
+    Commandline options: no options
+    """
     if len(map_arg_list) < 2:
         raise _bundle_map_update_not_valid(map_name)
     op, options = map_arg_list[0], map_arg_list[1:]
     if op == "add":
-        result_parts[op].append(prepare_options(options))
-    elif op == "remove":
-        result_parts[op].extend(options)
+        result_parts["add"].append(prepare_options(options))
+    elif op in {"delete", "remove"}:
+        result_parts["remove"].extend(options)
     else:
         raise _bundle_map_update_not_valid(map_name)
 
 def _bundle_map_update_not_valid(map_name):
+    """
+    Commandline options: no options
+    """
     return CmdLineInputError(
         (
             "When using '{map}' you must specify either 'add' and options or "
-            "'remove' and id(s)"
+            "either of 'delete' or 'remove' and id(s)"
         ).format(map=map_name)
     )
 
 def parse_bundle_update_options(arg_list):
+    """
+    Commandline options: no options
+    """
     groups = _parse_bundle_groups(arg_list)
     port_map = {"add": [], "remove": []}
     for map_group in groups.get("port-map", []):
