@@ -11,11 +11,7 @@ import time
 import xml.dom.minidom
 
 from pcs import (
-    pcsd,
-    resource,
     settings,
-    status,
-    usage,
     utils,
 )
 from pcs.utils import parallel_for_nodes
@@ -23,10 +19,10 @@ from pcs.cli.common import parse_args
 from pcs.cli.common.errors import (
     CmdLineInputError,
     ERR_NODE_LIST_AND_ALL_MUTUALLY_EXCLUSIVE,
+    HINT_SYNTAX_CHANGE,
+    msg_command_replaced,
 )
 from pcs.cli.common.reports import process_library_reports, build_report_message
-from pcs.cli.common.routing import create_router
-import pcs.cli.cluster.command as cluster_command
 from pcs.common import report_codes
 from pcs.common.node_communicator import (
     HostNotFound,
@@ -59,105 +55,11 @@ from pcs.lib.errors import (
 )
 from pcs.lib.external import disable_service
 from pcs.lib.env import MIN_FEATURE_SET_VERSION_FOR_DIFF
-from pcs.lib.env_tools import get_existing_nodes_names
+from pcs.lib.node import get_existing_nodes_names
 import pcs.lib.pacemaker.live as lib_pacemaker
 
 # pylint: disable=too-many-branches, too-many-statements
 
-def cluster_cmd(lib, argv, modifiers):
-    # pylint: disable=superfluous-parens
-    if not argv:
-        usage.cluster()
-        exit(1)
-
-    sub_cmd = argv.pop(0)
-    try:
-        if (sub_cmd == "help"):
-            usage.cluster([" ".join(argv)] if argv else [])
-        elif (sub_cmd == "setup"):
-            cluster_setup(lib, argv, modifiers)
-        elif (sub_cmd == "sync"):
-            create_router(
-                dict(
-                    corosync=sync_nodes,
-                ),
-                ["cluster", "sync"],
-                default_cmd="corosync",
-            )(lib, argv, modifiers)
-        elif (sub_cmd == "status"):
-            status.cluster_status(lib, argv, modifiers)
-        elif (sub_cmd == "pcsd-status"):
-            status.cluster_pcsd_status(lib, argv, modifiers)
-        elif (sub_cmd == "certkey"):
-            pcsd.pcsd_certkey(lib, argv, modifiers)
-        elif (sub_cmd == "auth"):
-            cluster_auth_cmd(lib, argv, modifiers)
-        elif (sub_cmd == "start"):
-            cluster_start_cmd(lib, argv, modifiers)
-        elif (sub_cmd == "stop"):
-            cluster_stop_cmd(lib, argv, modifiers)
-        elif (sub_cmd == "kill"):
-            kill_cluster(lib, argv, modifiers)
-        elif (sub_cmd == "enable"):
-            cluster_enable_cmd(lib, argv, modifiers)
-        elif (sub_cmd == "disable"):
-            cluster_disable_cmd(lib, argv, modifiers)
-        elif (sub_cmd == "cib"):
-            get_cib(lib, argv, modifiers)
-        elif (sub_cmd == "cib-push"):
-            cluster_push(lib, argv, modifiers)
-        elif (sub_cmd == "cib-upgrade"):
-            cluster_cib_upgrade_cmd(lib, argv, modifiers)
-        elif (sub_cmd == "edit"):
-            cluster_edit(lib, argv, modifiers)
-        elif (sub_cmd == "node"):
-            node_command_map = {
-                "add": node_add,
-                "add-guest": cluster_command.node_add_guest,
-                "add-outside": node_add_outside_cluster,
-                "add-remote": cluster_command.node_add_remote,
-                "clear": cluster_command.node_clear,
-                "delete": node_remove,
-                "delete-guest": cluster_command.node_remove_guest,
-                "delete-remote": cluster_command.create_node_remove_remote(
-                    resource.resource_remove
-                ),
-                "remove": node_remove,
-                "remove-guest": cluster_command.node_remove_guest,
-                "remove-remote": cluster_command.create_node_remove_remote(
-                    resource.resource_remove
-                ),
-            }
-            if argv and argv[0] in node_command_map:
-                try:
-                    node_command_map[argv[0]](lib, argv[1:], modifiers)
-                except CmdLineInputError as e:
-                    utils.exit_on_cmdline_input_errror(
-                        e, "cluster", "node " + argv[0]
-                    )
-            else:
-                raise CmdLineInputError()
-        elif (sub_cmd == "uidgid"):
-            cluster_uidgid(lib, argv, modifiers)
-        elif (sub_cmd == "corosync"):
-            cluster_get_corosync_conf(lib, argv, modifiers)
-        elif (sub_cmd == "reload"):
-            cluster_reload(lib, argv, modifiers)
-        elif (sub_cmd == "destroy"):
-            cluster_destroy(lib, argv, modifiers)
-        elif (sub_cmd == "verify"):
-            cluster_verify(lib, argv, modifiers)
-        elif (sub_cmd == "report"):
-            cluster_report(lib, argv, modifiers)
-        elif (sub_cmd == "remove_nodes_from_cib"):
-            remove_nodes_from_cib(lib, argv, modifiers)
-        else:
-            sub_cmd = ""
-            raise CmdLineInputError()
-    except LibraryError as e:
-        process_library_reports(e.args)
-    except CmdLineInputError as e:
-        utils.exit_on_cmdline_input_errror(e, "cluster", sub_cmd)
 
 def cluster_cib_upgrade_cmd(lib, argv, modifiers):
     """
@@ -254,8 +156,16 @@ def sync_nodes(lib, argv, modifiers):
     modifiers.ensure_only_supported("--request-timeout")
     if argv:
         raise CmdLineInputError()
+
     config = utils.getCorosyncConf()
-    nodes = utils.get_corosync_conf_facade(conf_text=config).get_nodes_names()
+    nodes, report_list = get_existing_nodes_names(
+        utils.get_corosync_conf_facade(conf_text=config)
+    )
+    if not nodes:
+        report_list.append(reports.corosync_config_no_nodes_defined())
+    if report_list:
+        process_library_reports(report_list)
+
     for node in nodes:
         utils.setCorosyncConfig(node, config)
 
@@ -305,9 +215,15 @@ def start_cluster_all():
         wait_timeout = utils.validate_wait_get_timeout(False)
         wait = True
 
-    all_nodes = utils.get_corosync_conf_facade().get_nodes_names()
-    start_cluster_nodes(all_nodes)
+    all_nodes, report_list = get_existing_nodes_names(
+        utils.get_corosync_conf_facade()
+    )
+    if not all_nodes:
+        report_list.append(reports.corosync_config_no_nodes_defined())
+    if report_list:
+        process_library_reports(report_list)
 
+    start_cluster_nodes(all_nodes)
     if wait:
         wait_for_nodes_started(all_nodes, wait_timeout)
 
@@ -327,6 +243,7 @@ def start_cluster_nodes(nodes):
     timeout = int(
         settings.default_request_timeout * math.ceil(len(nodes) / 8.0)
     )
+    utils.read_known_hosts_file() # cache known hosts
     node_errors = parallel_for_nodes(
         utils.startCluster, nodes, quiet=True, timeout=timeout
     )
@@ -404,6 +321,7 @@ def wait_for_nodes_started(node_list, timeout=None):
         else:
             print(output)
     else:
+        utils.read_known_hosts_file() # cache known hosts
         node_errors = parallel_for_nodes(
             wait_for_remote_node_started, node_list, stop_at, interval
         )
@@ -416,7 +334,15 @@ def stop_cluster_all():
       * --force - no error when possible quorum loss
       * --request-timeout - timeout for HTTP requests
     """
-    stop_cluster_nodes(utils.get_corosync_conf_facade().get_nodes_names())
+    all_nodes, report_list = get_existing_nodes_names(
+        utils.get_corosync_conf_facade()
+    )
+    if not all_nodes:
+        report_list.append(reports.corosync_config_no_nodes_defined())
+    if report_list:
+        process_library_reports(report_list)
+
+    stop_cluster_nodes(all_nodes)
 
 def stop_cluster_nodes(nodes):
     """
@@ -424,14 +350,19 @@ def stop_cluster_nodes(nodes):
       * --force - no error when possible quorum loss
       * --request-timeout - timeout for HTTP requests
     """
-    all_nodes = utils.get_corosync_conf_facade().get_nodes_names()
+    all_nodes, report_list = get_existing_nodes_names(
+        utils.get_corosync_conf_facade()
+    )
     unknown_nodes = set(nodes) - set(all_nodes)
     if unknown_nodes:
+        if report_list:
+            process_library_reports(report_list)
         utils.err(
             "nodes '%s' do not appear to exist in configuration"
-            % "', '".join(unknown_nodes)
+            % "', '".join(sorted(unknown_nodes))
         )
 
+    utils.read_known_hosts_file() # cache known hosts
     stopping_all = set(nodes) >= set(all_nodes)
     if "--force" not in utils.pcs_options and not stopping_all:
         error_list = []
@@ -534,14 +465,30 @@ def enable_cluster_all():
     Commandline options:
       * --request-timeout - timeout for HTTP requests
     """
-    enable_cluster_nodes(utils.get_corosync_conf_facade().get_nodes_names())
+    all_nodes, report_list = get_existing_nodes_names(
+        utils.get_corosync_conf_facade()
+    )
+    if not all_nodes:
+        report_list.append(reports.corosync_config_no_nodes_defined())
+    if report_list:
+        process_library_reports(report_list)
+
+    enable_cluster_nodes(all_nodes)
 
 def disable_cluster_all():
     """
     Commandline options:
       * --request-timeout - timeout for HTTP requests
     """
-    disable_cluster_nodes(utils.get_corosync_conf_facade().get_nodes_names())
+    all_nodes, report_list = get_existing_nodes_names(
+        utils.get_corosync_conf_facade()
+    )
+    if not all_nodes:
+        report_list.append(reports.corosync_config_no_nodes_defined())
+    if report_list:
+        process_library_reports(report_list)
+
+    disable_cluster_nodes(all_nodes)
 
 def enable_cluster_nodes(nodes):
     """
@@ -567,6 +514,7 @@ def destroy_cluster(argv):
       * --request-timeout - timeout for HTTP requests
     """
     if argv:
+        utils.read_known_hosts_file() # cache known hosts
         # stop pacemaker and resources while cluster is still quorate
         nodes = argv
         node_errors = parallel_for_nodes(
@@ -601,9 +549,14 @@ def stop_cluster(argv):
         return
 
     if "--force" not in utils.pcs_options:
+        # corosync 3.0.1 and older:
+        # - retval is 0 on success if a node is not in a partition with quorum
+        # - retval is 1 on error OR on success if a node has quorum
+        # corosync 3.0.2 and newer:
+        # - retval is 0 on success if a node has quorum
+        # - retval is 1 on error
+        # - retval is 2 on success if a node is not in a partition with quorum
         output, dummy_retval = utils.run(["corosync-quorumtool", "-p", "-s"])
-        # retval is 0 on success if node is not in partition with quorum
-        # retval is 1 on error OR on success if node has quorum
         try:
             if (
                 corosync_live.QuorumStatus.from_string(output)
@@ -690,7 +643,7 @@ def kill_local_cluster_services():
         "corosync-qdevice",
         "corosync",
     ]
-    return utils.run(["killall", "-9"] + all_cluster_daemons)
+    return utils.run(["/usr/bin/killall", "-9"] + all_cluster_daemons)
 
 def cluster_push(lib, argv, modifiers):
     """
@@ -793,8 +746,8 @@ def cluster_push(lib, argv, modifiers):
 
         runner = utils.cmd_runner()
         command = [
-            "crm_diff", "--original", diff_against, "--new", filename,
-            "--no-version"
+            os.path.join(settings.pacemaker_binaries, "crm_diff"),
+            "--original", diff_against, "--new", filename, "--no-version"
         ]
         patch, stderr, retval = runner.run(command)
         #  0 (CRM_EX_OK) - success with no difference
@@ -809,7 +762,10 @@ def cluster_push(lib, argv, modifiers):
             )
             sys.exit(0)
 
-        command = ["cibadmin", "--patch", "--xml-pipe"]
+        command = [
+            os.path.join(settings.pacemaker_binaries, "cibadmin"),
+            "--patch", "--xml-pipe"
+        ]
         output, stderr, retval = runner.run(command, patch)
         if retval != 0:
             utils.err("unable to push cib\n" + stderr + output)
@@ -855,16 +811,15 @@ def cluster_edit(lib, argv, modifiers):
         for arg in argv:
             if "=" not in arg:
                 raise CmdLineInputError()
-            else:
-                arg_name, arg_value = arg.split("=", 1)
-                if arg_name == "scope" and not modifiers.get("--config"):
-                    if not utils.is_valid_cib_scope(arg_value):
-                        utils.err("invalid CIB scope '%s'" % arg_value)
-                    else:
-                        scope_arg = arg
-                        scope = arg_value
+            arg_name, arg_value = arg.split("=", 1)
+            if arg_name == "scope" and not modifiers.get("--config"):
+                if not utils.is_valid_cib_scope(arg_value):
+                    utils.err("invalid CIB scope '%s'" % arg_value)
                 else:
-                    raise CmdLineInputError()
+                    scope_arg = arg
+                    scope = arg_value
+            else:
+                raise CmdLineInputError()
         if modifiers.get("--config"):
             scope = "configuration"
             # Leave scope_arg empty as cluster_push will pick up a --config
@@ -1128,7 +1083,17 @@ def cluster_uidgid(lib, argv, modifiers, silent_list=False):
                 )
 
     else:
-        raise CmdLineInputError()
+        # The hint is defined to print error messages which point users to the
+        # changes section in pcs manpage.
+        # To be removed in the next significant version.
+        raise CmdLineInputError(hint=(
+            msg_command_replaced(
+                "pcs cluster uidgid delete",
+                "pcs cluster uidgid remove",
+            )
+            if command == "rm"
+            else None
+        ))
 
 def cluster_get_corosync_conf(lib, argv, modifiers):
     """
@@ -1180,7 +1145,7 @@ def cluster_destroy(lib, argv, modifiers):
     if argv:
         raise CmdLineInputError()
     if modifiers.get("--all"):
-        # destroy remote and guest nodes
+        # load data
         cib = None
         lib_env = utils.get_lib_env()
         try:
@@ -1190,9 +1155,22 @@ def cluster_destroy(lib, argv, modifiers):
                 "Unable to load CIB to get guest and remote nodes from it, "
                 "those nodes will not be deconfigured."
             )
+        corosync_nodes, report_list = get_existing_nodes_names(
+            utils.get_corosync_conf_facade()
+        )
+        if not corosync_nodes:
+            report_list.append(reports.corosync_config_no_nodes_defined())
+        if report_list:
+            process_library_reports(report_list)
+
+        # destroy remote and guest nodes
         if cib is not None:
             try:
-                all_remote_nodes = get_existing_nodes_names(cib=cib)
+                all_remote_nodes, report_list = get_existing_nodes_names(
+                    cib=cib
+                )
+                if report_list:
+                    process_library_reports(report_list)
                 if all_remote_nodes:
                     _destroy_pcmk_remote_env(
                         lib_env,
@@ -1201,10 +1179,10 @@ def cluster_destroy(lib, argv, modifiers):
                         allow_fails=True
                     )
             except LibraryError as e:
-                utils.process_library_reports(e.args)
+                process_library_reports(e.args)
 
         # destroy full-stack nodes
-        destroy_cluster(utils.get_corosync_conf_facade().get_nodes_names())
+        destroy_cluster(corosync_nodes)
     else:
         print("Shutting down pacemaker/corosync services...")
         for service in ["pacemaker", "corosync-qdevice", "corosync"]:
@@ -1227,7 +1205,7 @@ def cluster_destroy(lib, argv, modifiers):
 
         print("Removing all cluster configuration files...")
         dummy_output, dummy_retval = utils.run([
-            "rm", "-f",
+            "/bin/rm", "-f",
             settings.corosync_conf_file,
             settings.corosync_authkey_file,
             settings.pacemaker_authkey_file,
@@ -1243,8 +1221,8 @@ def cluster_destroy(lib, argv, modifiers):
         ]
         for name in state_files:
             dummy_output, dummy_retval = utils.run([
-                "find", "/var/lib/pacemaker", "-name", name,
-                "-exec", "rm", "-f", "{}", ";"
+                "/usr/bin/find", "/var/lib/pacemaker", "-name", name,
+                "-exec", "/bin/rm", "-f", "{}", ";"
             ])
         try:
             qdevice_net.client_destroy()
@@ -1392,7 +1370,10 @@ def cluster_auth_cmd(lib, argv, modifiers):
         "--corosync_conf", "--request-timeout", "-u", "-p"
     )
     if argv:
-        raise CmdLineInputError()
+        # The hint is defined to print error messages which point users to the
+        # changes section in pcs manpage.
+        # To be removed in the next significant version.
+        raise CmdLineInputError(hint=HINT_SYNTAX_CHANGE)
     lib_env = utils.get_lib_env()
     target_factory = lib_env.get_node_target_factory()
     cluster_node_list = lib_env.get_corosync_conf().get_nodes()
@@ -1431,7 +1412,7 @@ def cluster_auth_cmd(lib, argv, modifiers):
         for node_name in not_authorized_node_name_list:
             for node in cluster_node_list:
                 if node.name == node_name:
-                    if node.addrs_plain:
+                    if node.addrs_plain():
                         not_auth_node_list.append(node)
                     else:
                         print(
@@ -1444,7 +1425,7 @@ def cluster_auth_cmd(lib, argv, modifiers):
                 username=username,
                 password=password,
                 dest_list=[dict(
-                    addr=node.addrs_plain[0],
+                    addr=node.addrs_plain()[0],
                     port=settings.pcsd_default_port,
                 )],
             ) for node in not_auth_node_list
@@ -1527,7 +1508,11 @@ def cluster_setup(lib, argv, modifiers):
         corosync and pacemaker authkeys
     """
     modifiers.ensure_only_supported(
-        "--wait", "--start", "--enable", "--force", "--no-keys-sync"
+        "--wait", "--start", "--enable", "--force", "--no-keys-sync",
+        # The hint is defined to print error messages which point users to the
+        # changes section in pcs manpage.
+        # To be removed in the next significant version.
+        hint_syntax_changed=modifiers.is_specified("--name")
     )
     # pylint: disable=invalid-name
     if len(argv) < 2:
@@ -1641,3 +1626,87 @@ def remove_nodes_from_cib(lib, argv, modifiers):
     if not argv:
         raise CmdLineInputError("No nodes specified")
     lib.cluster.remove_nodes_from_cib(argv)
+
+def link_add(lib, argv, modifiers):
+    """
+    Options:
+      * --force - treat validation issues and not resolvable addresses as
+        warnings instead of errors
+      * --skip-offline - skip unreachable nodes
+      * --request-timeout - HTTP request timeout
+    """
+    modifiers.ensure_only_supported(
+        "--force", "--request-timeout", "--skip-offline"
+    )
+    if not argv:
+        raise CmdLineInputError()
+
+    force_flags = []
+    if modifiers.get("--force"):
+        force_flags.append(report_codes.FORCE)
+    if modifiers.get("--skip-offline"):
+        force_flags.append(report_codes.SKIP_OFFLINE_NODES)
+
+    parsed = parse_args.group_by_keywords(
+        argv,
+        {"options"},
+        implicit_first_group_key="nodes",
+        keyword_repeat_allowed=False
+    )
+
+    lib.cluster.add_link(
+        parse_args.prepare_options(parsed["nodes"]),
+        parse_args.prepare_options(parsed["options"]),
+        force_flags=force_flags,
+    )
+
+def link_remove(lib, argv, modifiers):
+    """
+    Options:
+      * --skip-offline - skip unreachable nodes
+      * --request-timeout - HTTP request timeout
+    """
+    modifiers.ensure_only_supported("--request-timeout", "--skip-offline")
+    if not argv:
+        raise CmdLineInputError()
+
+    force_flags = []
+    if modifiers.get("--skip-offline"):
+        force_flags.append(report_codes.SKIP_OFFLINE_NODES)
+
+    lib.cluster.remove_links(argv, force_flags=force_flags)
+
+def link_update(lib, argv, modifiers):
+    """
+    Options:
+      * --force - treat validation issues and not resolvable addresses as
+        warnings instead of errors
+      * --skip-offline - skip unreachable nodes
+      * --request-timeout - HTTP request timeout
+    """
+    modifiers.ensure_only_supported(
+        "--force", "--request-timeout", "--skip-offline"
+    )
+    if len(argv) < 2:
+        raise CmdLineInputError()
+
+    force_flags = []
+    if modifiers.get("--force"):
+        force_flags.append(report_codes.FORCE)
+    if modifiers.get("--skip-offline"):
+        force_flags.append(report_codes.SKIP_OFFLINE_NODES)
+
+    linknumber = argv[0]
+    parsed = parse_args.group_by_keywords(
+        argv[1:],
+        {"options"},
+        implicit_first_group_key="nodes",
+        keyword_repeat_allowed=False
+    )
+
+    lib.cluster.update_link(
+        linknumber,
+        parse_args.prepare_options(parsed["nodes"]),
+        parse_args.prepare_options(parsed["options"]),
+        force_flags=force_flags,
+    )
