@@ -1,6 +1,6 @@
 # pylint: disable=line-too-long
 
-# force cathegories
+# force categories
 FORCE = "FORCE"
 FORCE_ALERT_RECIPIENT_VALUE_NOT_UNIQUE = "FORCE_ALERT_RECIPIENT_VALUE_NOT_UNIQUE"
 FORCE_ALREADY_IN_CLUSTER = "FORCE_ALREADY_IN_CLUSTER"
@@ -35,32 +35,41 @@ BAD_CLUSTER_STATE_FORMAT = 'BAD_CLUSTER_STATE_FORMAT'
 BOOTH_ADDRESS_DUPLICATION = "BOOTH_ADDRESS_DUPLICATION"
 BOOTH_ALREADY_IN_CIB = "BOOTH_ALREADY_IN_CIB"
 BOOTH_CANNOT_DETERMINE_LOCAL_SITE_IP = "BOOTH_CANNOT_DETERMINE_LOCAL_SITE_IP"
-BOOTH_CANNOT_IDENTIFY_KEYFILE = "BOOTH_CANNOT_IDENTIFY_KEYFILE"
 BOOTH_CONFIG_ACCEPTED_BY_NODE = "BOOTH_CONFIG_ACCEPTED_BY_NODE"
 BOOTH_CONFIG_DISTRIBUTION_NODE_ERROR = "BOOTH_CONFIG_DISTRIBUTION_NODE_ERROR"
 BOOTH_CONFIG_DISTRIBUTION_STARTED = "BOOTH_CONFIG_DISTRIBUTION_STARTED"
-BOOTH_CONFIG_FILE_ALREADY_EXISTS = "BOOTH_CONFIG_FILE_ALREADY_EXISTS"
-BOOTH_CONFIG_IO_ERROR = "BOOTH_CONFIG_IO_ERROR"
 BOOTH_CONFIG_IS_USED = "BOOTH_CONFIG_IS_USED"
-BOOTH_CONFIG_READ_ERROR = "BOOTH_CONFIG_READ_ERROR"
 BOOTH_CONFIG_UNEXPECTED_LINES = "BOOTH_CONFIG_UNEXPECTED_LINES"
 BOOTH_DAEMON_STATUS_ERROR = "BOOTH_DAEMON_STATUS_ERROR"
 BOOTH_EVEN_PEERS_NUM = "BOOTH_EVEN_PEERS_NUM"
 BOOTH_FETCHING_CONFIG_FROM_NODE = "BOOTH_FETCHING_CONFIG_FROM_NODE"
-BOOTH_INVALID_CONFIG_NAME = "BOOTH_INVALID_CONFIG_NAME"
 BOOTH_INVALID_NAME = "BOOTH_INVALID_NAME"
 BOOTH_LACK_OF_SITES = "BOOTH_LACK_OF_SITES"
 BOOTH_MULTIPLE_TIMES_IN_CIB = "BOOTH_MULTIPLE_TIMES_IN_CIB"
 BOOTH_NOT_EXISTS_IN_CIB = "BOOTH_NOT_EXISTS_IN_CIB"
 BOOTH_PEERS_STATUS_ERROR = "BOOTH_PEERS_STATUS_ERROR"
-BOOTH_SKIPPING_CONFIG = "BOOTH_SKIPPING_CONFIG"
 BOOTH_TICKET_DOES_NOT_EXIST = "BOOTH_TICKET_DOES_NOT_EXIST"
 BOOTH_TICKET_DUPLICATE = "BOOTH_TICKET_DUPLICATE"
 BOOTH_TICKET_NAME_INVALID = "BOOTH_TICKET_NAME_INVALID"
 BOOTH_TICKET_OPERATION_FAILED = "BOOTH_TICKET_OPERATION_FAILED"
 BOOTH_TICKET_STATUS_ERROR = "BOOTH_TICKET_STATUS_ERROR"
-BOOTH_UNSUPORTED_FILE_LOCATION = "BOOTH_UNSUPORTED_FILE_LOCATION"
+BOOTH_UNSUPPORTED_FILE_LOCATION = "BOOTH_UNSUPPORTED_FILE_LOCATION"
+CANNOT_BAN_RESOURCE_MASTER_RESOURCE_NOT_PROMOTABLE = "CANNOT_BAN_RESOURCE_MASTER_RESOURCE_NOT_PROMOTABLE"
+CANNOT_BAN_RESOURCE_STOPPED_NO_NODE_SPECIFIED = "CANNOT_BAN_RESOURCE_STOPPED_NO_NODE_SPECIFIED"
+CANNOT_GROUP_RESOURCE_ADJACENT_RESOURCE_FOR_NEW_GROUP = "CANNOT_GROUP_RESOURCE_ADJACENT_RESOURCE_FOR_NEW_GROUP"
+CANNOT_GROUP_RESOURCE_ADJACENT_RESOURCE_NOT_IN_GROUP = "CANNOT_GROUP_RESOURCE_ADJACENT_RESOURCE_NOT_IN_GROUP"
+CANNOT_GROUP_RESOURCE_ALREADY_IN_THE_GROUP = "CANNOT_GROUP_RESOURCE_ALREADY_IN_THE_GROUP"
+CANNOT_GROUP_RESOURCE_MORE_THAN_ONCE = "CANNOT_GROUP_RESOURCE_MORE_THAN_ONCE"
+CANNOT_GROUP_RESOURCE_NEXT_TO_ITSELF = "CANNOT_GROUP_RESOURCE_NEXT_TO_ITSELF"
+CANNOT_GROUP_RESOURCE_NO_RESOURCES = "CANNOT_GROUP_RESOURCE_NO_RESOURCES"
+CANNOT_GROUP_RESOURCE_WRONG_TYPE = "CANNOT_GROUP_RESOURCE_WRONG_TYPE"
+CANNOT_MOVE_RESOURCE_BUNDLE = "CANNOT_MOVE_RESOURCE_BUNDLE"
+CANNOT_MOVE_RESOURCE_CLONE = "CANNOT_MOVE_RESOURCE_CLONE"
+CANNOT_MOVE_RESOURCE_MASTER_RESOURCE_NOT_PROMOTABLE = "CANNOT_MOVE_RESOURCE_MASTER_RESOURCE_NOT_PROMOTABLE"
+CANNOT_MOVE_RESOURCE_PROMOTABLE_NOT_MASTER = "CANNOT_MOVE_RESOURCE_PROMOTABLE_NOT_MASTER"
+CANNOT_MOVE_RESOURCE_STOPPED_NO_NODE_SPECIFIED = "CANNOT_MOVE_RESOURCE_STOPPED_NO_NODE_SPECIFIED"
 CANNOT_REMOVE_ALL_CLUSTER_NODES = "CANNOT_REMOVE_ALL_CLUSTER_NODES"
+CANNOT_UNMOVE_UNBAN_RESOURCE_MASTER_RESOURCE_NOT_PROMOTABLE = "CANNOT_UNMOVE_UNBAN_RESOURCE_MASTER_RESOURCE_NOT_PROMOTABLE"
 CIB_ACL_ROLE_IS_ALREADY_ASSIGNED_TO_TARGET = "CIB_ACL_ROLE_IS_ALREADY_ASSIGNED_TO_TARGET"
 CIB_ACL_ROLE_IS_NOT_ASSIGNED_TO_TARGET = "CIB_ACL_ROLE_IS_NOT_ASSIGNED_TO_TARGET"
 CIB_ACL_TARGET_ALREADY_EXISTS = "CIB_ACL_TARGET_ALREADY_EXISTS"
@@ -77,6 +86,7 @@ CIB_LOAD_ERROR_SCOPE_MISSING = "CIB_LOAD_ERROR_SCOPE_MISSING"
 CIB_PUSH_FORCED_FULL_DUE_TO_CRM_FEATURE_SET = "CIB_PUSH_FORCED_FULL_DUE_TO_CRM_FEATURE_SET"
 CIB_PUSH_ERROR = "CIB_PUSH_ERROR"
 CIB_SAVE_TMP_ERROR = "CIB_SAVE_TMP_ERROR"
+CIB_SIMULATE_ERROR = "CIB_SIMULATE_ERROR"
 CIB_UPGRADE_FAILED = "CIB_UPGRADE_FAILED"
 CIB_UPGRADE_FAILED_TO_MINIMAL_REQUIRED_VERSION = "CIB_UPGRADE_FAILED_TO_MINIMAL_REQUIRED_VERSION"
 CIB_UPGRADE_SUCCESSFUL = "CIB_UPGRADE_SUCCESSFUL"
@@ -89,34 +99,44 @@ CLUSTER_SETUP_SUCCESS = "CLUSTER_SETUP_SUCCESS"
 CLUSTER_START_STARTED = "CLUSTER_START_STARTED"
 CLUSTER_START_SUCCESS = "CLUSTER_START_SUCCESS"
 CLUSTER_WILL_BE_DESTROYED = "CLUSTER_WILL_BE_DESTROYED"
+LIVE_ENVIRONMENT_NOT_CONSISTENT = "LIVE_ENVIRONMENT_NOT_CONSISTENT"
 LIVE_ENVIRONMENT_REQUIRED = "LIVE_ENVIRONMENT_REQUIRED"
 LIVE_ENVIRONMENT_REQUIRED_FOR_LOCAL_NODE = "LIVE_ENVIRONMENT_REQUIRED_FOR_LOCAL_NODE"
 COROSYNC_ADDRESS_IP_VERSION_WRONG_FOR_LINK = "COROSYNC_ADDRESS_IP_VERSION_WRONG_FOR_LINK"
 COROSYNC_BAD_NODE_ADDRESSES_COUNT = "COROSYNC_BAD_NODE_ADDRESSES_COUNT"
 COROSYNC_CONFIG_ACCEPTED_BY_NODE = "COROSYNC_CONFIG_ACCEPTED_BY_NODE"
-COROSYNC_CONFIG_DISTRIBUTION_STARTED = "COROSYNC_CONFIG_DISTRIBUTION_STARTED"
+COROSYNC_CONFIG_CANNOT_SAVE_INVALID_NAMES_VALUES = "COROSYNC_CONFIG_CANNOT_SAVE_INVALID_NAMES_VALUES"
 COROSYNC_CONFIG_DISTRIBUTION_NODE_ERROR = "COROSYNC_CONFIG_DISTRIBUTION_NODE_ERROR"
+COROSYNC_CONFIG_DISTRIBUTION_STARTED = "COROSYNC_CONFIG_DISTRIBUTION_STARTED"
+COROSYNC_CONFIG_MISSING_NAMES_OF_NODES = "COROSYNC_CONFIG_MISSING_NAMES_OF_NODES"
+COROSYNC_CONFIG_NO_NODES_DEFINED = "COROSYNC_CONFIG_NO_NODES_DEFINED"
 COROSYNC_CONFIG_RELOADED = "COROSYNC_CONFIG_RELOADED"
 COROSYNC_CONFIG_RELOAD_ERROR = "COROSYNC_CONFIG_RELOAD_ERROR"
 COROSYNC_CONFIG_RELOAD_NOT_POSSIBLE = "COROSYNC_CONFIG_RELOAD_NOT_POSSIBLE"
 COROSYNC_IP_VERSION_MISMATCH_IN_LINKS = "COROSYNC_IP_VERSION_MISMATCH_IN_LINKS"
+COROSYNC_CANNOT_ADD_REMOVE_LINKS_BAD_TRANSPORT = "COROSYNC_CANNOT_ADD_REMOVE_LINKS_BAD_TRANSPORT"
+COROSYNC_CANNOT_ADD_REMOVE_LINKS_NO_LINKS_SPECIFIED = "COROSYNC_CANNOT_ADD_REMOVE_LINKS_NO_LINKS_SPECIFIED"
+COROSYNC_CANNOT_ADD_REMOVE_LINKS_TOO_MANY_FEW_LINKS = "COROSYNC_CANNOT_ADD_REMOVE_LINKS_TOO_MANY_FEW_LINKS"
+COROSYNC_LINK_ALREADY_EXISTS_CANNOT_ADD = "COROSYNC_LINK_ALREADY_EXISTS_CANNOT_ADD"
+COROSYNC_LINK_DOES_NOT_EXIST_CANNOT_REMOVE = "COROSYNC_LINK_DOES_NOT_EXIST_CANNOT_REMOVE"
+COROSYNC_LINK_DOES_NOT_EXIST_CANNOT_UPDATE = "COROSYNC_LINK_DOES_NOT_EXIST_CANNOT_UPDATE"
 COROSYNC_LINK_NUMBER_DUPLICATION = "COROSYNC_LINK_NUMBER_DUPLICATION"
 COROSYNC_NODE_ADDRESS_COUNT_MISMATCH = "COROSYNC_NODE_ADDRESS_COUNT_MISMATCH"
 COROSYNC_NODE_CONFLICT_CHECK_SKIPPED = "COROSYNC_NODE_CONFLICT_CHECK_SKIPPED"
 COROSYNC_NODES_MISSING = "COROSYNC_NODES_MISSING"
-COROSYNC_NOT_RUNNING_CHECK_STARTED = "COROSYNC_NOT_RUNNING_CHECK_STARTED"
 COROSYNC_NOT_RUNNING_CHECK_NODE_ERROR = "COROSYNC_NOT_RUNNING_CHECK_NODE_ERROR"
+COROSYNC_NOT_RUNNING_CHECK_STARTED = "COROSYNC_NOT_RUNNING_CHECK_STARTED"
 COROSYNC_NOT_RUNNING_ON_NODE = "COROSYNC_NOT_RUNNING_ON_NODE"
 COROSYNC_OPTIONS_INCOMPATIBLE_WITH_QDEVICE = "COROSYNC_OPTIONS_INCOMPATIBLE_WITH_QDEVICE"
 COROSYNC_QUORUM_ATB_CANNOT_BE_DISABLED_DUE_TO_SBD = "COROSYNC_QUORUM_ATB_CANNOT_BE_DISABLED_DUE_TO_SBD"
 COROSYNC_QUORUM_ATB_WILL_BE_ENABLED_DUE_TO_SBD = "COROSYNC_QUORUM_ATB_WILL_BE_ENABLED_DUE_TO_SBD"
 COROSYNC_QUORUM_GET_STATUS_ERROR = "COROSYNC_QUORUM_GET_STATUS_ERROR"
 COROSYNC_QUORUM_HEURISTICS_ENABLED_WITH_NO_EXEC = "COROSYNC_QUORUM_HEURISTICS_ENABLED_WITH_NO_EXEC"
+COROSYNC_QUORUM_LOSS_UNABLE_TO_CHECK = "COROSYNC_QUORUM_LOSS_UNABLE_TO_CHECK"
 COROSYNC_QUORUM_SET_EXPECTED_VOTES_ERROR = "COROSYNC_QUORUM_SET_EXPECTED_VOTES_ERROR"
 COROSYNC_QUORUM_WILL_BE_LOST = "COROSYNC_QUORUM_WILL_BE_LOST"
-COROSYNC_QUORUM_LOSS_UNABLE_TO_CHECK = "COROSYNC_QUORUM_LOSS_UNABLE_TO_CHECK"
 COROSYNC_RUNNING_ON_NODE = "COROSYNC_RUNNING_ON_NODE"
-COROSYNC_TOO_MANY_LINKS = "COROSYNC_TOO_MANY_LINKS"
+COROSYNC_TOO_MANY_LINKS_OPTIONS = "COROSYNC_TOO_MANY_LINKS_OPTIONS"
 COROSYNC_TRANSPORT_UNSUPPORTED_OPTIONS = "COROSYNC_TRANSPORT_UNSUPPORTED_OPTIONS"
 CRM_MON_ERROR = "CRM_MON_ERROR"
 DEFAULTS_CAN_BE_OVERRIDEN = "DEFAULTS_CAN_BE_OVERRIDEN"
@@ -133,7 +153,6 @@ FILES_REMOVE_FROM_NODES_SKIPPED = "FILES_REMOVE_FROM_NODES_SKIPPED"
 FILE_ALREADY_EXISTS = "FILE_ALREADY_EXISTS"
 FILE_DISTRIBUTION_ERROR = "FILE_DISTRIBUTION_ERROR"
 FILE_DISTRIBUTION_SUCCESS = "FILE_DISTRIBUTION_SUCCESS"
-FILE_DOES_NOT_EXIST = "FILE_DOES_NOT_EXIST"
 FILE_IO_ERROR = "FILE_IO_ERROR"
 FILE_REMOVE_FROM_NODE_ERROR = "FILE_REMOVE_FROM_NODE_ERROR"
 FILE_REMOVE_FROM_NODE_SUCCESS = "FILE_REMOVE_FROM_NODE_SUCCESS"
@@ -150,7 +169,6 @@ INVALID_OPTIONS = "INVALID_OPTIONS"
 INVALID_USERDEFINED_OPTIONS = "INVALID_USERDEFINED_OPTIONS"
 INVALID_OPTION_TYPE = "INVALID_OPTION_TYPE"
 INVALID_OPTION_VALUE = "INVALID_OPTION_VALUE"
-INVALID_RESOURCE_NAME = 'INVALID_RESOURCE_NAME'
 INVALID_RESOURCE_AGENT_NAME = 'INVALID_RESOURCE_AGENT_NAME'
 INVALID_RESPONSE_FORMAT = "INVALID_RESPONSE_FORMAT"
 INVALID_SCORE = "INVALID_SCORE"
@@ -160,6 +178,7 @@ MULTIPLE_SCORE_OPTIONS = "MULTIPLE_SCORE_OPTIONS"
 MULTIPLE_RESULTS_FOUND = "MULTIPLE_RESULTS_FOUND"
 MUTUALLY_EXCLUSIVE_OPTIONS = "MUTUALLY_EXCLUSIVE_OPTIONS"
 NODE_ADDRESSES_ALREADY_EXIST = "NODE_ADDRESSES_ALREADY_EXIST"
+NODE_ADDRESSES_CANNOT_BE_EMPTY = "NODE_ADDRESSES_CANNOT_BE_EMPTY"
 NODE_ADDRESSES_DUPLICATION = "NODE_ADDRESSES_DUPLICATION"
 NODE_ADDRESSES_UNRESOLVABLE = "NODE_ADDRESSES_UNRESOLVABLE"
 NODE_COMMUNICATION_COMMAND_UNSUCCESSFUL = "NODE_COMMUNICATION_COMMAND_UNSUCCESSFUL"
@@ -187,9 +206,14 @@ NODE_TO_CLEAR_IS_STILL_IN_CLUSTER = "NODE_TO_CLEAR_IS_STILL_IN_CLUSTER"
 OMITTING_NODE = "OMITTING_NODE"
 OBJECT_WITH_ID_IN_UNEXPECTED_CONTEXT = "OBJECT_WITH_ID_IN_UNEXPECTED_CONTEXT"
 PACEMAKER_LOCAL_NODE_NAME_NOT_FOUND = "PACEMAKER_LOCAL_NODE_NAME_NOT_FOUND"
-PARSE_ERROR_COROSYNC_CONF_MISSING_CLOSING_BRACE = "PARSE_ERROR_COROSYNC_CONF_MISSING_CLOSING_BRACE"
 PARSE_ERROR_COROSYNC_CONF = "PARSE_ERROR_COROSYNC_CONF"
+PARSE_ERROR_COROSYNC_CONF_EXTRA_CHARACTERS_AFTER_OPENING_BRACE = "PARSE_ERROR_COROSYNC_CONF_EXTRA_CHARACTERS_AFTER_OPENING_BRACE"
+PARSE_ERROR_COROSYNC_CONF_EXTRA_CHARACTERS_BEFORE_OR_AFTER_CLOSING_BRACE = "PARSE_ERROR_COROSYNC_CONF_EXTRA_CHARACTERS_BEFORE_OR_AFTER_CLOSING_BRACE"
+PARSE_ERROR_COROSYNC_CONF_LINE_IS_NOT_SECTION_NOR_KEY_VALUE = "PARSE_ERROR_COROSYNC_CONF_LINE_IS_NOT_SECTION_NOR_KEY_VALUE"
+PARSE_ERROR_COROSYNC_CONF_MISSING_CLOSING_BRACE = "PARSE_ERROR_COROSYNC_CONF_MISSING_CLOSING_BRACE"
+PARSE_ERROR_COROSYNC_CONF_MISSING_SECTION_NAME_BEFORE_OPENING_BRACE = "PARSE_ERROR_COROSYNC_CONF_MISSING_SECTION_NAME_BEFORE_OPENING_BRACE"
 PARSE_ERROR_COROSYNC_CONF_UNEXPECTED_CLOSING_BRACE = "PARSE_ERROR_COROSYNC_CONF_UNEXPECTED_CLOSING_BRACE"
+PARSE_ERROR_JSON_FILE = "PARSE_ERROR_JSON_FILE"
 PCSD_VERSION_TOO_OLD = "PCSD_VERSION_TOO_OLD"
 PCSD_SSL_CERT_AND_KEY_DISTRIBUTION_STARTED = "PCSD_SSL_CERT_AND_KEY_DISTRIBUTION_STARTED"
 PCSD_SSL_CERT_AND_KEY_SET_SUCCESS = "PCSD_SSL_CERT_AND_KEY_SET_SUCCESS"
@@ -216,23 +240,31 @@ QDEVICE_NOT_RUNNING = "QDEVICE_NOT_RUNNING"
 QDEVICE_CLIENT_RELOAD_STARTED = "QDEVICE_CLIENT_RELOAD_STARTED"
 QDEVICE_REMOVE_OR_CLUSTER_STOP_NEEDED = "QDEVICE_REMOVE_OR_CLUSTER_STOP_NEEDED"
 QDEVICE_USED_BY_CLUSTERS = "QDEVICE_USED_BY_CLUSTERS"
-REQUIRED_OPTION_IS_MISSING = "REQUIRED_OPTION_IS_MISSING"
+REQUIRED_OPTIONS_ARE_MISSING = "REQUIRED_OPTIONS_ARE_MISSING"
 REQUIRED_OPTION_OF_ALTERNATIVES_IS_MISSING = "REQUIRED_OPTION_OF_ALTERNATIVES_IS_MISSING"
+RESOURCE_BAN_PCMK_ERROR = "RESOURCE_BAN_PCMK_ERROR"
+RESOURCE_BAN_PCMK_SUCCESS = "RESOURCE_BAN_PCMK_SUCCESS"
 RESOURCE_BUNDLE_ALREADY_CONTAINS_A_RESOURCE = "RESOURCE_BUNDLE_ALREADY_CONTAINS_A_RESOURCE"
 RESOURCE_BUNDLE_UNSUPPORTED_CONTAINER_TYPE = "RESOURCE_BUNDLE_UNSUPPORTED_CONTAINER_TYPE"
-RESOURCE_IN_BUNDLE_NOT_ACCESSIBLE = "RESOURCE_IN_BUNDLE_NOT_ACCESSIBLE"
-RESOURCE_CANNOT_BE_NEXT_TO_ITSELF_IN_GROUP = "RESOURCE_CANNOT_BE_NEXT_TO_ITSELF_IN_GROUP"
 RESOURCE_CLEANUP_ERROR = "RESOURCE_CLEANUP_ERROR"
 RESOURCE_DOES_NOT_RUN = "RESOURCE_DOES_NOT_RUN"
+RESOURCE_DISABLE_AFFECTS_OTHER_RESOURCES = "RESOURCE_DISABLE_AFFECTS_OTHER_RESOURCES"
 RESOURCE_FOR_CONSTRAINT_IS_MULTIINSTANCE = 'RESOURCE_FOR_CONSTRAINT_IS_MULTIINSTANCE'
+RESOURCE_IN_BUNDLE_NOT_ACCESSIBLE = "RESOURCE_IN_BUNDLE_NOT_ACCESSIBLE"
+RESOURCE_INSTANCE_ATTR_VALUE_NOT_UNIQUE = "RESOURCE_INSTANCE_ATTR_VALUE_NOT_UNIQUE"
 RESOURCE_IS_GUEST_NODE_ALREADY = "RESOURCE_IS_GUEST_NODE_ALREADY"
 RESOURCE_IS_UNMANAGED = "RESOURCE_IS_UNMANAGED"
 RESOURCE_MANAGED_NO_MONITOR_ENABLED = "RESOURCE_MANAGED_NO_MONITOR_ENABLED"
-RESOURCE_OPERATION_INTERVAL_DUPLICATION = "RESOURCE_OPERATION_INTERVAL_DUPLICATION"
+RESOURCE_MOVE_PCMK_ERROR = "RESOURCE_MOVE_PCMK_ERROR"
+RESOURCE_MOVE_PCMK_SUCCESS = "RESOURCE_MOVE_PCMK_SUCCESS"
 RESOURCE_OPERATION_INTERVAL_ADAPTED = "RESOURCE_OPERATION_INTERVAL_ADAPTED"
+RESOURCE_OPERATION_INTERVAL_DUPLICATION = "RESOURCE_OPERATION_INTERVAL_DUPLICATION"
 RESOURCE_REFRESH_ERROR = "RESOURCE_REFRESH_ERROR"
 RESOURCE_REFRESH_TOO_TIME_CONSUMING = 'RESOURCE_REFRESH_TOO_TIME_CONSUMING'
 RESOURCE_RUNNING_ON_NODES = "RESOURCE_RUNNING_ON_NODES"
+RESOURCE_UNMOVE_UNBAN_PCMK_ERROR = "RESOURCE_UNMOVE_UNBAN_PCMK_ERROR"
+RESOURCE_UNMOVE_UNBAN_PCMK_SUCCESS = "RESOURCE_UNMOVE_UNBAN_PCMK_SUCCESS"
+RESOURCE_UNMOVE_UNBAN_PCMK_EXPIRED_NOT_SUPPORTED = "RESOURCE_UNMOVE_UNBAN_PCMK_EXPIRED_NOT_SUPPORTED"
 RUN_EXTERNAL_PROCESS_ERROR = "RUN_EXTERNAL_PROCESS_ERROR"
 RUN_EXTERNAL_PROCESS_FINISHED = "RUN_EXTERNAL_PROCESS_FINISHED"
 RUN_EXTERNAL_PROCESS_STARTED = "RUN_EXTERNAL_PROCESS_STARTED"
@@ -253,7 +285,6 @@ SBD_DISABLING_STARTED = "SBD_DISABLING_STARTED"
 SBD_ENABLING_STARTED = "SBD_ENABLING_STARTED"
 SBD_LIST_WATCHDOG_ERROR = "SBD_LIST_WATCHDOG_ERROR"
 SBD_NO_DEVICE_FOR_NODE = "SBD_NO_DEVICE_FOR_NODE"
-SBD_NOT_ENABLED = "SBD_NOT_ENABLED"
 SBD_NOT_INSTALLED = "SBD_NOT_INSTALLED"
 SBD_NOT_USED_CANNOT_SET_SBD_OPTIONS = "SBD_NOT_USED_CANNOT_SET_SBD_OPTIONS"
 SBD_TOO_MANY_DEVICES_FOR_NODE = "SBD_TOO_MANY_DEVICES_FOR_NODE"
@@ -297,7 +328,6 @@ UNABLE_TO_READ_COROSYNC_CONFIG = "UNABLE_TO_READ_COROSYNC_CONFIG"
 UNABLE_TO_GET_SBD_CONFIG = "UNABLE_TO_GET_SBD_CONFIG"
 UNABLE_TO_GET_SBD_STATUS = "UNABLE_TO_GET_SBD_STATUS"
 UNABLE_TO_PERFORM_OPERATION_ON_ANY_NODE = "UNABLE_TO_PERFORM_OPERATION_ON_ANY_NODE"
-UNKNOWN_COMMAND = 'UNKNOWN_COMMAND'
 WATCHDOG_INVALID = "WATCHDOG_INVALID"
 UNSUPPORTED_OPERATION_ON_NON_SYSTEMD_SYSTEMS = "UNSUPPORTED_OPERATION_ON_NON_SYSTEMD_SYSTEMS"
 USE_COMMAND_NODE_ADD_REMOTE = "USE_COMMAND_NODE_ADD_REMOTE"
