@@ -1,5 +1,13 @@
-import threading
 from collections import namedtuple
+from enum import Enum
+import threading
+from typing import (
+    Iterable,
+    List,
+    Mapping,
+    Optional,
+)
+
 from lxml import etree
 
 def run_parallel(worker, data_list):
@@ -14,10 +22,37 @@ def run_parallel(worker, data_list):
         thread.join()
 
 def format_environment_error(e):
+    return format_os_error(e)
+
+def format_os_error(e):
     if e.filename:
         return "{0}: '{1}'".format(e.strerror, e.filename)
     return e.strerror
 
+def indent(line_list: Iterable[str], indent_step: int = 2) -> List[str]:
+    """
+    return line list where each line of input is prefixed by N spaces
+
+    line_list -- original lines
+    indent_step -- count of spaces for line prefix
+    """
+    return [
+        "{0}{1}".format(" " * indent_step, line) if line else line
+        for line in line_list
+    ]
+
+def format_list(
+    item_list: Iterable[str],
+    optional_transformations: Optional[Mapping[str, str]] = None
+) -> str:
+    if not optional_transformations:
+        optional_transformations = {}
+
+    return ", ".join(sorted([
+        "'{0}'".format(optional_transformations.get(item, item))
+        for item in item_list
+    ]))
+
 def join_multilines(strings):
     return "\n".join([a.strip() for a in strings if a.strip()])
 
@@ -35,6 +70,12 @@ def xml_fromstring(xml):
         etree.XMLParser(huge_tree=True)
     )
 
+class AutoNameEnum(str, Enum):
+    def _generate_next_value_(name, start, count, last_values):
+        # pylint: disable=no-self-argument
+        del start, count, last_values
+        return name
+
 class Version(namedtuple("Version", ["major", "minor", "revision"])):
     def __new__(cls, major, minor=None, revision=None):
         return super(Version, cls).__new__(cls, major, minor, revision)
