@@ -14,6 +14,7 @@ import tempfile
 import time
 import platform
 import shutil
+import difflib
 
 try:
     import clufter.facts
@@ -27,7 +28,6 @@ except ImportError:
 from pcs import (
     cluster,
     constraint,
-    prop,
     quorum,
     resource,
     settings,
@@ -37,60 +37,23 @@ from pcs import (
     utils,
     alert,
 )
-from pcs.lib.errors import LibraryError
-from pcs.lib.commands import quorum as lib_quorum
-import pcs.cli.constraint_colocation.command as colocation_command
-import pcs.cli.constraint_order.command as order_command
-import pcs.cli.constraint_ticket.command as ticket_command
-from pcs.cli.common.console_report import indent
+from pcs.cli.common import middleware
 from pcs.cli.common.errors import CmdLineInputError
+from pcs.cli.common.reports import process_library_reports
+from pcs.cli.constraint import command as constraint_command
+from pcs.cli.constraint_colocation import (
+    console_report as colocation_console_report,
+)
+from pcs.cli.constraint_order import console_report as order_console_report
+from pcs.cli.constraint_ticket import console_report as ticket_console_report
+from pcs.common.tools import indent
+from pcs.lib.commands import quorum as lib_quorum
+from pcs.lib.errors import LibraryError
+from pcs.lib.external import is_service_running
+from pcs.lib.node import get_existing_nodes_names
 
 # pylint: disable=too-many-branches, too-many-locals, too-many-statements
 
-def config_cmd(lib, argv, modifiers):
-    if not argv:
-        config_show(lib, argv, modifiers)
-        return
-
-    try:
-        sub_cmd = argv.pop(0)
-        if sub_cmd == "help":
-            usage.config(argv)
-        elif sub_cmd == "show":
-            config_show(lib, argv, modifiers)
-        elif sub_cmd == "backup":
-            config_backup(lib, argv, modifiers)
-        elif sub_cmd == "restore":
-            config_restore(lib, argv, modifiers)
-        elif sub_cmd == "checkpoint":
-            if not argv:
-                config_checkpoint_list(lib, argv, modifiers)
-            elif argv[0] == "view":
-                config_checkpoint_view(lib, argv[1:], modifiers)
-            elif argv[0] == "restore":
-                config_checkpoint_restore(lib, argv[1:], modifiers)
-            else:
-                raise CmdLineInputError()
-        elif sub_cmd == "import-cman":
-            config_import_cman(lib, argv, modifiers)
-        elif sub_cmd == "export":
-            if not argv:
-                raise CmdLineInputError()
-            elif argv[0] == "pcs-commands":
-                config_export_pcs_commands(lib, argv[1:], modifiers)
-            elif argv[0] == "pcs-commands-verbose":
-                config_export_pcs_commands(
-                    lib, argv[1:], modifiers, verbose=True
-                )
-            else:
-                raise CmdLineInputError()
-        else:
-            raise CmdLineInputError()
-    except LibraryError as e:
-        utils.process_library_reports(e.args)
-    except CmdLineInputError as e:
-        utils.exit_on_cmdline_input_errror(e, "config", sub_cmd)
-
 def config_show(lib, argv, modifiers):
     """
     Options:
@@ -104,7 +67,7 @@ def config_show(lib, argv, modifiers):
     print("Cluster Name: %s" % utils.getClusterName())
     status.nodes_status(lib, ["config"], modifiers.get_subset("-f"))
     print()
-    config_show_cib(lib)
+    print("\n".join(_config_show_cib_lines(lib)))
     if (
         utils.hasCorosyncConf()
         and
@@ -126,57 +89,104 @@ def config_show(lib, argv, modifiers):
             config = lib_quorum.get_config(utils.get_lib_env())
             print("\n".join(indent(quorum.quorum_config_to_str(config))))
         except LibraryError as e:
-            utils.process_library_reports(e.args)
+            process_library_reports(e.args)
 
-def config_show_cib(lib):
+def _config_show_cib_lines(lib):
     """
     Commandline options:
       * -f - CIB file
     """
-    print("Resources:")
     # update of pcs_options will change output of constraint show
     utils.pcs_options["--full"] = 1
     # get latest modifiers object after updating pcs_options
     modifiers = utils.get_input_modifiers()
-    resource.resource_config(
-        lib,
-        [],
-        modifiers.get_subset("-f")
-    )
+    cib_xml = utils.get_cib()
+    cib_etree = utils.get_cib_etree(cib_xml=cib_xml)
+    cib_dom = utils.get_cib_dom(cib_xml=cib_xml)
 
-    print()
-    print("Stonith Devices:")
-    resource.resource_config(
-        lib,
-        [],
-        modifiers.get_subset("-f"),
-        stonith=True,
-    )
-    print("Fencing Levels:")
-    levels = stonith.stonith_level_config_to_str(
+    resource_lines = []
+    stonith_lines = []
+    for resource_el in cib_etree.find(".//resources"):
+        is_stonith = (
+            "class" in resource_el.attrib
+            and
+            resource_el.attrib["class"] == "stonith"
+        )
+        resource_el_lines = resource.resource_node_lines(resource_el)
+        if is_stonith:
+            stonith_lines += resource_el_lines
+        else:
+            resource_lines += resource_el_lines
+
+    all_lines = []
+
+    all_lines.append("Resources:")
+    all_lines.extend(indent(resource_lines, indent_step=1))
+    all_lines.append("")
+    all_lines.append("Stonith Devices:")
+    all_lines.extend(indent(stonith_lines, indent_step=1))
+    all_lines.append("Fencing Levels:")
+    levels_lines = stonith.stonith_level_config_to_str(
         lib.fencing_topology.get_config()
     )
-    if levels:
-        print("\n".join(indent(levels, 2)))
+    if levels_lines:
+        all_lines.extend(indent(levels_lines, indent_step=2))
 
-    print()
-    constraint.location_show(lib, [], modifiers.get_subset("-f", "--full"))
-    order_command.show(lib, [], modifiers.get_subset("-f", "--full"))
-    colocation_command.show(lib, [], modifiers.get_subset("-f", "--full"))
-    ticket_command.show(lib, [], modifiers.get_subset("-f", "--full"))
-
-    print()
-    alert.print_alert_config(lib, [], modifiers.get_subset("-f"))
-
-    print()
-    print("Resources Defaults:")
-    resource.show_defaults("rsc_defaults", indent=" ")
-    print("Operations Defaults:")
-    resource.show_defaults("op_defaults", indent=" ")
-    print()
-    prop.list_property(
-        lib, [], modifiers.get_subset("--defaults", "--all", "-f")
+    all_lines.append("")
+    constraints_element = cib_dom.getElementsByTagName('constraints')[0]
+    all_lines.extend(
+        constraint.location_lines(
+            constraints_element,
+            showDetail=True,
+            show_expired=True,
+            verify_expiration=False
+        )
     )
+    all_lines.extend(constraint_command.show(
+        "Ordering Constraints:",
+        lib.constraint_order.show,
+        order_console_report.constraint_plain,
+        modifiers.get_subset("-f", "--full"),
+    ))
+    all_lines.extend(constraint_command.show(
+         "Colocation Constraints:",
+        lib.constraint_colocation.show,
+        colocation_console_report.constraint_plain,
+        modifiers.get_subset("-f", "--full"),
+    ))
+    all_lines.extend(constraint_command.show(
+        "Ticket Constraints:",
+        lib.constraint_ticket.show,
+        ticket_console_report.constraint_plain,
+        modifiers.get_subset("-f", "--full"),
+    ))
+
+    all_lines.append("")
+    all_lines.extend(alert.alert_config_lines(lib))
+
+    all_lines.append("")
+    all_lines.append("Resources Defaults:")
+    all_lines.extend(indent(
+        resource.show_defaults(cib_dom, "rsc_defaults"),
+        indent_step=1
+    ))
+    all_lines.append("Operations Defaults:")
+    all_lines.extend(indent(
+        resource.show_defaults(cib_dom, "op_defaults"),
+        indent_step=1
+    ))
+
+    all_lines.append("")
+    all_lines.append("Cluster Properties:")
+    properties = utils.get_set_properties()
+    all_lines.extend(indent(
+        [
+            "{0}: {1}".format(prop, val)
+            for prop, val in sorted(properties.items())
+        ],
+        indent_step=1
+    ))
+    return all_lines
 
 def config_backup(lib, argv, modifiers):
     """
@@ -303,9 +313,13 @@ def config_restore_remote(infile_name, infile_obj):
 
     config_backup_check_version(extracted["version.txt"])
 
-    node_list = utils.get_corosync_conf_facade(
-        conf_text=extracted["corosync.conf"].decode("utf-8")
-    ).get_nodes_names()
+    node_list, report_list = get_existing_nodes_names(
+        utils.get_corosync_conf_facade(
+            conf_text=extracted["corosync.conf"].decode("utf-8")
+        )
+    )
+    if report_list:
+        process_library_reports(report_list)
     if not node_list:
         utils.err("no nodes found in the tarball")
 
@@ -366,11 +380,11 @@ def config_restore_local(infile_name, infile_obj):
     Commandline options: no options
     """
     if (
-        status.is_service_running("corosync")
+        is_service_running(utils.cmd_runner(), "corosync")
         or
-        status.is_service_running("pacemaker")
+        is_service_running(utils.cmd_runner(), "pacemaker")
         or
-        status.is_service_running("pacemaker_remote")
+        is_service_running(utils.cmd_runner(), "pacemaker_remote")
     ):
         utils.err(
             "Cluster is currently running on this node. You need to stop "
@@ -642,20 +656,89 @@ def config_checkpoint_list(lib, argv, modifiers):
             % (cib_info[1], datetime.datetime.fromtimestamp(round(cib_info[0])))
         )
 
+def _checkpoint_to_lines(lib, checkpoint_number):
+    # backup current settings
+    orig_usefile = utils.usefile
+    orig_filename = utils.filename
+    orig_middleware = lib.middleware_factory
+    # configure old code to read the CIB from a file
+    utils.usefile = True
+    utils.filename = os.path.join(
+        settings.cib_dir,
+        "cib-%s.raw" % checkpoint_number
+    )
+    # configure new code to read the CIB from a file
+    lib.middleware_factory = orig_middleware._replace(
+        cib=middleware.cib(utils.filename, utils.touch_cib_file)
+    )
+    # export the CIB to text
+    result = False, []
+    if os.path.isfile(utils.filename):
+        result = True, _config_show_cib_lines(lib)
+    # restore original settings
+    utils.usefile = orig_usefile
+    utils.filename = orig_filename
+    lib.middleware_factory = orig_middleware
+    return result
+
 def config_checkpoint_view(lib, argv, modifiers):
     """
     Options: no options
     """
     modifiers.ensure_only_supported()
     if len(argv) != 1:
-        usage.config(["checkpoint", "view"])
+        usage.config(["checkpoint view"])
         sys.exit(1)
 
-    utils.usefile = True
-    utils.filename = os.path.join(settings.cib_dir, "cib-%s.raw" % argv[0])
-    if not os.path.isfile(utils.filename):
+    loaded, lines = _checkpoint_to_lines(lib, argv[0])
+    if not loaded:
         utils.err("unable to read the checkpoint")
-    config_show_cib(lib)
+    print("\n".join(lines))
+
+def config_checkpoint_diff(lib, argv, modifiers):
+    """
+    Commandline options:
+      * -f - CIB file
+    """
+    modifiers.ensure_only_supported("-f")
+    if len(argv) != 2:
+        usage.config(["checkpoint diff"])
+        sys.exit(1)
+
+    if argv[0] == argv[1]:
+        utils.err("cannot diff a checkpoint against itself")
+
+    errors = []
+    checkpoints_lines = []
+    for checkpoint in argv:
+        if checkpoint == "live":
+            lines = _config_show_cib_lines(lib)
+            if not lines:
+                errors.append("unable to read live configuration")
+            else:
+                checkpoints_lines.append(lines)
+        else:
+            loaded, lines = _checkpoint_to_lines(lib, checkpoint)
+            if not loaded:
+                errors.append(
+                    "unable to read checkpoint '{0}'".format(checkpoint)
+                )
+            else:
+                checkpoints_lines.append(lines)
+
+    if errors:
+        utils.err("\n".join(errors))
+
+    print("Differences between {0} (-) and {1} (+):".format(*[
+        "live configuration" if label == "live" else f"checkpoint {label}"
+        for label in argv
+    ]))
+    print("\n".join([
+        line.rstrip() for line in difflib.Differ().compare(
+            checkpoints_lines[0],
+            checkpoints_lines[1]
+        )]
+    ))
 
 def config_checkpoint_restore(lib, argv, modifiers):
     """
@@ -666,7 +749,7 @@ def config_checkpoint_restore(lib, argv, modifiers):
     del lib
     modifiers.ensure_only_supported("-f")
     if len(argv) != 1:
-        usage.config(["checkpoint", "restore"])
+        usage.config(["checkpoint restore"])
         sys.exit(1)
 
     cib_path = os.path.join(settings.cib_dir, "cib-%s.raw" % argv[0])
@@ -925,7 +1008,7 @@ def config_export_pcs_commands(lib, argv, modifiers, verbose=False):
             invalid_args = True
     # check options
     if invalid_args:
-        usage.config(["export", "pcs-commands"])
+        usage.config(["export pcs-commands"])
         sys.exit(1)
     # complete optional options
     if dist is None:
