@@ -1,10 +1,13 @@
 import sys
 from collections import defaultdict
+from os.path import isfile
 import xml.dom.minidom
 from xml.dom.minidom import parseString
+from enum import Enum
 
 from pcs import (
     rule as rule_utils,
+    settings,
     usage,
     utils,
 )
@@ -13,20 +16,26 @@ from pcs.cli import (
     constraint_order,
 )
 from pcs.cli.common import parse_args
-from pcs.cli.common.console_report import error, warn
+from pcs.cli.common.console_report import warn
 from pcs.cli.common.errors import CmdLineInputError
+from pcs.cli.common.reports import process_library_reports
 import pcs.cli.constraint_colocation.command as colocation_command
 import pcs.cli.constraint_order.command as order_command
 from pcs.cli.constraint_ticket import command as ticket_command
+from pcs.common import report_codes
+from pcs.lib import reports
 from pcs.lib.cib.constraint import resource_set
 from pcs.lib.cib.constraint.order import ATTRIB as order_attrib
-from pcs.lib.env_tools import get_existing_nodes_names
-from pcs.lib.errors import LibraryError
-from pcs.lib.pacemaker.values import sanitize_id
+from pcs.lib.node import get_existing_nodes_names
+from pcs.lib.pacemaker.values import (
+    RESOURCE_ROLES,
+    sanitize_id,
+    SCORE_INFINITY,
+)
 
 # pylint: disable=too-many-branches, too-many-statements
 # pylint: disable=invalid-name, too-many-nested-blocks
-# pylint: disable=too-many-locals
+# pylint: disable=too-many-locals, too-many-lines
 
 OPTIONS_ACTION = resource_set.ATTRIB["action"]
 
@@ -39,122 +48,84 @@ OPTIONS_KIND = order_attrib["kind"]
 LOCATION_NODE_VALIDATION_SKIP_MSG = (
     "Validation for node existence in the cluster will be skipped"
 )
+CRM_RULE_MISSING_MSG = (
+    "Warning: crm_rule is not available, therefore expired constraints may be "
+    "shown. Consider upgrading pacemaker.\n"
+)
 
 RESOURCE_TYPE_RESOURCE = "resource"
 RESOURCE_TYPE_REGEXP = "regexp"
 
-def constraint_cmd(lib, argv, modifiers):
+RULE_IN_EFFECT = "in effect"
+RULE_EXPIRED = "expired"
+RULE_NOT_IN_EFFECT = "not yet in effect"
+RULE_UNKNOWN_STATUS = "unknown status"
+
+class CrmRuleReturnCode(Enum):
+    IN_EFFECT = 0
+    EXPIRED = 110
+    TO_BE_IN_EFFECT = 111
+
+
+def constraint_location_cmd(lib, argv, modifiers):
     if not argv:
-        argv = ["list"]
-    sub_cmd = argv.pop(0)
+        sub_cmd = "show"
+    else:
+        sub_cmd = argv.pop(0)
 
     try:
-        if sub_cmd == "help":
-            usage.constraint(argv)
-        elif sub_cmd == "location":
-            try:
-                if not argv:
-                    sub_cmd2 = "show"
-                else:
-                    sub_cmd2 = argv.pop(0)
-
-                if sub_cmd2 == "add":
-                    location_add(lib, argv, modifiers)
-                elif sub_cmd2 in ["remove", "delete"]:
-                    location_remove(lib, argv, modifiers)
-                elif sub_cmd2 == "show":
-                    location_show(lib, argv, modifiers)
-                elif len(argv) >= 2:
-                    if argv[0] == "rule":
-                        location_rule(lib, [sub_cmd2] + argv, modifiers)
-                    else:
-                        location_prefer(lib, [sub_cmd2] + argv, modifiers)
-                else:
-                    raise CmdLineInputError()
-            except CmdLineInputError as e:
-                utils.exit_on_cmdline_input_errror(
-                    e, "constraint", f"location {sub_cmd2}"
-                )
-        elif sub_cmd == "order":
-            if not argv:
-                sub_cmd2 = "show"
-            else:
-                sub_cmd2 = argv.pop(0)
-
-            try:
-                if sub_cmd2 == "set":
-                    order_command.create_with_set(lib, argv, modifiers)
-                elif sub_cmd2 in ["remove", "delete"]:
-                    order_rm(lib, argv, modifiers)
-                elif sub_cmd2 == "show":
-                    order_command.show(lib, argv, modifiers)
-                else:
-                    order_start(lib, [sub_cmd2] + argv, modifiers)
-            except CmdLineInputError as e:
-                utils.exit_on_cmdline_input_errror(
-                    e, "constraint", f"order {sub_cmd2}"
-                )
-        elif sub_cmd == "ticket":
-            usage_name = "ticket"
-            try:
-                command_map = {
-                    "set": ticket_command.create_with_set,
-                    "add": ticket_command.add,
-                    "delete": ticket_command.remove,
-                    "remove": ticket_command.remove,
-                    "show": ticket_command.show,
-                }
-                sub_command = argv[0] if argv else "show"
-                if sub_command not in command_map:
-                    raise CmdLineInputError()
-                usage_name = "ticket "+sub_command
-
-                command_map[sub_command](lib, argv[1:], modifiers)
-            except CmdLineInputError as e:
-                utils.exit_on_cmdline_input_errror(e, "constraint", usage_name)
-
-        elif sub_cmd == "colocation":
-            if not argv:
-                sub_cmd2 = "show"
-            else:
-                sub_cmd2 = argv.pop(0)
-
-            try:
-                if sub_cmd2 == "add":
-                    colocation_add(lib, argv, modifiers)
-                elif sub_cmd2 in ["remove", "delete"]:
-                    colocation_rm(lib, argv, modifiers)
-                elif sub_cmd2 == "set":
-                    colocation_command.create_with_set(lib, argv, modifiers)
-                elif sub_cmd2 == "show":
-                    colocation_command.show(lib, argv, modifiers)
-                else:
-                    raise CmdLineInputError()
-            except CmdLineInputError as e:
-                utils.exit_on_cmdline_input_errror(
-                    e, "constraint", f"colocation {sub_cmd2}"
-                )
+        if sub_cmd == "add":
+            location_add(lib, argv, modifiers)
         elif sub_cmd in ["remove", "delete"]:
-            constraint_rm(lib, argv, modifiers)
-        elif sub_cmd in ("show", "list"):
-            # all these commands accept -f and --full therefore there is no
-            # need to change something here
+            location_remove(lib, argv, modifiers)
+        elif sub_cmd == "show":
             location_show(lib, argv, modifiers)
-            order_command.show(lib, argv, modifiers)
-            colocation_command.show(lib, argv, modifiers)
-            ticket_command.show(lib, argv, modifiers)
-        elif sub_cmd == "ref":
-            constraint_ref(lib, argv, modifiers)
-        elif sub_cmd == "rule":
-            constraint_rule(lib, argv, modifiers)
+        elif len(argv) >= 2:
+            if argv[0] == "rule":
+                location_rule(lib, [sub_cmd] + argv, modifiers)
+            else:
+                location_prefer(lib, [sub_cmd] + argv, modifiers)
         else:
             raise CmdLineInputError()
-    except LibraryError as e:
-        utils.process_library_reports(e.args)
     except CmdLineInputError as e:
-        utils.exit_on_cmdline_input_errror(e, "constraint", sub_cmd)
+        utils.exit_on_cmdline_input_errror(
+            e, "constraint", ["location", sub_cmd]
+        )
 
 
+def constraint_order_cmd(lib, argv, modifiers):
+    if not argv:
+        sub_cmd = "show"
+    else:
+        sub_cmd = argv.pop(0)
+
+    try:
+        if sub_cmd == "set":
+            order_command.create_with_set(lib, argv, modifiers)
+        elif sub_cmd in ["remove", "delete"]:
+            order_rm(lib, argv, modifiers)
+        elif sub_cmd == "show":
+            order_command.show(lib, argv, modifiers)
+        else:
+            order_start(lib, [sub_cmd] + argv, modifiers)
+    except CmdLineInputError as e:
+        utils.exit_on_cmdline_input_errror(
+            e, "constraint", ["order", sub_cmd]
+        )
+
+
+def constraint_show(lib, argv, modifiers):
+    """
+    Options:
+      * --all - print expired constraints
+      * -f - CIB file
+      * --full
+    """
+    location_show(lib, argv, modifiers)
+    order_command.show(lib, argv, modifiers.get_subset("--full", "-f"))
+    colocation_command.show(lib, argv, modifiers.get_subset("--full", "-f"))
+    ticket_command.show(lib, argv, modifiers.get_subset("--full", "-f"))
+
 
 def colocation_rm(lib, argv, modifiers):
     """
@@ -194,29 +165,11 @@ def colocation_rm(lib, argv, modifiers):
         print("No matching resources found in ordering list")
 
 
-# When passed an array of arguments if the first argument doesn't have an '='
-# then it's the score, otherwise they're all arguments
-# Return a tuple with the score and array of name,value pairs
-def parse_score_options(argv):
-    """
-    Commandline options: no options
-    """
-    if not argv:
-        return "INFINITY", []
-
-    arg_array = []
-    first = argv[0]
-    if first.find('=') != -1:
-        score = "INFINITY"
-    else:
-        score = argv.pop(0)
-
-    for arg in argv:
-        args = arg.split('=')
-        if len(args) != 2:
-            continue
-        arg_array.append(args)
-    return (score, arg_array)
+def _validate_constraint_resource(cib_dom, resource_id):
+    resource_valid, resource_error, dummy_correct_id \
+        = utils.validate_constraint_resource(cib_dom, resource_id)
+    if not resource_valid:
+        utils.err(resource_error)
 
 # Syntax: colocation add [role] <src> with [role] <tgt> [score] [options]
 # possible commands:
@@ -230,6 +183,34 @@ def colocation_add(lib, argv, modifiers):
       * -f - CIB file
       * --force - allow constraint on any resource, allow duplicate constraints
     """
+    def _parse_score_options(argv):
+        # When passed an array of arguments if the first argument doesn't have
+        # an '=' then it's the score, otherwise they're all arguments. Return a
+        # tuple with the score and array of name,value pairs
+        """
+        Commandline options: no options
+        """
+        if not argv:
+            return SCORE_INFINITY, []
+        score = SCORE_INFINITY if "=" in argv[0] else argv.pop(0)
+        # create a list of 2-tuples (name, value)
+        arg_array = [
+            parse_args.split_option(arg, allow_empty_value=False)
+            for arg in argv
+        ]
+        return score, arg_array
+
+    def _validate_and_prepare_role(role):
+        role_cleaned = role.lower().capitalize()
+        if role_cleaned not in RESOURCE_ROLES:
+            utils.err(
+                "invalid role value '{0}', allowed values are: '{1}'".format(
+                    role,
+                    "', '".join(RESOURCE_ROLES)
+                )
+            )
+        return role_cleaned
+
     del lib
     modifiers.ensure_only_supported("-f", "--force")
     if len(argv) < 3:
@@ -239,38 +220,42 @@ def colocation_add(lib, argv, modifiers):
     role2 = ""
 
     if argv[2] == "with":
-        role1 = argv.pop(0).lower().capitalize()
+        role1 = _validate_and_prepare_role(argv.pop(0))
         resource1 = argv.pop(0)
     elif argv[1] == "with":
         resource1 = argv.pop(0)
     else:
         raise CmdLineInputError()
 
-    argv.pop(0) # Pop 'with'
+    if argv.pop(0) != "with":
+        raise CmdLineInputError()
+    if "with" in argv:
+        raise CmdLineInputError(
+            message="Multiple 'with's cannot be specified.",
+            hint=(
+                "Use the 'pcs constraint colocation set' command if you want "
+                "to create a constraint for more than two resources."
+            ),
+            show_both_usage_and_message=True
+        )
 
     if not argv:
         raise CmdLineInputError()
-    elif len(argv) == 1:
+    if len(argv) == 1:
         resource2 = argv.pop(0)
     else:
         if utils.is_score_or_opt(argv[1]):
             resource2 = argv.pop(0)
         else:
-            role2 = argv.pop(0).lower().capitalize()
+            role2 = _validate_and_prepare_role(argv.pop(0))
             resource2 = argv.pop(0)
 
-    score, nv_pairs = parse_score_options(argv)
+    score, nv_pairs = _parse_score_options(argv)
 
 
     cib_dom = utils.get_cib_dom()
-    resource_valid, resource_error, dummy_correct_id \
-        = utils.validate_constraint_resource(cib_dom, resource1)
-    if not resource_valid:
-        utils.err(resource_error)
-    resource_valid, resource_error, dummy_correct_id \
-        = utils.validate_constraint_resource(cib_dom, resource2)
-    if not resource_valid:
-        utils.err(resource_error)
+    _validate_constraint_resource(cib_dom, resource1)
+    _validate_constraint_resource(cib_dom, resource2)
 
     id_in_nvpairs = None
     for name, value in nv_pairs:
@@ -430,43 +415,40 @@ def order_start(lib, argv, modifiers):
     order_options = []
     if argv:
         order_options = order_options + argv[:]
+    if "then" in order_options:
+        raise CmdLineInputError(
+            message="Multiple 'then's cannot be specified.",
+            hint=(
+                "Use the 'pcs constraint order set' command if you want to "
+                "create a constraint for more than two resources."
+            ),
+            show_both_usage_and_message=True
+        )
 
     order_options.append("first-action="+first_action)
     order_options.append("then-action="+then_action)
-    order_add([resource1, resource2] + order_options, modifiers)
+    _order_add(resource1, resource2, order_options, modifiers)
 
-def order_add(argv, modifiers):
+def _order_add(resource1, resource2, options_list, modifiers):
     """
     Commandline options:
       * -f - CIB file
       * --force - allow constraint for any resource, allow duplicate constraints
     """
-    if len(argv) < 2:
-        raise CmdLineInputError()
-
-    resource1 = argv.pop(0)
-    resource2 = argv.pop(0)
-
     cib_dom = utils.get_cib_dom()
-    resource_valid, resource_error, dummy_correct_id \
-        = utils.validate_constraint_resource(cib_dom, resource1)
-    if not resource_valid:
-        utils.err(resource_error)
-    resource_valid, resource_error, dummy_correct_id \
-        = utils.validate_constraint_resource(cib_dom, resource2)
-    if not resource_valid:
-        utils.err(resource_error)
+    _validate_constraint_resource(cib_dom, resource1)
+    _validate_constraint_resource(cib_dom, resource2)
 
     order_options = []
     id_specified = False
     sym = None
-    for arg in argv:
+    for arg in options_list:
         if arg == "symmetrical":
             sym = "true"
         elif arg == "nonsymmetrical":
             sym = "false"
-        elif "=" in arg:
-            name, value = arg.split("=", 1)
+        else:
+            name, value = parse_args.split_option(arg, allow_empty_value=False)
             if name == "id":
                 id_valid, id_error = utils.validate_xml_id(
                     value,
@@ -570,23 +552,19 @@ def order_find_duplicates(dom, constraint_el):
 def location_show(lib, argv, modifiers):
     """
     Options:
+      * --all - print expired constraints
       * --full - print all details
       * -f - CIB file
     """
     del lib
-    modifiers.ensure_only_supported("-f", "--full")
+    modifiers.ensure_only_supported("-f", "--full", "--all")
+    by_node = False
+
     if argv and argv[0] == "nodes":
-        byNode = True
-        showDetail = False
-    elif modifiers.get("--full"):
-        byNode = False
-        showDetail = True
-    else:
-        byNode = False
-        showDetail = False
+        by_node = True
 
     if len(argv) > 1:
-        if byNode:
+        if by_node:
             valid_noderes = argv[1:]
         else:
             valid_noderes = [
@@ -601,6 +579,22 @@ def location_show(lib, argv, modifiers):
         valid_noderes = []
 
     (dummy_dom, constraintsElement) = getCurrentConstraints()
+    print("\n".join(location_lines(
+        constraintsElement,
+        showDetail=modifiers.get("--full"),
+        byNode=by_node,
+        valid_noderes=valid_noderes,
+        show_expired=modifiers.get("--all"),
+    )))
+
+def location_lines(
+    constraintsElement, showDetail=False, byNode=False, valid_noderes=None,
+    show_expired=False, verify_expiration=True
+):
+    """
+    Commandline options: no options
+    """
+    all_lines = []
     nodehashon = {}
     nodehashoff = {}
     rschashon = {}
@@ -609,8 +603,14 @@ def location_show(lib, argv, modifiers):
     all_loc_constraints = constraintsElement.getElementsByTagName(
         'rsc_location'
     )
+    cib = utils.get_cib()
 
-    print("Location Constraints:")
+    if not isfile(settings.crm_rule):
+        if verify_expiration:
+            sys.stderr.write(CRM_RULE_MISSING_MSG)
+        verify_expiration = False
+
+    all_lines.append("Location Constraints:")
     for rsc_loc in all_loc_constraints:
         if rsc_loc.hasAttribute("rsc-pattern"):
             lc_rsc_type = RESOURCE_TYPE_REGEXP
@@ -688,74 +688,86 @@ def location_show(lib, argv, modifiers):
             if valid_noderes:
                 if node not in valid_noderes:
                     continue
-            print("  Node: " + node)
+            all_lines.append("  Node: " + node)
 
             nodehash_label = (
                 (nodehashon, "    Allowed to run:"),
                 (nodehashoff, "    Not allowed to run:")
             )
-            for nodehash, label in nodehash_label:
-                if node in nodehash:
-                    print(label)
-                    for options in nodehash[node]:
-                        line_parts = [(
-                            "      " + options["rsc_label"]
-                            + " (" + options["id"] + ")"
-                        )]
-                        if options["role"]:
-                            line_parts.append(
-                                "(role: {0})".format(options["role"])
-                            )
-                        if options["resource-discovery"]:
-                            line_parts.append(
-                                "(resource-discovery={0})".format(
-                                    options["resource-discovery"]
-                                )
-                            )
-                        line_parts.append("Score: " + options["score"])
-                        print(" ".join(line_parts))
-        show_location_rules(ruleshash, showDetail)
+            all_lines += _hashtable_to_lines(
+                nodehash_label, "rsc_label", node, showDetail
+            )
+        all_lines += _show_location_rules(
+            ruleshash, cib, show_detail=showDetail, show_expired=show_expired,
+            verify_expiration=verify_expiration
+        )
     else:
         for rsc in rsclist:
+            rsc_lines = []
             if valid_noderes:
                 if rsc[0:2] not in valid_noderes:
                     continue
-            print("  {0}".format(rsc[2]))
+            rsc_lines.append("  {0}".format(rsc[2]))
             rschash_label = (
                 (rschashon, "    Enabled on:"),
                 (rschashoff, "    Disabled on:"),
             )
-            for rschash, label in rschash_label:
-                if rsc in rschash:
-                    for options in rschash[rsc]:
-                        if not options["node"]:
-                            continue
-                        line_parts = [
-                            label,
-                            options["node"],
-                            "(score:{0})".format(options["score"]),
-                        ]
-                        if options["role"]:
-                            line_parts.append(
-                                "(role: {0})".format(options["role"])
-                            )
-                        if options["resource-discovery"]:
-                            line_parts.append(
-                                "(resource-discovery={0})".format(
-                                    options["resource-discovery"]
-                                )
-                            )
-                        if showDetail:
-                            line_parts.append("(id:{0})".format(options["id"]))
-                        print(" ".join(line_parts))
+            rsc_lines += _hashtable_to_lines(
+                rschash_label, "node", rsc, showDetail
+            )
             miniruleshash = {}
             miniruleshash[rsc] = ruleshash[rsc]
-            show_location_rules(miniruleshash, showDetail, True)
+            rsc_lines += _show_location_rules(
+                miniruleshash,
+                cib,
+                show_detail=showDetail,
+                show_expired=show_expired,
+                verify_expiration=verify_expiration,
+                noheader=True,
+            )
+            # Append to all_lines only if the resource has any constraints
+            if len(rsc_lines) > 2:
+                all_lines += rsc_lines
+    return all_lines
 
-def show_location_rules(ruleshash, showDetail, noheader=False):
+def _hashtable_to_lines(hash_label, hash_type, hash_key, show_detail):
+    hash_lines = []
+    for hashtable, label in hash_label:
+        if hash_key in hashtable:
+            labeled_lines = []
+            for options in hashtable[hash_key]:
+                # Skips nodeless constraints and prints nodes/resources
+                if not options[hash_type]:
+                    continue
+                line_parts = ["      {0}{1}".format(
+                    "Node: " if hash_type == "node" else "",
+                    options[hash_type]
+                )]
+                line_parts.append(f"(score:{options['score']})")
+                if options["role"]:
+                    line_parts.append(f"(role:{options['role']})")
+                if options["resource-discovery"]:
+                    line_parts.append(
+                        "(resource-discovery={0})".format(
+                            options["resource-discovery"]
+                        )
+                    )
+                if show_detail:
+                    line_parts.append(f"(id:{options['id']})")
+                labeled_lines.append(" ".join(line_parts))
+            if labeled_lines:
+                labeled_lines.insert(0, label)
+            hash_lines += labeled_lines
+    return hash_lines
+
+def _show_location_rules(
+        ruleshash, cib, show_detail, show_expired=False, verify_expiration=True,
+        noheader=False
+):
     """
     Commandline options: no options
     """
+    all_lines = []
     constraint_options = {}
     for rsc in sorted(
         ruleshash.keys(),
@@ -769,7 +781,7 @@ def show_location_rules(ruleshash, showDetail, noheader=False):
     ):
         constrainthash = defaultdict(list)
         if not noheader:
-            print("  {0}".format(rsc[2]))
+            all_lines.append("  {0}".format(rsc[2]))
         for rule in ruleshash[rsc]:
             constraint_id = rule.parentNode.getAttribute("id")
             constrainthash[constraint_id].append(rule)
@@ -793,12 +805,66 @@ def show_location_rules(ruleshash, showDetail, noheader=False):
             else:
                 constraint_option_info = ""
 
-            print("    Constraint: " + constraint_id + constraint_option_info)
+            rule_lines = []
+            # When expiration check is needed, starting value should be True and
+            # when it's not, check is skipped so the initial value must be False
+            # to print the constraint
+            is_constraint_expired = verify_expiration
             for rule in constrainthash[constraint_id]:
-                print(rule_utils.ExportDetailed().get_string(
-                    rule, showDetail, "      "
+                rule_status = RULE_UNKNOWN_STATUS
+                if verify_expiration:
+                    rule_status = _get_rule_status(rule.getAttribute("id"), cib)
+                    if rule_status != RULE_EXPIRED:
+                        is_constraint_expired = False
+
+                rule_lines.append(rule_utils.ExportDetailed().get_string(
+                    rule,
+                    rule_status == RULE_EXPIRED and show_expired,
+                    show_detail,
+                    indent="      "
                 ))
 
+            if not show_expired and is_constraint_expired:
+                continue
+
+            all_lines.append(
+                "    Constraint{0}: {1}{2}".format(
+                    " (expired)" if is_constraint_expired else "",
+                    constraint_id,
+                    constraint_option_info
+                )
+            )
+            all_lines += rule_lines
+    return all_lines
+
+def _verify_node_name(node, existing_nodes):
+    report_list = []
+    if node not in existing_nodes:
+        report_list.append(reports.node_not_found(
+            node,
+            forceable=report_codes.FORCE_NODE_DOES_NOT_EXIST
+        ))
+    return report_list
+
+def _verify_score(score):
+    if not utils.is_score(score):
+        utils.err(
+            "invalid score '%s', use integer or INFINITY or -INFINITY"
+            % score
+        )
+
+def _get_rule_status(rule_id, cib):
+    _, _, retval = utils.cmd_runner().run(
+        [settings.crm_rule, "--check", "--rule=" + rule_id, "-X-"],
+        cib
+    )
+    translation_map = {
+        CrmRuleReturnCode.IN_EFFECT.value: RULE_IN_EFFECT,
+        CrmRuleReturnCode.EXPIRED.value: RULE_EXPIRED,
+        CrmRuleReturnCode.TO_BE_IN_EFFECT.value: RULE_NOT_IN_EFFECT,
+    }
+    return translation_map.get(retval, RULE_UNKNOWN_STATUS)
+
 def location_prefer(lib, argv, modifiers):
     """
     Options:
@@ -822,37 +888,57 @@ def location_prefer(lib, argv, modifiers):
     else:
         raise CmdLineInputError()
 
+    skip_node_check = False
+    if modifiers.is_specified("-f") or modifiers.get("--force"):
+        skip_node_check = True
+        warn(LOCATION_NODE_VALIDATION_SKIP_MSG)
+    else:
+        lib_env = utils.get_lib_env()
+        existing_nodes, report_list = get_existing_nodes_names(
+            corosync_conf=lib_env.get_corosync_conf(),
+            cib=lib_env.get_cib(),
+        )
+        if report_list:
+            process_library_reports(report_list)
 
+    report_list = []
+    parameters_list = []
     for nodeconf in argv:
         nodeconf_a = nodeconf.split("=", 1)
+        node = nodeconf_a[0]
+        if not skip_node_check:
+            report_list += _verify_node_name(node, existing_nodes)
         if len(nodeconf_a) == 1:
-            node = nodeconf_a[0]
             if prefer:
                 score = "INFINITY"
             else:
                 score = "-INFINITY"
         else:
             score = nodeconf_a[1]
-            if not utils.is_score(score):
-                utils.err(
-                    "invalid score '%s', use integer or INFINITY or -INFINITY"
-                    % score
-                )
+            _verify_score(score)
             if not prefer:
                 if score[0] == "-":
                     score = score[1:]
                 else:
                     score = "-" + score
-            node = nodeconf_a[0]
-        location_add(lib, [
-            sanitize_id("location-{0}-{1}-{2}".format(rsc_value, node, score)),
+
+        parameters_list.append([
+            sanitize_id(f"location-{rsc_value}-{node}-{score}"),
             rsc,
             node,
             score
-        ], modifiers.get_subset("--force", "-f"))
+        ])
+
+    if report_list:
+        process_library_reports(report_list)
+
+    modifiers = modifiers.get_subset("--force", "-f")
+
+    for parameters in parameters_list:
+        location_add(lib, parameters, modifiers, skip_score_and_node_check=True)
 
 
-def location_add(lib, argv, modifiers):
+def location_add(lib, argv, modifiers, skip_score_and_node_check=False):
     """
     Options:
       * --force - allow unknown options, allow constraint for any resource type
@@ -888,15 +974,25 @@ def location_add(lib, argv, modifiers):
                     "bad option '%s', use --force to override" % options[-1][0]
                 )
 
+    # Verify that specified node exists in the cluster and score is valid
+    if not skip_score_and_node_check:
+        if modifiers.is_specified("-f") or modifiers.get("--force"):
+            warn(LOCATION_NODE_VALIDATION_SKIP_MSG)
+        else:
+            lib_env = utils.get_lib_env()
+            existing_nodes, report_list = get_existing_nodes_names(
+                corosync_conf=lib_env.get_corosync_conf(),
+                cib=lib_env.get_cib(),
+            )
+            report_list = _verify_node_name(node, existing_nodes)
+            if report_list:
+                process_library_reports(report_list)
+        _verify_score(score)
+
     id_valid, id_error = utils.validate_xml_id(constraint_id, 'constraint id')
     if not id_valid:
         utils.err(id_error)
 
-    if not utils.is_score(score):
-        utils.err(
-            "invalid score '%s', use integer or INFINITY or -INFINITY" % score
-        )
-
     required_version = None
     if [x for x in options if x[0] == "resource-discovery"]:
         required_version = 2, 2, 0
@@ -915,22 +1011,6 @@ def location_add(lib, argv, modifiers):
         if not rsc_valid:
             utils.err(rsc_error)
 
-    # Verify that specified node exists in the cluster
-    if not (modifiers.is_specified("-f") or modifiers.get("--force")):
-        lib_env = utils.get_lib_env()
-        existing_nodes = get_existing_nodes_names(
-            corosync_conf=lib_env.get_corosync_conf(),
-            cib=lib_env.get_cib(),
-        )
-        if node not in existing_nodes:
-            raise error(
-                f"Node '{node}' does not seem to be in the cluster"
-                ", use --force to override"
-            )
-    else:
-        warn(LOCATION_NODE_VALIDATION_SKIP_MSG)
-
-
     # Verify current constraint doesn't already exist
     # If it does we replace it with the new constraint
     dummy_dom, constraintsElement = getCurrentConstraints(dom)
@@ -1108,7 +1188,7 @@ def location_rule_check_duplicates(dom, constraint_el, force):
                 lines.append("  Constraint: %s" % dup.getAttribute("id"))
                 for dup_rule in utils.dom_get_children_by_tag_name(dup, "rule"):
                     lines.append(rule_utils.ExportDetailed().get_string(
-                        dup_rule, True, "    "
+                        dup_rule, False, True, indent="    "
                     ))
             utils.err(
                 "duplicate constraint already exists, use --force to override\n"
@@ -1452,14 +1532,13 @@ def constraint_rule(lib, argv, modifiers):
                         print("Removing Rule: {0}".format(rule.get("id")))
                         loc_con.remove(rule)
                         found = True
-                        break
                     else:
                         print(
                             "Removing Constraint: {0}".format(loc_con.get("id"))
                         )
                         constraints.remove(loc_con)
                         found = True
-                        break
+                    break
 
             if found:
                 break
