@@ -6,6 +6,7 @@ from pcs.lib import reports
 from pcs.lib.cib.constraint import constraint
 from pcs.lib.cib import tools
 from pcs.lib.errors import LibraryError
+from pcs.lib.pacemaker.values import RESOURCE_ROLES
 from pcs.lib.xml_tools import remove_when_pointless
 
 TAG_NAME = 'rsc_ticket'
@@ -16,7 +17,7 @@ ATTRIB = {
 }
 ATTRIB_PLAIN = {
     "rsc": None,
-    "rsc-role": ("Stopped", "Started", "Master", "Slave"),
+    "rsc-role": RESOURCE_ROLES,
 }
 
 def _validate_options_common(options):
@@ -48,7 +49,7 @@ def prepare_options_with_set(cib, options, resource_set_list):
     )
     report = _validate_options_common(options)
     if "ticket" not in options or not options["ticket"].strip():
-        report.append(reports.required_option_is_missing(['ticket']))
+        report.append(reports.required_options_are_missing(['ticket']))
     if report:
         raise LibraryError(*report)
     return options
@@ -59,11 +60,11 @@ def prepare_options_plain(cib, options, ticket, resource_id):
     report = _validate_options_common(options)
 
     if not ticket:
-        report.append(reports.required_option_is_missing(['ticket']))
+        report.append(reports.required_options_are_missing(['ticket']))
     options["ticket"] = ticket
 
     if not resource_id:
-        report.append(reports.required_option_is_missing(['rsc']))
+        report.append(reports.required_options_are_missing(['rsc']))
     options["rsc"] = resource_id
 
     if "rsc-role" in options:
