@@ -5,26 +5,27 @@ from pcs.lib import reports, validate
 from pcs.lib.cib.nvpair import (
     append_new_meta_attributes,
     arrange_first_meta_attributes,
+    META_ATTRIBUTES_TAG,
 )
 from pcs.lib.cib.resource.primitive import TAG as TAG_PRIMITIVE
-from pcs.lib.cib.tools import find_element_by_tag_and_id
+from pcs.lib.cib.tools import ElementSearcher
 from pcs.lib.errors import (
     LibraryError,
     ReportItemSeverity,
-    ReportListAnalyzer,
 )
 from pcs.lib.pacemaker.values import sanitize_id
 from pcs.lib.xml_tools import (
+    append_when_useful,
     get_sub_element,
     update_attributes_remove_empty,
-    remove_when_pointless,
+    reset_element,
 )
 
 TAG = "bundle"
 
 GENERIC_CONTAINER_TYPES = {"docker", "podman", "rkt"}
 
-_generic_container_options = set((
+GENERIC_CONTAINER_OPTIONS = frozenset((
     "image",
     "masters",
     "network",
@@ -35,13 +36,28 @@ _generic_container_options = set((
     "replicas-per-host",
 ))
 
-_network_options = set((
+NETWORK_OPTIONS = frozenset((
     "control-port",
     "host-interface",
     "host-netmask",
     "ip-range-start",
 ))
 
+PORT_MAP_OPTIONS = frozenset((
+    "id",
+    "port",
+    "internal-port",
+    "range",
+))
+
+STORAGE_MAP_OPTIONS = frozenset((
+    "id",
+    "options",
+    "source-dir",
+    "source-dir-root",
+    "target-dir",
+))
+
 def is_bundle(resource_el):
     return resource_el.tag == TAG
 
@@ -61,41 +77,23 @@ def validate_new(
     list of dict storage_map -- list of storage mapping options
     bool force_options -- return warnings instead of forceable errors
     """
-    report_list = []
-
-    report_list.extend(
-        validate.run_collection_of_option_validators(
-            {"id": bundle_id},
-            [
-                # with id_provider it validates that the id is available as well
-                validate.value_id("id", "bundle name", id_provider),
-            ]
-        )
-    )
-
-    aux_reports = _validate_container_type(container_type)
-    report_list.extend(aux_reports)
-    if not ReportListAnalyzer(aux_reports).error_list:
-        report_list.extend(
-            # TODO call the proper function once more container_types are
-            # supported by pacemaker
-            _validate_generic_container_options_new(
-                container_options,
-                force_options
-            )
-        )
-    report_list.extend(
+    return (
+        validate.ValueId(
+            "id",
+            option_name_for_report="bundle name",
+            # with id_provider it validates that the id is available as well
+            id_provider=id_provider
+        ).validate({"id": bundle_id})
+        +
+        _validate_container(container_type, container_options, force_options)
+        +
         _validate_network_options_new(network_options, force_options)
-    )
-    report_list.extend(
+        +
         _validate_port_map_list(port_map, id_provider, force_options)
-    )
-    report_list.extend(
+        +
         _validate_storage_map_list(storage_map, id_provider, force_options)
     )
 
-    return report_list
-
 def append_new(
     parent_element, id_provider, bundle_id, container_type, container_options,
     network_options, port_map, storage_map, meta_attributes
@@ -115,31 +113,87 @@ def append_new(
     dict meta_attributes -- meta attributes
     """
     bundle_element = etree.SubElement(parent_element, TAG, {"id": bundle_id})
-    # TODO create the proper element once more container_types are supported
-    # by pacemaker
-    container_element = etree.SubElement(bundle_element, container_type)
-    # Do not add options with empty values. When updating, an empty value means
-    # remove the option.
-    update_attributes_remove_empty(container_element, container_options)
+    _append_container(bundle_element, container_type, container_options)
     if network_options or port_map:
-        network_element = etree.SubElement(bundle_element, "network")
-        # Do not add options with empty values. When updating, an empty value
-        # means remove the option.
-        update_attributes_remove_empty(network_element, network_options)
-    for port_map_options in port_map:
-        _append_port_map(
-            network_element, id_provider, bundle_id, port_map_options
+        _append_network(
+            bundle_element,
+            id_provider,
+            bundle_id,
+            network_options,
+            port_map,
         )
     if storage_map:
-        storage_element = etree.SubElement(bundle_element, "storage")
-    for storage_map_options in storage_map:
-        _append_storage_map(
-            storage_element, id_provider, bundle_id, storage_map_options
-        )
+        _append_storage(bundle_element, id_provider, bundle_id, storage_map)
     if meta_attributes:
         append_new_meta_attributes(bundle_element, meta_attributes, id_provider)
     return bundle_element
 
+def validate_reset(
+    id_provider, bundle_el, container_options, network_options, port_map,
+    storage_map, force_options=False
+):
+    """
+    Validate bundle parameters, return list of report items
+
+    IdProvider id_provider -- elements' ids generator and uniqueness checker
+    etree bundle_el -- the bundle to be reset
+    dict container_options -- container options
+    dict network_options -- network options
+    list of dict port_map -- list of port mapping options
+    list of dict storage_map -- list of storage mapping options
+    bool force_options -- return warnings instead of forceable errors
+    """
+    return (
+        _validate_container_reset(bundle_el, container_options, force_options)
+        +
+        _validate_network_options_new(network_options, force_options)
+        +
+        _validate_port_map_list(port_map, id_provider, force_options)
+        +
+        _validate_storage_map_list(storage_map, id_provider, force_options)
+    )
+
+def validate_reset_to_minimal(bundle_element):
+    """
+    Validate removing configuration of bundle_element and keep the minimal one.
+
+    etree bundle_element -- the bundle element that will be reset
+    """
+    if not _is_supported_container(_get_container_element(bundle_element)):
+        return [_get_report_unsupported_container(bundle_element)]
+    return []
+
+def reset_to_minimal(bundle_element):
+    """
+    Remove configuration of bundle_element and keep the minimal one.
+
+    etree bundle_element -- the bundle element that will be reset
+    """
+    # Elements network, storage and meta_attributes must be kept even if they
+    # are without children.
+    # See https://bugzilla.redhat.com/show_bug.cgi?id=1642514
+    # Element of container type is required.
+
+    # There can be other elements beside bundle configuration (e.g. primitive).
+    # These elements stay untouched.
+    # Like any function that manipulates with cib, this also assumes prior
+    # validation that container is supported.
+    for child in list(bundle_element):
+        if child.tag in ["network", "storage"]:
+            reset_element(child)
+        if child.tag == META_ATTRIBUTES_TAG:
+            reset_element(child, keep_attrs=["id"])
+        if child.tag in list(GENERIC_CONTAINER_TYPES):
+            # GENERIC_CONTAINER_TYPES elements require the "image" attribute to
+            # be set.
+            reset_element(child, keep_attrs=["image"])
+
+def _get_report_unsupported_container(bundle_el):
+    return reports.resource_bundle_unsupported_container_type(
+        bundle_el.get("id"),
+        GENERIC_CONTAINER_TYPES,
+    )
+
 def validate_update(
     id_provider, bundle_el, container_options, network_options,
     port_map_add, port_map_remove, storage_map_add, storage_map_remove,
@@ -159,72 +213,33 @@ def validate_update(
     list of string storage_map_remove -- list of storage mapping ids to remove
     bool force_options -- return warnings instead of forceable errors
     """
-    report_list = []
-
-    # validate container options only if they are being updated
-    if container_options:
-        container_el = _get_container_element(bundle_el)
-        if (
-            container_el is not None
-            and
-            container_el.tag in GENERIC_CONTAINER_TYPES
-        ):
-            report_list.extend(
-                _validate_generic_container_options_update(
-                    container_el,
-                    container_options,
-                    force_options
-                )
-            )
-        else:
-            report_list.append(
-                reports.resource_bundle_unsupported_container_type(
-                    bundle_el.get("id"), GENERIC_CONTAINER_TYPES
-                )
-            )
-
-    network_el = bundle_el.find("network")
-    if network_el is None:
-        report_list.extend(
-            _validate_network_options_new(network_options, force_options)
-        )
-    else:
-        report_list.extend(
-            _validate_network_options_update(
-                bundle_el,
-                network_el,
-                network_options,
-                force_options
-            )
-        )
-
     # TODO It will probably be needed to split the following validators to
     # create and update variants. It should be done once the need exists and
     # not sooner.
-    report_list.extend(
+    return (
+        _validate_container_update(bundle_el, container_options, force_options)
+        +
+        _validate_network_update(bundle_el, network_options, force_options)
+        +
         _validate_port_map_list(port_map_add, id_provider, force_options)
-    )
-    report_list.extend(
+        +
         _validate_storage_map_list(storage_map_add, id_provider, force_options)
-    )
-    report_list.extend(
+        +
         _validate_map_ids_exist(
             bundle_el, "port-mapping", "port-map", port_map_remove
         )
-    )
-    report_list.extend(
+        +
         _validate_map_ids_exist(
             bundle_el, "storage-mapping", "storage-map", storage_map_remove
         )
     )
-    return report_list
 
 def update(
     id_provider, bundle_el, container_options, network_options,
     port_map_add, port_map_remove, storage_map_add, storage_map_remove,
     meta_attributes
 ):
-    # pylint: disable=too-many-arguments
+    # pylint: disable=too-many-arguments, too-many-locals
     """
     Modify an existing bundle (does not touch encapsulated resources)
 
@@ -238,13 +253,24 @@ def update(
     list of string storage_map_remove -- list of storage mapping ids to remove
     dict meta_attributes -- meta attributes to update
     """
+    # Do not ever remove meta_attributes, network and storage elements, even if
+    # they are empty. There may be ACLs set in pacemaker which allow "write"
+    # for their children (adding, changing and removing) but not themselves. In
+    # such a case, removing those elements would cause the whole change to be
+    # rejected by pacemaker with a "permission denied" message.
+    # https://bugzilla.redhat.com/show_bug.cgi?id=1642514
+
     bundle_id = bundle_el.get("id")
     update_attributes_remove_empty(
         _get_container_element(bundle_el),
         container_options
     )
 
-    network_element = get_sub_element(bundle_el, "network")
+    network_element = get_sub_element(
+        bundle_el,
+        "network",
+        append_if_missing=False
+    )
     if network_options:
         update_attributes_remove_empty(network_element, network_options)
     # It's crucial to remove port maps prior to appending new ones: If we are
@@ -257,30 +283,37 @@ def update(
             network_element.findall("port-mapping"),
             port_map_remove
         )
-    for port_map_options in port_map_add:
-        _append_port_map(
-            network_element, id_provider, bundle_id, port_map_options
-        )
+    if port_map_add:
+        for port_map_options in port_map_add:
+            _append_port_map(
+                network_element, id_provider, bundle_id, port_map_options
+            )
+    append_when_useful(bundle_el, network_element)
 
-    storage_element = get_sub_element(bundle_el, "storage")
+    storage_element = get_sub_element(
+        bundle_el,
+        "storage",
+        append_if_missing=False
+    )
     # See the comment above about removing port maps prior to adding new ones.
     if storage_map_remove:
         _remove_map_elements(
             storage_element.findall("storage-mapping"),
             storage_map_remove
         )
-    for storage_map_options in storage_map_add:
-        _append_storage_map(
-            storage_element, id_provider, bundle_id, storage_map_options
-        )
+    if storage_map_add:
+        for storage_map_options in storage_map_add:
+            _append_storage_map(
+                storage_element, id_provider, bundle_id, storage_map_options
+            )
+    append_when_useful(bundle_el, storage_element)
 
     if meta_attributes:
-        arrange_first_meta_attributes(bundle_el, meta_attributes)
-
-    # remove empty elements with no attributes
-    # meta attributes are handled in their own function
-    remove_when_pointless(network_element)
-    remove_when_pointless(storage_element)
+        arrange_first_meta_attributes(
+            bundle_el,
+            meta_attributes,
+            id_provider
+        )
 
 def is_pcmk_remote_accessible(bundle_element):
     """
@@ -321,72 +354,105 @@ def get_inner_resource(bundle_el):
         return resources[0]
     return None
 
-def _validate_container_type(container_type):
-    return validate.value_in(
-        "type",
-        GENERIC_CONTAINER_TYPES,
-        "container type"
-    )(
-        {
-            "type": container_type,
-        }
+def _is_supported_container(container_el):
+    return (
+        container_el is not None
+        and
+        container_el.tag in GENERIC_CONTAINER_TYPES
     )
 
-def _validate_generic_container_options_new(options, force_options):
+def _validate_container(container_type, container_options, force_options=False):
+    if container_type not in GENERIC_CONTAINER_TYPES:
+        return [
+            reports.invalid_option_value(
+                "container type",
+                container_type,
+                GENERIC_CONTAINER_TYPES,
+            )
+        ]
+    return _validate_generic_container_options(container_options, force_options)
+
+
+def _validate_generic_container_options(container_options, force_options=False):
     validators = [
-        validate.is_required("image", "container"),
-        validate.value_not_empty("image", "image name"),
-        validate.value_nonnegative_integer("masters"),
-        validate.value_nonnegative_integer("promoted-max"),
-        validate.mutually_exclusive(["masters", "promoted-max"], "container"),
-        validate.value_positive_integer("replicas"),
-        validate.value_positive_integer("replicas-per-host"),
+        validate.NamesIn(
+            GENERIC_CONTAINER_OPTIONS,
+            option_type="container",
+            **validate.set_warning(report_codes.FORCE_OPTIONS, force_options)
+        ),
+        validate.IsRequiredAll(["image"], option_type="container"),
+        validate.ValueNotEmpty("image", "image name"),
+        validate.ValueNonnegativeInteger("masters"),
+        validate.ValueNonnegativeInteger("promoted-max"),
+        validate.MutuallyExclusive(
+            ["masters", "promoted-max"],
+            option_type="container",
+        ),
+        validate.ValuePositiveInteger("replicas"),
+        validate.ValuePositiveInteger("replicas-per-host"),
     ]
+
     deprecation_reports = []
-    if "masters" in options:
+    if "masters" in container_options:
         deprecation_reports.append(
             reports.deprecated_option(
                 "masters", ["promoted-max"], "container",
                 severity=ReportItemSeverity.WARNING
             )
         )
+
     return (
-        validate.run_collection_of_option_validators(options, validators)
+        validate.ValidatorAll(validators).validate(container_options)
         +
         deprecation_reports
-        +
-        validate.names_in(
-            _generic_container_options,
-            options.keys(),
-            "container",
-            report_codes.FORCE_OPTIONS,
-            force_options
-        )
+    )
+
+def _validate_container_reset(bundle_el, container_options, force_options):
+    # Unlike in the case of update, in reset empty options are not necessary
+    # valid - user MUST set everything (including required options e.g. image).
+    if (
+        container_options
+        and
+        not _is_supported_container(_get_container_element(bundle_el))
+    ):
+        return [_get_report_unsupported_container(bundle_el)]
+    return _validate_generic_container_options(container_options, force_options)
+
+def _validate_container_update(bundle_el, options, force_options):
+    # Validate container options only if they are being updated. Empty options
+    # are valid - user DOESN'T NEED to change anything.
+    if not options:
+        return []
+
+    container_el = _get_container_element(bundle_el)
+    if not _is_supported_container(container_el):
+        return [_get_report_unsupported_container(bundle_el)]
+    return _validate_generic_container_options_update(
+        container_el, options, force_options
     )
 
 def _validate_generic_container_options_update(
-    docker_el, options, force_options
+    container_el, options, force_options
 ):
-    validators = [
-        # image is a mandatory attribute and cannot be removed
-        validate.value_not_empty("image", "image name"),
-        validate.value_empty_or_valid(
-            "masters",
-            validate.value_nonnegative_integer("masters")
-        ),
-        validate.value_empty_or_valid(
-            "promoted-max",
-            validate.value_nonnegative_integer("promoted-max")
-        ),
-        validate.value_empty_or_valid(
-            "replicas",
-            validate.value_positive_integer("replicas")
-        ),
-        validate.value_empty_or_valid(
-            "replicas-per-host",
-            validate.value_positive_integer("replicas-per-host")
-        ),
+    validators_optional_options = [
+        validate.ValueNonnegativeInteger("masters"),
+        validate.ValueNonnegativeInteger("promoted-max"),
+        validate.ValuePositiveInteger("replicas"),
+        validate.ValuePositiveInteger("replicas-per-host"),
     ]
+    for val in validators_optional_options:
+        val.empty_string_valid = True
+    validators = [
+        validate.NamesIn(
+            # allow to remove options even if they are not allowed
+            GENERIC_CONTAINER_OPTIONS | _options_to_remove(options),
+            option_type="container",
+            **validate.set_warning(report_codes.FORCE_OPTIONS, force_options)
+        ),
+        # image is a mandatory attribute and cannot be removed
+        validate.ValueNotEmpty("image", "image name")
+    ] + validators_optional_options
+
     # CIB does not allow both to be set. Deleting both is not a problem,
     # though. Deleting one while setting another also works and is further
     # checked bellow.
@@ -396,9 +462,9 @@ def _validate_generic_container_options_update(
         options.get("promoted-max", "") == ""
     ):
         validators.append(
-            validate.mutually_exclusive(
+            validate.MutuallyExclusive(
                 ["masters", "promoted-max"],
-                "container"
+                option_type="container",
             )
         )
 
@@ -419,7 +485,7 @@ def _validate_generic_container_options_update(
     if (
         options.get("masters")
         and
-        docker_el.get("promoted-max") and options.get("promoted-max") != ""
+        container_el.get("promoted-max") and options.get("promoted-max") != ""
     ):
         deprecation_reports.append(
             reports.prerequisite_option_must_not_be_set(
@@ -429,7 +495,7 @@ def _validate_generic_container_options_update(
     if (
         options.get("promoted-max")
         and
-        docker_el.get("masters") and options.get("masters") != ""
+        container_el.get("masters") and options.get("masters") != ""
     ):
         deprecation_reports.append(
             reports.prerequisite_option_must_not_be_set(
@@ -438,37 +504,22 @@ def _validate_generic_container_options_update(
         )
 
     return (
-        validate.run_collection_of_option_validators(options, validators)
+        validate.ValidatorAll(validators).validate(options)
         +
         deprecation_reports
-        +
-        validate.names_in(
-            # allow to remove options even if they are not allowed
-            _generic_container_options | _options_to_remove(options),
-            options.keys(),
-            "container",
-            report_codes.FORCE_OPTIONS,
-            force_options
-        )
     )
 
 def _validate_network_options_new(options, force_options):
+    kwargs = validate.set_warning(report_codes.FORCE_OPTIONS, force_options)
     validators = [
         # TODO add validators for other keys (ip-range-start - IPv4)
-        validate.value_port_number("control-port"),
-        _value_host_netmask("host-netmask", force_options),
+        validate.NamesIn(NETWORK_OPTIONS, option_type="network", **kwargs),
+        validate.ValuePortNumber("control-port"),
+        # Leaving a possibility to force this validation for the case pacemaker
+        # starts supporting IPv6 or other format of the netmask.
+        ValueHostNetmask("host-netmask", **kwargs),
     ]
-    return (
-        validate.run_collection_of_option_validators(options, validators)
-        +
-        validate.names_in(
-            _network_options,
-            options.keys(),
-            "network",
-            report_codes.FORCE_OPTIONS,
-            force_options
-        )
-    )
+    return validate.ValidatorAll(validators).validate(options)
 
 def _is_pcmk_remote_acccessible_after_update(network_el, options):
     port_name = "control-port"
@@ -488,6 +539,14 @@ def _is_pcmk_remote_acccessible_after_update(network_el, options):
 
     return not (case1 or case2 or case3)
 
+def _validate_network_update(bundle_el, options, force_options):
+    network_el = bundle_el.find("network")
+    if network_el is None:
+        return _validate_network_options_new(options, force_options)
+    return _validate_network_options_update(
+        bundle_el, network_el, options, force_options
+    )
+
 def _validate_network_options_update(
     bundle_el, network_el, options, force_options
 ):
@@ -507,121 +566,123 @@ def _validate_network_options_update(
                 inner_primitive.get("id")
             )
         )
-    validators = [
+
+    kwargs = validate.set_warning(report_codes.FORCE_OPTIONS, force_options)
+    validators_optional_options = [
         # TODO add validators for other keys (ip-range-start - IPv4)
-        validate.value_empty_or_valid(
-            "control-port",
-            validate.value_port_number("control-port"),
-        ),
-        validate.value_empty_or_valid(
-            "host-netmask",
-            _value_host_netmask("host-netmask", force_options),
-        ),
+        validate.ValuePortNumber("control-port"),
+        # Leaving a possibility to force this validation for the case pacemaker
+        # starts supporting IPv6 or other format of the netmask.
+        ValueHostNetmask("host-netmask", **kwargs),
     ]
+    for val in validators_optional_options:
+        val.empty_string_valid = True
+    validators = [
+        validate.NamesIn(
+            # allow to remove options even if they are not allowed
+            NETWORK_OPTIONS | _options_to_remove(options),
+            option_type="network",
+            **kwargs
+        )
+    ] + validators_optional_options
+
     return (
         report_list
         +
-        validate.run_collection_of_option_validators(options, validators)
-        +
-        validate.names_in(
-            # allow to remove options even if they are not allowed
-            _network_options | _options_to_remove(options),
-            options.keys(),
-            "network",
-            report_codes.FORCE_OPTIONS,
-            force_options
-        )
+        validate.ValidatorAll(validators).validate(options)
     )
 
 def _validate_port_map_list(options_list, id_provider, force_options):
-    allowed_options = [
-        "id",
-        "port",
-        "internal-port",
-        "range",
-    ]
+    kwargs = validate.set_warning(report_codes.FORCE_OPTIONS, force_options)
+    option_type = "port-map"
     validators = [
-        validate.value_id("id", "port-map id", id_provider),
-        validate.depends_on_option(
-            "internal-port", "port", "port-map", "port-map"
+        validate.NamesIn(PORT_MAP_OPTIONS, option_type=option_type, **kwargs),
+        validate.ValueId(
+            "id", option_name_for_report="port-map id", id_provider=id_provider
         ),
-        validate.is_required_some_of(["port", "range"], "port-map"),
-        validate.mutually_exclusive(["port", "range"], "port-map"),
-        validate.value_port_number("port"),
-        validate.value_port_number("internal-port"),
-        validate.value_port_range(
-            "range",
-            code_to_allow_extra_values=report_codes.FORCE_OPTIONS,
-            extra_values_allowed=force_options
+        validate.DependsOnOption(
+            ["internal-port"], "port",
+            option_type=option_type, prerequisite_type=option_type,
         ),
+        validate.IsRequiredSome(["port", "range"], option_type=option_type),
+        validate.MutuallyExclusive(["port", "range"], option_type=option_type),
+        validate.ValuePortNumber("port"),
+        validate.ValuePortNumber("internal-port"),
+        validate.ValuePortRange("range", **kwargs),
     ]
+    validator_all = validate.ValidatorAll(validators)
+
     report_list = []
     for options in options_list:
-        report_list.extend(
-            validate.run_collection_of_option_validators(options, validators)
-            +
-            validate.names_in(
-                allowed_options,
-                options.keys(),
-                "port-map",
-                report_codes.FORCE_OPTIONS,
-                force_options
-            )
-        )
+        report_list.extend(validator_all.validate(options))
     return report_list
 
 def _validate_storage_map_list(options_list, id_provider, force_options):
-    allowed_options = [
-        "id",
-        "options",
-        "source-dir",
-        "source-dir-root",
-        "target-dir",
-    ]
-    source_dir_options = ["source-dir", "source-dir-root"]
+    kwargs = validate.set_warning(report_codes.FORCE_OPTIONS, force_options)
+    option_type = "storage-map"
     validators = [
-        validate.value_id("id", "storage-map id", id_provider),
-        validate.is_required_some_of(source_dir_options, "storage-map"),
-        validate.mutually_exclusive(source_dir_options, "storage-map"),
-        validate.is_required("target-dir", "storage-map"),
+        validate.NamesIn(
+            STORAGE_MAP_OPTIONS, option_type=option_type, **kwargs
+        ),
+        validate.ValueId(
+            "id",
+            option_name_for_report="storage-map id",
+            id_provider=id_provider
+        ),
+        validate.IsRequiredSome(
+            ["source-dir", "source-dir-root"],
+            option_type=option_type,
+        ),
+        validate.MutuallyExclusive(
+            ["source-dir", "source-dir-root"],
+            option_type=option_type,
+        ),
+        validate.IsRequiredAll(["target-dir"], option_type=option_type),
     ]
+    validator_all = validate.ValidatorAll(validators)
+
     report_list = []
     for options in options_list:
-        report_list.extend(
-            validate.run_collection_of_option_validators(options, validators)
-            +
-            validate.names_in(
-                allowed_options,
-                options.keys(),
-                "storage-map",
-                report_codes.FORCE_OPTIONS,
-                force_options
-            )
-        )
+        report_list.extend(validator_all.validate(options))
     return report_list
 
 def _validate_map_ids_exist(bundle_el, map_type, map_label, id_list):
     report_list = []
     for _id in id_list:
-        try:
-            find_element_by_tag_and_id(
-                map_type, bundle_el, _id, id_types=[map_label]
-            )
-        except LibraryError as e:
-            report_list.extend(e.args)
+        searcher = ElementSearcher(
+            map_type, _id, bundle_el, element_type_desc=map_label
+        )
+        if not searcher.element_found():
+            report_list.extend(searcher.get_errors())
     return report_list
 
-def _value_host_netmask(option_name, force_options):
-    return validate.value_cond(
-        option_name,
-        lambda value: validate.is_integer(value, 1, 32),
-        "a number of bits of the mask (1-32)",
-        # Leaving a possibility to force this validation, if pacemaker
-        # starts supporting IPv6 or other format of the netmask
-        code_to_allow_extra_values=report_codes.FORCE_OPTIONS,
-        extra_values_allowed=force_options
+class ValueHostNetmask(validate.ValuePredicateBase):
+    def _is_valid(self, value):
+        return validate.is_integer(value, 1, 32)
+
+    def _get_allowed_values(self):
+        return "a number of bits of the mask (1..32)"
+
+def _append_container(bundle_element, container_type, container_options):
+    # Do not add options with empty values. When updating, an empty value means
+    # remove the option.
+    update_attributes_remove_empty(
+        etree.SubElement(bundle_element, container_type),
+        container_options,
     )
 
+def _append_network(
+    bundle_element, id_provider, id_base, network_options, port_map
+):
+    network_element = etree.SubElement(bundle_element, "network")
+    # Do not add options with empty values. When updating, an empty value means
+    # remove the option.
+    update_attributes_remove_empty(network_element, network_options)
+    for port_map_options in port_map:
+        _append_port_map(
+            network_element, id_provider, id_base, port_map_options
+        )
+
 def _append_port_map(parent_element, id_provider, id_base, port_map_options):
     if "id" not in port_map_options:
         id_suffix = None
@@ -639,6 +700,16 @@ def _append_port_map(parent_element, id_provider, id_base, port_map_options):
     update_attributes_remove_empty(port_map_element, port_map_options)
     return port_map_element
 
+def _append_storage(bundle_element, id_provider, id_base, storage_map):
+    storage_element = etree.SubElement(bundle_element, "storage")
+    for storage_map_options in storage_map:
+        _append_storage_map(
+            storage_element,
+            id_provider,
+            id_base,
+            storage_map_options,
+        )
+
 def _append_storage_map(
     parent_element, id_provider, id_base, storage_map_options
 ):
