@@ -6,7 +6,6 @@ from pcs.lib.cib.nvpair import(
     arrange_first_meta_attributes,
     get_meta_attribute_value,
 )
-from pcs.lib.xml_tools import remove_when_pointless
 
 
 #TODO pcs currently does not care about multiple meta_attributes and here
@@ -53,37 +52,22 @@ def get_guest_option_value(options, default=None):
 def validate_set_as_guest(
     tree, existing_nodes_names, existing_nodes_addrs, node_name, options
 ):
-    report_list = validate.names_in(
-        GUEST_OPTIONS,
-        options.keys(),
-        "guest",
-    )
-
     validator_list = [
-        validate.value_time_interval("remote-connect-timeout"),
-        validate.value_port_number("remote-port"),
+        validate.NamesIn(GUEST_OPTIONS, option_type="guest"),
+        validate.ValueTimeInterval("remote-connect-timeout"),
+        validate.ValuePortNumber("remote-port"),
     ]
-    report_list.extend(
-        validate.run_collection_of_option_validators(options, validator_list)
-    )
-
-    report_list.extend(
+    return (
+        validate.ValidatorAll(validator_list).validate(options)
+        +
+        validate.ValueNotEmpty("node name", None)
+            .validate({"node name": node_name.strip()})
+        +
         validate_conflicts(
             tree, existing_nodes_names, existing_nodes_addrs, node_name, options
         )
     )
 
-    if not node_name.strip():
-        report_list.append(
-            reports.invalid_option_value(
-                "node name",
-                node_name,
-                "no empty value",
-            )
-        )
-
-    return report_list
-
 def is_guest_node(resource_element):
     """
     Return True if resource_element is already set as guest node.
@@ -106,7 +90,8 @@ def validate_is_not_guest(resource_element):
     ]
 
 def set_as_guest(
-    resource_element, node, addr=None, port=None, connect_timeout=None
+    resource_element, id_provider, node, addr=None, port=None,
+    connect_timeout=None
 ):
     """
     Set resource as guest node.
@@ -122,7 +107,7 @@ def set_as_guest(
     if connect_timeout:
         meta_options["remote-connect-timeout"] = str(connect_timeout)
 
-    arrange_first_meta_attributes(resource_element, meta_options)
+    arrange_first_meta_attributes(resource_element, meta_options, id_provider)
 
 def unset_guest(resource_element):
     """
@@ -130,6 +115,12 @@ def unset_guest(resource_element):
 
     etree.Element resource_element
     """
+    # Do not ever remove the nvset element, even if it is empty. There may be
+    # ACLs set in pacemaker which allow "write" for nvpairs (adding, changing
+    # and removing) but not nvsets. In such a case, removing the nvset would
+    # cause the whole change to be rejected by pacemaker with a "permission
+    # denied" message.
+    # https://bugzilla.redhat.com/show_bug.cgi?id=1642514
     guest_nvpair_list = resource_element.xpath(
         "./meta_attributes/nvpair[{0}]".format(
             " or ".join([
@@ -141,7 +132,6 @@ def unset_guest(resource_element):
     for nvpair in guest_nvpair_list:
         meta_attributes = nvpair.getparent()
         meta_attributes.remove(nvpair)
-        remove_when_pointless(meta_attributes)
 
 def get_node_name_from_options(meta_options, default=None):
     """
