@@ -14,6 +14,7 @@ from pcs.lib.errors import LibraryError
 from pcs.lib.pacemaker.values import (
     is_true,
     timeout_to_seconds,
+    RESOURCE_ROLES
 )
 
 OPERATION_NVPAIR_ATTRIBUTES = [
@@ -35,13 +36,6 @@ ATTRIBUTES = [
     "OCF_CHECK_LEVEL",
 ]
 
-ROLE_VALUES = [
-    "Stopped",
-    "Started",
-    "Slave",
-    "Master",
-]
-
 ON_FAIL_VALUES = [
     "ignore",
     "block",
@@ -119,49 +113,33 @@ def normalized_to_operations(normalized_pairs):
 def validate_operation_list(
     operation_list, allowed_operation_name_list, allow_invalid=False
 ):
-    options_validators = [
-        validate.is_required("name", "resource operation"),
-        validate.value_in("role", ROLE_VALUES),
-        validate.value_in("on-fail", ON_FAIL_VALUES),
-        validate.value_in("record-pending", BOOLEAN_VALUES),
-        validate.value_in("enabled", BOOLEAN_VALUES),
-        validate.mutually_exclusive(
-            ["interval-origin", "start-delay"],
-            "resource operation"
-        ),
-        validate.value_in(
+    kwargs = validate.set_warning(report_codes.FORCE_OPTIONS, allow_invalid)
+    option_type = "resource operation"
+
+    validators = [
+        validate.NamesIn(ATTRIBUTES, option_type=option_type),
+        validate.IsRequiredAll(["name"], option_type=option_type),
+        validate.ValueIn(
             "name",
             allowed_operation_name_list,
             option_name_for_report="operation name",
-            code_to_allow_extra_values=report_codes.FORCE_OPTIONS,
-            extra_values_allowed=allow_invalid,
+            **kwargs
         ),
-        validate.value_id("id", option_name_for_report="operation id"),
+        validate.ValueIn("role", RESOURCE_ROLES),
+        validate.ValueIn("on-fail", ON_FAIL_VALUES),
+        validate.ValueIn("record-pending", BOOLEAN_VALUES),
+        validate.ValueIn("enabled", BOOLEAN_VALUES),
+        validate.MutuallyExclusive(
+            ["interval-origin", "start-delay"],
+            option_type=option_type
+        ),
+        validate.ValueId("id", option_name_for_report="operation id"),
     ]
+    validator_all = validate.ValidatorAll(validators)
+
     report_list = []
     for operation in operation_list:
-        report_list.extend(
-            validate_operation(operation, options_validators)
-        )
-    return report_list
-
-def validate_operation(operation, options_validator_list):
-    """
-    Return a list with reports (ReportItems) about problems inside
-        operation.
-    dict operation contains attributes of operation
-    """
-    report_list = validate.names_in(
-        ATTRIBUTES,
-        operation.keys(),
-        "resource operation",
-    )
-
-    report_list.extend(validate.run_collection_of_option_validators(
-        operation,
-        options_validator_list
-    ))
-
+        report_list.extend(validator_all.validate(operation))
     return report_list
 
 def get_remaining_defaults(
@@ -264,7 +242,7 @@ def validate_different_intervals(operation_list):
         )]
     return []
 
-def create_id(context_element, name, interval):
+def create_id(context_element, id_provider, name, interval):
     """
     Create id for op element.
     etree context_element is used for the name building
@@ -273,23 +251,28 @@ def create_id(context_element, name, interval):
     """
     return create_subelement_id(
         context_element,
-        "{0}-interval-{1}".format(name, interval)
+        "{0}-interval-{1}".format(name, interval),
+        id_provider
     )
 
-def create_operations(primitive_element, operation_list):
+def create_operations(primitive_element, id_provider, operation_list):
     """
     Create operation element containing operations from operation_list
+
     list operation_list contains dictionaries with attributes of operation
+    IdProvider id_provider -- elements' ids generator
     etree primitive_element is context element
     """
     operations_element = etree.SubElement(primitive_element, "operations")
     for operation in sorted(operation_list, key=lambda op: op["name"]):
-        append_new_operation(operations_element, operation)
+        append_new_operation(operations_element, id_provider, operation)
 
-def append_new_operation(operations_element, options):
+def append_new_operation(operations_element, id_provider, options):
     """
     Create op element and apend it to operations_element.
+
     etree operations_element is the context element
+    IdProvider id_provider -- elements' ids generator
     dict options are attributes of operation
     """
     attribute_map = dict(
@@ -303,6 +286,7 @@ def append_new_operation(operations_element, options):
         attribute_map.update({
             "id": create_id(
                 operations_element.getparent(),
+                id_provider,
                 options["name"],
                 options["interval"]
             )
@@ -318,7 +302,9 @@ def append_new_operation(operations_element, options):
     )
 
     if nvpair_attribute_map:
-        append_new_instance_attributes(op_element, nvpair_attribute_map)
+        append_new_instance_attributes(
+            op_element, nvpair_attribute_map, id_provider
+        )
 
     return op_element
 
