@@ -9,6 +9,7 @@ from pcs.lib.xml_tools import get_sub_element
 
 CONFIGURATION = "configuration"
 CONSTRAINTS = "configuration/constraints"
+CRM_CONFIG = "configuration/crm_config"
 NODES = "configuration/nodes"
 RESOURCES = "configuration/resources"
 
@@ -21,6 +22,7 @@ RSC_DEFAULTS = "rsc_defaults"
 __MANDATORY_SECTIONS = [
     CONFIGURATION,
     CONSTRAINTS,
+    CRM_CONFIG,
     NODES,
     RESOURCES,
 ]
@@ -56,3 +58,8 @@ def get(tree, section_name):
         return get_sub_element(get(tree, CONFIGURATION), section_name)
 
     raise AssertionError("Unknown cib section '{0}'".format(section_name))
+
+def exists(tree, section_name):
+    if section_name not in __MANDATORY_SECTIONS + __OPTIONAL_SECTIONS:
+        raise AssertionError("Unknown cib section '{0}'".format(section_name))
+    return tree.find(".//{0}".format(section_name)) is not None
