@@ -1,9 +1,3 @@
-from __future__ import (
-    absolute_import,
-    division,
-    print_function,
-)
-
 from functools import partial
 
 from pcs.lib.cib.constraint import constraint, ticket
@@ -28,7 +22,6 @@ create_with_set = partial(
 
 def create(
     env, ticket_key, resource_id, options,
-    autocorrection_allowed=False,
     resource_in_clone_alowed=False,
     duplication_alowed=False,
 ):
@@ -49,8 +42,7 @@ def create(
         options,
         ticket_key,
         constraint.find_valid_resource_id(
-            env.report_processor, cib,
-            autocorrection_allowed, resource_in_clone_alowed, resource_id
+            env.report_processor, cib, resource_in_clone_alowed, resource_id
         ),
     )
 
@@ -71,8 +63,9 @@ def create(
 def remove(env, ticket_key, resource_id):
     """
     remove all ticket constraint from resource
-    If resource is in resource set with another resources then only resource ref
-    is removed. If resource is alone in resource set whole constraint is removed.
+    If resource is in resource set with another resources then only resource
+    ref is removed. If resource is alone in resource set whole constraint is
+    removed.
     """
     constraint_section = get_constraints(env.get_cib())
     any_plain_removed = ticket.remove_plain(
