@@ -1,10 +1,15 @@
 from pcs import settings
 from pcs.common import report_codes
+from pcs.common.file import RawFileError
 from pcs.common.reports import SimpleReportProcessor
 from pcs.lib import reports, node_communication_format
 from pcs.lib.tools import generate_binary_key
 from pcs.lib.cib.resource import guest_node, primitive, remote_node
-from pcs.lib.cib.tools import get_resources, find_element_by_tag_and_id
+from pcs.lib.cib.tools import (
+    IdProvider,
+    ElementSearcher,
+    get_resources,
+)
 from pcs.lib.communication.nodes import (
     DistributeFiles,
     GetHostInfo,
@@ -16,8 +21,10 @@ from pcs.lib.communication.tools import (
     run as run_com,
     run_and_raise,
 )
-from pcs.lib.env_tools import get_existing_nodes_names_addrs
+from pcs.lib.node import get_existing_nodes_names_addrs
 from pcs.lib.errors import LibraryError
+from pcs.lib.file.instance import FileInstance
+from pcs.lib.file.raw_file import raw_file_error_report
 from pcs.lib.pacemaker import state
 from pcs.lib.pacemaker.live import remove_node
 
@@ -133,14 +140,23 @@ def _prepare_pacemaker_remote_environment(
         )
 
     # share pacemaker authkey
-    if env.pacemaker.has_authkey:
-        authkey_content = env.pacemaker.get_authkey_content()
-        authkey_targets = online_new_target_list
-    else:
-        authkey_content = generate_binary_key(
-            random_bytes_count=settings.pacemaker_authkey_bytes
-        )
-        authkey_targets = existing_nodes_target_list + online_new_target_list
+    authkey_file = FileInstance.for_pacemaker_key()
+    try:
+        if authkey_file.raw_file.exists():
+            authkey_content = authkey_file.read_raw()
+            authkey_targets = online_new_target_list
+        else:
+            authkey_content = generate_binary_key(
+                random_bytes_count=settings.pacemaker_authkey_bytes
+            )
+            authkey_targets = (
+                existing_nodes_target_list + online_new_target_list
+            )
+    except RawFileError as e:
+        report_processor.report(raw_file_error_report(e))
+    if report_processor.has_errors:
+        raise LibraryError()
+
     if authkey_targets:
         com_cmd = DistributeFiles(
             report_processor,
@@ -207,8 +223,8 @@ def node_add_remote(
     env.ensure_wait_satisfiable(wait)
 
     report_processor = SimpleReportProcessor(env.report_processor)
-    target_factory = env.get_node_target_factory()
     cib = env.get_cib()
+    id_provider = IdProvider(cib)
     if env.is_cib_live:
         corosync_conf = env.get_corosync_conf()
     else:
@@ -216,10 +232,14 @@ def node_add_remote(
         report_processor.report(
             reports.corosync_node_conflict_check_skipped("not_live_cib")
         )
-    existing_nodes_names, existing_nodes_addrs = get_existing_nodes_names_addrs(
-       corosync_conf,
-       cib
+    existing_nodes_names, existing_nodes_addrs, report_list = (
+        get_existing_nodes_names_addrs(corosync_conf, cib)
     )
+    if env.is_cib_live:
+        # We just reported corosync checks are going to be skipped so we
+        # shouldn't complain about errors related to corosync nodes
+        report_processor.report_list(report_list)
+
     resource_agent = remote_node.get_agent(
         env.report_processor,
         env.cmd_runner()
@@ -227,6 +247,7 @@ def node_add_remote(
 
     existing_target_list = []
     if env.is_cib_live:
+        target_factory = env.get_node_target_factory()
         existing_target_list, new_target_list = _get_targets_for_add(
             target_factory, report_processor, existing_nodes_names, [node_name],
             skip_offline_nodes
@@ -263,6 +284,7 @@ def node_add_remote(
             env.report_processor,
             resource_agent,
             get_resources(cib),
+            id_provider,
             node_addr,
             node_name,
             operations,
@@ -279,9 +301,16 @@ def node_add_remote(
         already_exists = []
         unified_report_list = []
         for report in report_list + list(e.args):
-            if report.code != report_codes.ID_ALREADY_EXISTS:
+            if report.code not in (
+                report_codes.ID_ALREADY_EXISTS,
+                report_codes.RESOURCE_INSTANCE_ATTR_VALUE_NOT_UNIQUE,
+            ):
                 unified_report_list.append(report)
-            elif report.info["id"] not in already_exists:
+            elif (
+                "id" in report.info
+                and
+                report.info["id"] not in already_exists
+            ):
                 unified_report_list.append(report)
                 already_exists.append(report.info["id"])
         report_list = unified_report_list
@@ -338,8 +367,8 @@ def node_add_guest(
     env.ensure_wait_satisfiable(wait)
 
     report_processor = SimpleReportProcessor(env.report_processor)
-    target_factory = env.get_node_target_factory()
     cib = env.get_cib()
+    id_provider = IdProvider(cib)
     if env.is_cib_live:
         corosync_conf = env.get_corosync_conf()
     else:
@@ -347,13 +376,17 @@ def node_add_guest(
         report_processor.report(
             reports.corosync_node_conflict_check_skipped("not_live_cib")
         )
-    existing_nodes_names, existing_nodes_addrs = get_existing_nodes_names_addrs(
-       corosync_conf,
-       cib
+    existing_nodes_names, existing_nodes_addrs, report_list = (
+        get_existing_nodes_names_addrs(corosync_conf, cib)
     )
+    if env.is_cib_live:
+        # We just reported corosync checks are going to be skipped so we
+        # shouldn't complain about errors related to corosync nodes
+        report_processor.report_list(report_list)
 
     existing_target_list = []
     if env.is_cib_live:
+        target_factory = env.get_node_target_factory()
         existing_target_list, new_target_list = _get_targets_for_add(
             target_factory, report_processor, existing_nodes_names, [node_name],
             skip_offline_nodes
@@ -384,15 +417,12 @@ def node_add_guest(
         node_name,
         options
     )
-    try:
-        resource_element = find_element_by_tag_and_id(
-            primitive.TAG,
-            get_resources(cib),
-            resource_id
-        )
+    searcher = ElementSearcher(primitive.TAG, resource_id, get_resources(cib))
+    if searcher.element_found():
+        resource_element = searcher.get_element()
         report_list.extend(guest_node.validate_is_not_guest(resource_element))
-    except LibraryError as e:
-        report_list.extend(e.args)
+    else:
+        report_list.extend(searcher.get_errors())
 
     report_processor.report_list(report_list)
     if report_processor.has_errors:
@@ -401,6 +431,7 @@ def node_add_guest(
     # everything validated, let's set it up
     guest_node.set_as_guest(
         resource_element,
+        id_provider,
         node_name,
         options.get("remote-addr", None),
         options.get("remote-port", None),
