@@ -1,35 +1,54 @@
 from contextlib import contextmanager
+from functools import partial
+from typing import (
+    Any,
+    Mapping,
+)
 
-from pcs.common import report_codes
+from pcs.common import file_type_codes, report_codes
 from pcs.common.tools import Version
 from pcs.lib import reports
 from pcs.lib.cib import (
     resource,
     status as cib_status,
 )
-from pcs.lib.cib.resource import operations, remote_node, guest_node
 from pcs.lib.cib.tools import (
     find_element_by_tag_and_id,
     get_resources,
     get_status,
     IdProvider,
 )
-from pcs.lib.env_tools import get_existing_nodes_names_addrs
-from pcs.lib.errors import LibraryError
-from pcs.lib.pacemaker.values import (
-    timeout_to_seconds,
-    validate_id,
+from pcs.lib.env import LibraryEnvironment
+from pcs.lib.node import get_existing_nodes_names_addrs
+from pcs.lib.errors import (
+    LibraryError,
+    ReportItemSeverity as severities,
+)
+from pcs.lib.pacemaker import simulate as simulate_tools
+from pcs.lib.pacemaker.live import (
+    has_resource_unmove_unban_expired_support,
+    resource_ban,
+    resource_move,
+    resource_unmove_unban,
+    simulate_cib,
+    wait_for_idle,
 )
 from pcs.lib.pacemaker.state import (
     ensure_resource_state,
+    get_resource_state,
     info_resource_state,
     is_resource_managed,
     ResourceNotFound,
 )
+from pcs.lib.pacemaker.values import (
+    timeout_to_seconds,
+    validate_id,
+)
 from pcs.lib.resource_agent import(
     find_valid_resource_agent_by_name as get_agent
 )
-from pcs.lib.validate import value_time_interval
+from pcs.lib.validate import ValueTimeInterval
+from pcs.lib.xml_tools import get_root
 
 @contextmanager
 def resource_environment(
@@ -62,7 +81,7 @@ def _validate_remote_connection(
     resource_agent, existing_nodes_addrs, resource_id, instance_attributes,
     allow_not_suitable_command
 ):
-    if resource_agent.get_name() != remote_node.AGENT_NAME.full_name:
+    if resource_agent.get_name() != resource.remote_node.AGENT_NAME.full_name:
         return []
 
     report_list = []
@@ -74,7 +93,7 @@ def _validate_remote_connection(
     )
 
     report_list.extend(
-        remote_node.validate_host_not_conflicts(
+        resource.remote_node.validate_host_not_conflicts(
             existing_nodes_addrs,
             resource_id,
             instance_attributes
@@ -86,14 +105,18 @@ def _validate_guest_change(
     tree, existing_nodes_names, existing_nodes_addrs, meta_attributes,
     allow_not_suitable_command, detect_remove=False
 ):
-    if not guest_node.is_node_name_in_options(meta_attributes):
+    if not resource.guest_node.is_node_name_in_options(meta_attributes):
         return []
 
-    node_name = guest_node.get_node_name_from_options(meta_attributes)
+    node_name = resource.guest_node.get_node_name_from_options(meta_attributes)
 
     report_list = []
     create_report = reports.use_command_node_add_guest
-    if detect_remove and not guest_node.get_guest_option_value(meta_attributes):
+    if (
+        detect_remove
+        and
+        not resource.guest_node.get_guest_option_value(meta_attributes)
+    ):
         create_report = reports.use_command_node_remove_guest
 
     report_list.append(
@@ -104,7 +127,7 @@ def _validate_guest_change(
     )
 
     report_list.extend(
-        guest_node.validate_conflicts(
+        resource.guest_node.validate_conflicts(
             tree,
             existing_nodes_names,
             existing_nodes_addrs,
@@ -134,20 +157,18 @@ def _check_special_cases(
     instance_attributes, allow_not_suitable_command
 ):
     if(
-        resource_agent.get_name() != remote_node.AGENT_NAME.full_name
+        resource_agent.get_name() != resource.remote_node.AGENT_NAME.full_name
         and
-        not guest_node.is_node_name_in_options(meta_attributes)
+        not resource.guest_node.is_node_name_in_options(meta_attributes)
     ):
         #if no special case happens we won't take care about corosync.conf that
         #is needed for getting nodes to validate against
         return
 
-    existing_nodes_names, existing_nodes_addrs = _get_nodes_to_validate_against(
-        env,
-        resources_section
+    existing_nodes_names, existing_nodes_addrs, report_list = (
+        _get_nodes_to_validate_against(env, resources_section)
     )
 
-    report_list = []
     report_list.extend(_validate_remote_connection(
         resource_agent,
         existing_nodes_addrs,
@@ -165,6 +186,22 @@ def _check_special_cases(
 
     env.report_processor.process_list(report_list)
 
+_find_bundle = partial(find_element_by_tag_and_id, resource.bundle.TAG)
+
+def _get_required_cib_version_for_container(
+    container_options, container_type=None
+):
+    if container_type == "podman":
+        return Version(3, 2, 0)
+
+    if "promoted-max" in container_options:
+        return Version(3, 0, 0)
+
+    if container_type == "rkt":
+        return Version(2, 10, 0)
+
+    return Version(2, 8, 0)
+
 def create(
     env, resource_id, resource_agent_name,
     operation_list, meta_attributes, instance_attributes,
@@ -217,6 +254,7 @@ def create(
             resource.common.are_meta_disabled(meta_attributes)
         )
     ) as resources_section:
+        id_provider = IdProvider(resources_section)
         _check_special_cases(
             env,
             resource_agent,
@@ -228,7 +266,7 @@ def create(
         )
 
         primitive_element = resource.primitive.create(
-            env.report_processor, resources_section,
+            env.report_processor, resources_section, id_provider,
             resource_id, resource_agent,
             operation_list, meta_attributes, instance_attributes,
             allow_invalid_operation,
@@ -236,7 +274,7 @@ def create(
             use_default_operations,
         )
         if ensure_disabled:
-            resource.common.disable(primitive_element)
+            resource.common.disable(primitive_element, id_provider)
 
 def create_as_clone(
     env, resource_id, resource_agent_name,
@@ -293,6 +331,7 @@ def create_as_clone(
             resource.common.is_clone_deactivated_by_meta(clone_meta_options)
         )
     ) as resources_section:
+        id_provider = IdProvider(resources_section)
         _check_special_cases(
             env,
             resource_agent,
@@ -304,7 +343,7 @@ def create_as_clone(
         )
 
         primitive_element = resource.primitive.create(
-            env.report_processor, resources_section,
+            env.report_processor, resources_section, id_provider,
             resource_id, resource_agent,
             operation_list, meta_attributes, instance_attributes,
             allow_invalid_operation,
@@ -313,11 +352,12 @@ def create_as_clone(
         )
         clone_element = resource.clone.append_new(
             resources_section,
+            id_provider,
             primitive_element,
             clone_meta_options,
         )
         if ensure_disabled:
-            resource.common.disable(clone_element)
+            resource.common.disable(clone_element, id_provider)
 
 def create_in_group(
     env, resource_id, resource_agent_name, group_id,
@@ -375,6 +415,7 @@ def create_in_group(
             resource.common.are_meta_disabled(meta_attributes)
         )
     ) as resources_section:
+        id_provider = IdProvider(resources_section)
         _check_special_cases(
             env,
             resource_agent,
@@ -386,7 +427,7 @@ def create_in_group(
         )
 
         primitive_element = resource.primitive.create(
-            env.report_processor, resources_section,
+            env.report_processor, resources_section, id_provider,
             resource_id, resource_agent,
             operation_list, meta_attributes, instance_attributes,
             allow_invalid_operation,
@@ -394,7 +435,7 @@ def create_in_group(
             use_default_operations,
         )
         if ensure_disabled:
-            resource.common.disable(primitive_element)
+            resource.common.disable(primitive_element, id_provider)
         validate_id(group_id, "group name")
         resource.group.place_resource(
             resource.group.provide_group(resources_section, group_id),
@@ -461,6 +502,7 @@ def create_into_bundle(
         ),
         required_cib_version=Version(2, 8, 0)
     ) as resources_section:
+        id_provider = IdProvider(resources_section)
         _check_special_cases(
             env,
             resource_agent,
@@ -472,7 +514,7 @@ def create_into_bundle(
         )
 
         primitive_element = resource.primitive.create(
-            env.report_processor, resources_section,
+            env.report_processor, resources_section, id_provider,
             resource_id, resource_agent,
             operation_list, meta_attributes, instance_attributes,
             allow_invalid_operation,
@@ -480,13 +522,9 @@ def create_into_bundle(
             use_default_operations,
         )
         if ensure_disabled:
-            resource.common.disable(primitive_element)
+            resource.common.disable(primitive_element, id_provider)
 
-        bundle_el = find_element_by_tag_and_id(
-            resource.bundle.TAG,
-            resources_section,
-            bundle_id
-        )
+        bundle_el = _find_bundle(resources_section, bundle_id)
         if not resource.bundle.is_pcmk_remote_accessible(bundle_el):
             env.report_processor.process(
                 reports.get_problem_creator(
@@ -529,13 +567,6 @@ def bundle_create(
     storage_map = storage_map or []
     meta_attributes = meta_attributes or {}
 
-    required_cib_version = Version(2, 8, 0)
-    if container_type == "rkt":
-        required_cib_version = Version(2, 10, 0)
-    if "promoted-max" in container_options:
-        required_cib_version = Version(3, 0, 0)
-    if container_type == "podman":
-        required_cib_version = Version(3, 2, 0)
     with resource_environment(
         env,
         wait,
@@ -545,7 +576,10 @@ def bundle_create(
             or
             resource.common.are_meta_disabled(meta_attributes)
         ),
-        required_cib_version=required_cib_version
+        required_cib_version=_get_required_cib_version_for_container(
+            container_options,
+            container_type,
+        ),
     ) as resources_section:
         # no need to run validations related to remote and guest nodes as those
         # nodes can only be created from primitive resources
@@ -575,7 +609,83 @@ def bundle_create(
             meta_attributes
         )
         if ensure_disabled:
-            resource.common.disable(bundle_element)
+            resource.common.disable(bundle_element, id_provider)
+
+def bundle_reset(
+    env, bundle_id, container_options=None,
+    network_options=None, port_map=None, storage_map=None, meta_attributes=None,
+    force_options=False,
+    ensure_disabled=False,
+    wait=False,
+):
+    # pylint: disable=too-many-arguments
+    """
+    Remove configuration of bundle bundle_id and create new one into it.
+
+    LibraryEnvironment env -- provides communication with externals
+    string bundle_id -- id of the bundle to reset
+    dict container_options -- container options
+    dict network_options -- network options
+    list of dict port_map -- a list of port mapping options
+    list of dict storage_map -- a list of storage mapping options
+    dict meta_attributes -- bundle's meta attributes
+    bool force_options -- return warnings instead of forceable errors
+    bool ensure_disabled -- set the bundle's target-role to "Stopped"
+    mixed wait -- False: no wait, None: wait default timeout, int: wait timeout
+    """
+    container_options = container_options or {}
+    network_options = network_options or {}
+    port_map = port_map or []
+    storage_map = storage_map or []
+    meta_attributes = meta_attributes or {}
+
+    with resource_environment(
+        env,
+        wait,
+        [bundle_id],
+        _ensure_disabled_after_wait(
+            ensure_disabled
+            or
+            resource.common.are_meta_disabled(meta_attributes)
+        ),
+        required_cib_version=_get_required_cib_version_for_container(
+            container_options
+        ),
+    ) as resources_section:
+        bundle_element = _find_bundle(resources_section, bundle_id)
+        env.report_processor.process_list(
+            resource.bundle.validate_reset_to_minimal(bundle_element)
+        )
+        resource.bundle.reset_to_minimal(bundle_element)
+
+        id_provider = IdProvider(resources_section)
+        env.report_processor.process_list(
+            resource.bundle.validate_reset(
+                id_provider,
+                bundle_element,
+                container_options,
+                network_options,
+                port_map,
+                storage_map,
+                # TODO meta attributes - there is no validation for now
+                force_options
+            )
+        )
+
+        resource.bundle.update(
+            id_provider,
+            bundle_element,
+            container_options,
+            network_options,
+            port_map_add=port_map,
+            port_map_remove=[],
+            storage_map_add=storage_map,
+            storage_map_remove=[],
+            meta_attributes=meta_attributes,
+        )
+
+        if ensure_disabled:
+            resource.common.disable(bundle_element, id_provider)
 
 def bundle_update(
     env, bundle_id, container_options=None, network_options=None,
@@ -608,23 +718,18 @@ def bundle_update(
     storage_map_remove = storage_map_remove or []
     meta_attributes = meta_attributes or {}
 
-    required_cib_version = Version(2, 8, 0)
-    if "promoted-max" in container_options:
-        required_cib_version = Version(3, 0, 0)
     with resource_environment(
         env,
         wait,
         [bundle_id],
-        required_cib_version=required_cib_version
+        required_cib_version=_get_required_cib_version_for_container(
+            container_options
+        ),
     ) as resources_section:
         # no need to run validations related to remote and guest nodes as those
         # nodes can only be created from primitive resources
         id_provider = IdProvider(resources_section)
-        bundle_element = find_element_by_tag_and_id(
-            resource.bundle.TAG,
-            resources_section,
-            bundle_id
-        )
+        bundle_element = _find_bundle(resources_section, bundle_id)
         env.report_processor.process_list(
             resource.bundle.validate_update(
                 id_provider,
@@ -651,9 +756,25 @@ def bundle_update(
             meta_attributes
         )
 
+def _disable_validate_and_edit_cib(env, resources_section, resource_ids):
+    id_provider = IdProvider(resources_section)
+    resource_el_list = _find_resources_or_raise(
+        resources_section,
+        resource_ids
+    )
+    env.report_processor.process_list(
+        _resource_list_enable_disable(
+            resource_el_list,
+            resource.common.disable,
+            id_provider,
+            env.get_cluster_state()
+        )
+    )
+
 def disable(env, resource_ids, wait):
     """
     Disallow specified resource to be started by the cluster
+
     LibraryEnvironment env --
     strings resource_ids -- ids of the resources to be disabled
     mixed wait -- False: no wait, None: wait default timeout, int: wait timeout
@@ -661,18 +782,83 @@ def disable(env, resource_ids, wait):
     with resource_environment(
         env, wait, resource_ids, _ensure_disabled_after_wait(True)
     ) as resources_section:
-        resource_el_list = _find_resources_or_raise(
-            resources_section,
-            resource_ids
+        _disable_validate_and_edit_cib(env, resources_section, resource_ids)
+
+def disable_safe(env, resource_ids, strict, wait):
+    """
+    Disallow specified resource to be started by the cluster only if there is
+    no effect on other resources
+
+    LibraryEnvironment env --
+    strings resource_ids -- ids of the resources to be disabled
+    bool strict -- if False, allow resources to be migrated
+    mixed wait -- False: no wait, None: wait default timeout, int: wait timeout
+    """
+    if not env.is_cib_live:
+        raise LibraryError(
+            reports.live_environment_required([file_type_codes.CIB])
         )
-        env.report_processor.process_list(
-            _resource_list_enable_disable(
-                resource_el_list,
-                resource.common.disable,
-                env.get_cluster_state()
+
+    with resource_environment(
+        env, wait, resource_ids, _ensure_disabled_after_wait(True)
+    ) as resources_section:
+        _disable_validate_and_edit_cib(env, resources_section, resource_ids)
+        plaintext_status, transitions, dummy_cib = simulate_cib(
+            env.cmd_runner(),
+            get_root(resources_section)
+        )
+        simulated_operations = (
+            simulate_tools.get_operations_from_transitions(transitions)
+        )
+        other_affected = set()
+        if strict:
+            other_affected = set(
+                simulate_tools.get_resources_from_operations(
+                    simulated_operations,
+                    exclude=resource_ids
+                )
             )
+        else:
+            other_affected = set(
+                simulate_tools.get_resources_left_stopped(
+                    simulated_operations,
+                    exclude=resource_ids
+                )
+                +
+                simulate_tools.get_resources_left_demoted(
+                    simulated_operations,
+                    exclude=resource_ids
+                )
+            )
+        if other_affected:
+            raise LibraryError(
+                reports.resource_disable_affects_other_resources(
+                    resource_ids,
+                    other_affected,
+                    plaintext_status,
+                )
+            )
+
+def disable_simulate(env, resource_ids):
+    """
+    Simulate disallowing specified resource to be started by the cluster
+
+    LibraryEnvironment env --
+    strings resource_ids -- ids of the resources to be disabled
+    """
+    if not env.is_cib_live:
+        raise LibraryError(
+            reports.live_environment_required([file_type_codes.CIB])
         )
 
+    resources_section = get_resources(env.get_cib())
+    _disable_validate_and_edit_cib(env, resources_section, resource_ids)
+    plaintext_status, dummy_transitions, dummy_cib = simulate_cib(
+        env.cmd_runner(),
+        get_root(resources_section)
+    )
+    return plaintext_status
+
 def enable(env, resource_ids, wait):
     """
     Allow specified resource to be started by the cluster
@@ -683,6 +869,7 @@ def enable(env, resource_ids, wait):
     with resource_environment(
         env, wait, resource_ids, _ensure_disabled_after_wait(False)
     ) as resources_section:
+        id_provider = IdProvider(resources_section)
         resource_el_list = _find_resources_or_raise(
             resources_section,
             resource_ids,
@@ -692,23 +879,26 @@ def enable(env, resource_ids, wait):
             _resource_list_enable_disable(
                 resource_el_list,
                 resource.common.enable,
+                id_provider,
                 env.get_cluster_state()
             )
         )
 
-def _resource_list_enable_disable(resource_el_list, func, cluster_state):
+def _resource_list_enable_disable(
+    resource_el_list, func, id_provider, cluster_state
+):
     report_list = []
     for resource_el in resource_el_list:
         res_id = resource_el.attrib["id"]
         try:
             if not is_resource_managed(cluster_state, res_id):
                 report_list.append(reports.resource_is_unmanaged(res_id))
-            func(resource_el)
+            func(resource_el, id_provider)
         except ResourceNotFound:
             report_list.append(
                 reports.id_not_found(
                     res_id,
-                    ["primitive", "clone", "group", "bundle"]
+                    ["primitive", "clone", "group", "bundle", "master"]
                )
             )
     return report_list
@@ -721,6 +911,7 @@ def unmanage(env, resource_ids, with_monitor=False):
     bool with_monitor -- disable resources' monitor operations
     """
     with resource_environment(env) as resources_section:
+        id_provider = IdProvider(resources_section)
         resource_el_list = _find_resources_or_raise(
             resources_section,
             resource_ids,
@@ -729,18 +920,18 @@ def unmanage(env, resource_ids, with_monitor=False):
         primitives = []
 
         for resource_el in resource_el_list:
-            resource.common.unmanage(resource_el)
+            resource.common.unmanage(resource_el, id_provider)
             if with_monitor:
                 primitives.extend(
                     resource.common.find_primitives(resource_el)
                 )
 
         for resource_el in set(primitives):
-            for op in operations.get_resource_operations(
+            for op in resource.operations.get_resource_operations(
                 resource_el,
                 ["monitor"]
             ):
-                operations.disable(op)
+                resource.operations.disable(op)
 
 def manage(env, resource_ids, with_monitor=False):
     """
@@ -750,6 +941,7 @@ def manage(env, resource_ids, with_monitor=False):
     bool with_monitor -- enable resources' monitor operations
     """
     with resource_environment(env) as resources_section:
+        id_provider = IdProvider(resources_section)
         report_list = []
         resource_el_list = _find_resources_or_raise(
             resources_section,
@@ -759,7 +951,7 @@ def manage(env, resource_ids, with_monitor=False):
         primitives = []
 
         for resource_el in resource_el_list:
-            resource.common.manage(resource_el)
+            resource.common.manage(resource_el, id_provider)
             primitives.extend(
                 resource.common.find_primitives(resource_el)
             )
@@ -768,17 +960,17 @@ def manage(env, resource_ids, with_monitor=False):
             set(primitives),
             key=lambda element: element.get("id", "")
         ):
-            op_list = operations.get_resource_operations(
+            op_list = resource.operations.get_resource_operations(
                 resource_el,
                 ["monitor"]
             )
             if with_monitor:
                 for op in op_list:
-                    operations.enable(op)
+                    resource.operations.enable(op)
             else:
                 monitor_enabled = False
                 for op in op_list:
-                    if operations.is_enabled(op):
+                    if resource.operations.is_enabled(op):
                         monitor_enabled = True
                         break
                 if op_list and not monitor_enabled:
@@ -791,6 +983,48 @@ def manage(env, resource_ids, with_monitor=False):
 
         env.report_processor.process_list(report_list)
 
+def group_add(
+    env, group_id, resource_id_list, adjacent_resource_id=None,
+    put_after_adjacent=True, wait=False
+):
+    """
+    Move specified resources into an existing or new group
+
+    LibraryEnvironment env provides all for communication with externals
+    string group_id -- id of the target group
+    iterable resource_id_list -- ids of resources to put into the group
+    string adjacent_resource_id -- put resources beside this one if specified
+    bool put_after_adjacent -- put resources after or before the adjacent one
+    mixed wait -- flag for controlling waiting for pacemaker idle mechanism
+    """
+    with resource_environment(env, wait, [group_id]) as resources_section:
+        id_provider = IdProvider(resources_section)
+
+        validator = resource.hierarchy.ValidateMoveResourcesToGroupByIds(
+            group_id,
+            resource_id_list,
+            adjacent_resource_id=adjacent_resource_id
+        )
+        # raise on error
+        env.report_processor.process_list(
+            validator.validate(resources_section, id_provider)
+        )
+
+        # If we get no group element from the validator and there were no
+        # errors, then the element does not exist and we can create it.
+        group_element = validator.group_element()
+        if group_element is None:
+            group_element = resource.group.append_new(
+                resources_section, group_id
+            )
+
+        resource.hierarchy.move_resources_to_group(
+            group_element,
+            validator.resource_element_list(),
+            adjacent_resource=validator.adjacent_resource_element(),
+            put_after_adjacent=put_after_adjacent,
+        )
+
 def get_failcounts(
     env, resource=None, node=None, operation=None, interval=None
 ):
@@ -811,7 +1045,7 @@ def get_failcounts(
         )
     if interval is not None:
         report_items.extend(
-            value_time_interval("interval")({"interval": interval})
+            ValueTimeInterval("interval").validate({"interval": interval})
         )
     if report_items:
         raise LibraryError(*report_items)
@@ -832,40 +1066,324 @@ def get_failcounts(
         interval=interval_ms
     )
 
-def _find_resources_or_raise(
-    resources_section, resource_ids, additional_search=None
-):
-    if not additional_search:
-        additional_search = lambda x: [x]
-    report_list = []
-    resource_el_list = []
-    resource_tags = (
-        resource.clone.ALL_TAGS
-        +
-        [resource.group.TAG, resource.primitive.TAG, resource.bundle.TAG]
+def move(env, resource_id, node=None, master=False, lifetime=None, wait=False):
+    """
+    Create a constraint to move a resource
+
+    LibraryEnvironment env
+    string resource_id -- id of a resource to be moved
+    string node -- node to move the resource to, ban on the current node if None
+    bool master -- limit the constraint to the Master role
+    string lifetime -- lifespan of the constraint, forever if None
+    mixed wait -- flag for controlling waiting for pacemaker idle mechanism
+    """
+    return _Move().run(
+        env,
+        resource_id,
+        node=node,
+        master=master,
+        lifetime=lifetime,
+        wait=wait
     )
-    for res_id in resource_ids:
-        try:
-            resource_el_list.extend(
-                additional_search(
-                    find_element_by_tag_and_id(
-                        resource_tags,
-                        resources_section,
-                        res_id,
-                        # pacemaker-2.0 deprecated masters. We treat masters as
-                        # clones. Do not report we were looking for a master,
-                        # say we were looking for a clone instead. Since we
-                        # look for all resource types including clones, just
-                        # drop the master.
-                        id_types=[
-                            tag for tag in resource_tags
-                            if tag != resource.clone.TAG_MASTER
-                        ]
-                    )
+
+def ban(env, resource_id, node=None, master=False, lifetime=None, wait=False):
+    """
+    Create a constraint to keep a resource of a node
+
+    LibraryEnvironment env
+    string resource_id -- id of a resource to be banned
+    string node -- node to ban the resource on, ban on the current node if None
+    bool master -- limit the constraint to the Master role
+    string lifetime -- lifespan of the constraint, forever if None
+    mixed wait -- flag for controlling waiting for pacemaker idle mechanism
+    """
+    return _Ban().run(
+        env,
+        resource_id,
+        node=node,
+        master=master,
+        lifetime=lifetime,
+        wait=wait
+    )
+
+class _MoveBanTemplate():
+    def _validate(self, resource_el, master):
+        raise NotImplementedError()
+
+    def _run_action(self, runner, resource_id, node, master, lifetime):
+        raise NotImplementedError()
+
+    def _report_action_stopped_resource(self, resource_id):
+        raise NotImplementedError()
+
+    def _report_action_pcmk_error(self, resource_id, stdout, stderr):
+        raise NotImplementedError()
+
+    def _report_action_pcmk_success(self, resource_id, stdout, stderr):
+        raise NotImplementedError()
+
+    def _report_wait_result(
+        self, resource_id, node, resource_running_on_before,
+        resource_running_on_after,
+    ):
+        raise NotImplementedError()
+
+    @staticmethod
+    def _running_on_nodes(resource_state):
+        if resource_state:
+            return frozenset(
+                resource_state.get("Master", [])
+                +
+                resource_state.get("Started", [])
+            )
+        return frozenset()
+
+    def run(
+        self,
+        env, resource_id, node=None, master=False, lifetime=None, wait=False
+    ):
+        # validate
+        env.ensure_wait_satisfiable(wait) # raises on error
+
+        report_list = []
+        resource_el = resource.common.find_one_resource_and_report(
+            get_resources(env.get_cib()),
+            resource_id,
+            report_list,
+        )
+        if resource_el is not None:
+            report_list.extend(self._validate(resource_el, master))
+        env.report_processor.process_list(report_list) # raises on error
+
+        # get current status for wait processing
+        if wait is not False:
+            resource_running_on_before = get_resource_state(
+                env.get_cluster_state(),
+                resource_id
+            )
+
+        # run the action
+        stdout, stderr, retval = self._run_action(
+            env.cmd_runner(), resource_id, node=node, master=master,
+            lifetime=lifetime
+        )
+        if retval != 0:
+            if (
+                f"Resource '{resource_id}' not moved: active in 0 locations"
+                in
+                stderr
+            ):
+                raise LibraryError(
+                    self._report_action_stopped_resource(resource_id)
+                )
+            raise LibraryError(
+                self._report_action_pcmk_error(resource_id, stdout, stderr)
+            )
+        env.report_processor.process(
+            self._report_action_pcmk_success(resource_id, stdout, stderr)
+        )
+
+        # process wait
+        if wait is not False:
+            wait_for_idle(env.cmd_runner(), env.get_wait_timeout(wait))
+            resource_running_on_after = get_resource_state(
+                env.get_cluster_state(),
+                resource_id
+            )
+            env.report_processor.process(
+                self._report_wait_result(
+                    resource_id,
+                    node,
+                    resource_running_on_before,
+                    resource_running_on_after,
                 )
             )
-        except LibraryError as e:
-            report_list.extend(e.args)
+
+class _Move(_MoveBanTemplate):
+    def _validate(self, resource_el, master):
+        return resource.common.validate_move(resource_el, master)
+
+    def _run_action(self, runner, resource_id, node, master, lifetime):
+        return resource_move(
+            runner, resource_id, node=node, master=master, lifetime=lifetime
+        )
+
+    def _report_action_stopped_resource(self, resource_id):
+        return reports.cannot_move_resource_stopped_no_node_specified(
+            resource_id
+        )
+
+    def _report_action_pcmk_error(self, resource_id, stdout, stderr):
+        return reports.resource_move_pcmk_error(resource_id, stdout, stderr)
+
+    def _report_action_pcmk_success(self, resource_id, stdout, stderr):
+        return reports.resource_move_pcmk_success(resource_id, stdout, stderr)
+
+    def _report_wait_result(
+        self, resource_id, node, resource_running_on_before,
+        resource_running_on_after
+    ):
+        allowed_nodes = frozenset([node] if node else [])
+        running_on_nodes = self._running_on_nodes(resource_running_on_after)
+
+        severity = severities.INFO
+        if (
+            resource_running_on_before # running resource moved
+            and (
+                not running_on_nodes
+                or
+                (allowed_nodes and allowed_nodes.isdisjoint(running_on_nodes))
+                or
+                (resource_running_on_before == resource_running_on_after)
+           )
+        ):
+            severity = severities.ERROR
+        if not resource_running_on_after:
+            return reports.resource_does_not_run(resource_id, severity=severity)
+        return reports.resource_running_on_nodes(
+            resource_id,
+            resource_running_on_after,
+            severity=severity
+        )
+
+class _Ban(_MoveBanTemplate):
+    def _validate(self, resource_el, master):
+        return resource.common.validate_ban(resource_el, master)
+
+    def _run_action(self, runner, resource_id, node, master, lifetime):
+        return resource_ban(
+            runner, resource_id, node=node, master=master, lifetime=lifetime
+        )
+
+    def _report_action_stopped_resource(self, resource_id):
+        return reports.cannot_ban_resource_stopped_no_node_specified(
+            resource_id
+        )
+
+    def _report_action_pcmk_error(self, resource_id, stdout, stderr):
+        return reports.resource_ban_pcmk_error(resource_id, stdout, stderr)
+
+    def _report_action_pcmk_success(self, resource_id, stdout, stderr):
+        return reports.resource_ban_pcmk_success(resource_id, stdout, stderr)
+
+    def _report_wait_result(
+        self, resource_id, node, resource_running_on_before,
+        resource_running_on_after
+    ):
+        running_on_nodes = self._running_on_nodes(resource_running_on_after)
+        if node:
+            banned_nodes = frozenset([node])
+        else:
+            banned_nodes = self._running_on_nodes(resource_running_on_before)
+
+        severity = severities.INFO
+        if (
+            not banned_nodes.isdisjoint(running_on_nodes)
+            or
+            (resource_running_on_before and not running_on_nodes)
+        ):
+            severity = severities.ERROR
+        if not resource_running_on_after:
+            return reports.resource_does_not_run(resource_id, severity=severity)
+        return reports.resource_running_on_nodes(
+            resource_id,
+            resource_running_on_after,
+            severity=severity
+        )
+
+def unmove_unban(
+    env, resource_id, node=None, master=False, expired=False, wait=False
+):
+    """
+    Remove all constraints created by move and ban
+
+    LibraryEnvironment env
+    string resource_id -- id of a resource to be unmoved/unbanned
+    string node -- node to limit unmoving/unbanning to, all nodes if None
+    bool master -- only remove constraints for Master role
+    bool expired -- only remove constrains which have already expired
+    mixed wait -- flag for controlling waiting for pacemaker idle mechanism
+    """
+    # validate
+    env.ensure_wait_satisfiable(wait) # raises on error
+
+    report_list = []
+    resource_el = resource.common.find_one_resource_and_report(
+        get_resources(env.get_cib()),
+        resource_id,
+        report_list,
+    )
+    if resource_el is not None:
+        report_list.extend(
+            resource.common.validate_unmove_unban(resource_el, master)
+        )
+    if (
+        expired
+        and
+        not has_resource_unmove_unban_expired_support(env.cmd_runner())
+    ):
+        report_list.append(
+            reports.resource_unmove_unban_pcmk_expired_not_supported()
+        )
+    env.report_processor.process_list(report_list) # raises on error
+
+    # run the action
+    stdout, stderr, retval = resource_unmove_unban(
+        env.cmd_runner(), resource_id, node=node, master=master, expired=expired
+    )
+    if retval != 0:
+        raise LibraryError(
+            reports.resource_unmove_unban_pcmk_error(
+                resource_id, stdout, stderr
+            )
+        )
+    env.report_processor.process(
+        reports.resource_unmove_unban_pcmk_success(resource_id, stdout, stderr)
+    )
+
+    # process wait
+    if wait is not False:
+        wait_for_idle(env.cmd_runner(), env.get_wait_timeout(wait))
+        env.report_processor.process(
+            info_resource_state(env.get_cluster_state(), resource_id)
+        )
+
+
+def get_resource_relations_tree(
+    env: LibraryEnvironment,
+    resource_id: str,
+) -> Mapping[str, Any]:
+    """
+    Return a dict representing tree-like structure of resources and their
+    relations.
+
+    env -- library environment
+    resource_id -- id of a resource which should be the root of the relation
+        tree
+    """
+    cib = env.get_cib()
+    _find_resources_or_raise(get_resources(cib), [resource_id])
+    resources_dict, relations_dict = (
+        resource.relations.ResourceRelationsFetcher(
+            cib
+        ).get_relations(resource_id)
+    )
+    return resource.relations.ResourceRelationTreeBuilder(
+        resources_dict, relations_dict
+    ).get_tree(resource_id).to_dto().to_dict()
+
+
+def _find_resources_or_raise(
+    context_element, resource_ids, additional_search=None, resource_tags=None
+):
+    report_list = []
+    resource_el_list = resource.common.find_resources_and_report(
+        context_element,
+        resource_ids,
+        report_list,
+        additional_search=additional_search,
+        resource_tags=resource_tags,
+    )
     if report_list:
         raise LibraryError(*report_list)
     return resource_el_list
