@@ -1,15 +1,19 @@
-from __future__ import (
-    absolute_import,
-    division,
-    print_function,
-)
-
+from pcs.lib import reports
 from pcs.lib.cib import resource
 from pcs.lib.cib.resource.common import are_meta_disabled
+from pcs.lib.cib.tools import IdProvider
 from pcs.lib.commands.resource import (
     _ensure_disabled_after_wait,
     resource_environment
 )
+from pcs.lib.errors import LibraryError
+from pcs.lib.pacemaker.live import (
+    FenceHistoryCommandErrorException,
+    fence_history_cleanup,
+    fence_history_text,
+    fence_history_update,
+    is_fence_history_supported,
+)
 from pcs.lib.pacemaker.values import validate_id
 from pcs.lib.resource_agent import find_valid_stonith_agent_by_name as get_agent
 
@@ -23,6 +27,7 @@ def create(
     ensure_disabled=False,
     wait=False,
 ):
+    # pylint: disable=too-many-arguments, too-many-locals
     """
     Create stonith as resource in a cib.
 
@@ -44,7 +49,7 @@ def create(
     bool use_default_operations is a flag for stopping stopping of adding
         default cib operations (specified in a stonith agent)
     bool ensure_disabled is flag that keeps resource in target-role "Stopped"
-    mixed wait is flag for controlling waiting for pacemaker iddle mechanism
+    mixed wait is flag for controlling waiting for pacemaker idle mechanism
     """
     stonith_agent = get_agent(
         env.report_processor,
@@ -63,9 +68,11 @@ def create(
             ensure_disabled or are_meta_disabled(meta_attributes),
         )
     ) as resources_section:
+        id_provider = IdProvider(resources_section)
         stonith_element = resource.primitive.create(
             env.report_processor,
             resources_section,
+            id_provider,
             stonith_id,
             stonith_agent,
             raw_operation_list=operations,
@@ -77,7 +84,7 @@ def create(
             resource_type="stonith"
         )
         if ensure_disabled:
-            resource.common.disable(stonith_element)
+            resource.common.disable(stonith_element, id_provider)
 
 def create_in_group(
     env, stonith_id, stonith_agent_name, group_id,
@@ -91,6 +98,7 @@ def create_in_group(
     put_after_adjacent=False,
     wait=False,
 ):
+    # pylint: disable=too-many-arguments, too-many-locals
     """
     Create stonith as resource in a cib and put it into defined group.
 
@@ -116,7 +124,7 @@ def create_in_group(
     string adjacent_resource_id identify neighbor of a newly created stonith
     bool put_after_adjacent is flag to put a newly create resource befor/after
         adjacent stonith
-    mixed wait is flag for controlling waiting for pacemaker iddle mechanism
+    mixed wait is flag for controlling waiting for pacemaker idle mechanism
     """
     stonith_agent = get_agent(
         env.report_processor,
@@ -135,8 +143,9 @@ def create_in_group(
             ensure_disabled or are_meta_disabled(meta_attributes),
         )
     ) as resources_section:
+        id_provider = IdProvider(resources_section)
         stonith_element = resource.primitive.create(
-            env.report_processor, resources_section,
+            env.report_processor, resources_section, id_provider,
             stonith_id, stonith_agent,
             operations, meta_attributes, instance_attributes,
             allow_invalid_operation,
@@ -144,7 +153,7 @@ def create_in_group(
             use_default_operations,
         )
         if ensure_disabled:
-            resource.common.disable(stonith_element)
+            resource.common.disable(stonith_element, id_provider)
         validate_id(group_id, "group name")
         resource.group.place_resource(
             resource.group.provide_group(resources_section, group_id),
@@ -152,3 +161,53 @@ def create_in_group(
             adjacent_resource_id,
             put_after_adjacent,
         )
+
+def history_get_text(env, node=None):
+    """
+    Get full fencing history in plain text
+
+    LibraryEnvironment env
+    string node -- get history for the specified node or all nodes if None
+    """
+    if not is_fence_history_supported():
+        raise LibraryError(reports.fence_history_not_supported())
+
+    try:
+        return fence_history_text(env.cmd_runner(), node)
+    except FenceHistoryCommandErrorException as e:
+        raise LibraryError(
+            reports.fence_history_command_error(str(e), "show")
+        )
+
+def history_cleanup(env, node=None):
+    """
+    Clear fencing history
+
+    LibraryEnvironment env
+    string node -- clear history for the specified node or all nodes if None
+    """
+    if not is_fence_history_supported():
+        raise LibraryError(reports.fence_history_not_supported())
+
+    try:
+        return fence_history_cleanup(env.cmd_runner(), node)
+    except FenceHistoryCommandErrorException as e:
+        raise LibraryError(
+            reports.fence_history_command_error(str(e), "cleanup")
+        )
+
+def history_update(env):
+    """
+    Update fencing history in a cluster (sync with other nodes)
+
+    LibraryEnvironment env
+    """
+    if not is_fence_history_supported():
+        raise LibraryError(reports.fence_history_not_supported())
+
+    try:
+        return fence_history_update(env.cmd_runner())
+    except FenceHistoryCommandErrorException as e:
+        raise LibraryError(
+            reports.fence_history_command_error(str(e), "update")
+        )
