@@ -1,27 +1,30 @@
-from pcs.common.node_communicator import NodeCommunicatorFactory
+from typing import (
+    Optional,
+)
+from xml.etree.ElementTree import Element
+
+from pcs.common.node_communicator import Communicator, NodeCommunicatorFactory
 from pcs.common.tools import Version
 from pcs.lib import reports
 from pcs.lib.booth.env import BoothEnv
 from pcs.lib.cib.tools import get_cib_crm_feature_set
-from pcs.lib.pacemaker.env import PacemakerEnv
+from pcs.lib.node import get_existing_nodes_names
 from pcs.lib.communication import qdevice
 from pcs.lib.communication.corosync import (
     CheckCorosyncOffline,
     DistributeCorosyncConf,
+    ReloadCorosyncConf,
 )
 from pcs.lib.communication.tools import (
     run,
     run_and_raise,
 )
 from pcs.lib.corosync.config_facade import ConfigFacade as CorosyncConfigFacade
-from pcs.lib.corosync.live import (
-    get_local_corosync_conf,
-    reload_config as reload_corosync_config,
-)
-from pcs.lib.external import (
-    is_service_running,
-    CommandRunner,
+from pcs.lib.corosync.config_parser import (
+    verify_section as verify_corosync_section
 )
+from pcs.lib.corosync.live import get_local_corosync_conf
+from pcs.lib.external import CommandRunner
 from pcs.lib.errors import LibraryError
 from pcs.lib.node_communication import (
     LibCommunicatorLogger,
@@ -56,7 +59,7 @@ class LibraryEnvironment:
         user_groups=None,
         cib_data=None,
         corosync_conf_data=None,
-        booth=None,
+        booth_files_data=None,
         known_hosts_getter=None,
         request_timeout=None,
     ):
@@ -67,12 +70,7 @@ class LibraryEnvironment:
         self._user_groups = [] if user_groups is None else user_groups
         self._cib_data = cib_data
         self._corosync_conf_data = corosync_conf_data
-        self._booth = (
-            BoothEnv(report_processor, booth) if booth is not None else None
-        )
-        #pacemaker is currently not mocked and it provides only an access to
-        #the authkey
-        self._pacemaker = PacemakerEnv()
+        self._booth_files_data = booth_files_data or {}
         self._request_timeout = request_timeout
         # TODO tokens probably should not be inserted from outside, but we're
         # postponing dealing with them, because it's not that easy to move
@@ -90,6 +88,7 @@ class LibraryEnvironment:
             self.user_groups,
             self._request_timeout
         )
+        self.__loaded_booth_env = None
 
         self.__timeout_cache = {}
 
@@ -109,7 +108,7 @@ class LibraryEnvironment:
     def user_groups(self):
         return self._user_groups
 
-    def get_cib(self, minimal_version=None):
+    def get_cib(self, minimal_version: Optional[Version] = None) -> Element:
         if self.__loaded_cib_diff_source is not None:
             raise AssertionError("CIB has already been loaded")
         self.__loaded_cib_diff_source = get_cib_xml(self.cmd_runner())
@@ -147,7 +146,7 @@ class LibraryEnvironment:
     def get_cluster_state(self):
         return get_cluster_state_dom(get_cluster_status_xml(self.cmd_runner()))
 
-    def _get_wait_timeout(self, wait):
+    def get_wait_timeout(self, wait):
         if wait is False:
             return False
 
@@ -165,7 +164,7 @@ class LibraryEnvironment:
 
         mixed wait can be False when waiting is not required or valid timeout
         """
-        self._get_wait_timeout(wait)
+        self.get_wait_timeout(wait)
 
     def push_cib(self, custom_cib=None, wait=False):
         """
@@ -231,7 +230,7 @@ class LibraryEnvironment:
             push_cib_diff_xml(cmd_runner, cib_diff_xml)
 
     def __do_push_cib(self, cmd_runner, push_strategy, wait):
-        timeout = self._get_wait_timeout(wait)
+        timeout = self.get_wait_timeout(wait)
         push_strategy()
         self._cib_upgrade_reported = False
         self.__loaded_cib_diff_source = None
@@ -263,17 +262,35 @@ class LibraryEnvironment:
             return get_local_corosync_conf()
         return self._corosync_conf_data
 
-    def get_corosync_conf(self):
+    def get_corosync_conf(self) -> CorosyncConfigFacade:
         return CorosyncConfigFacade.from_string(self.get_corosync_conf_data())
 
     def push_corosync_conf(
         self, corosync_conf_facade, skip_offline_nodes=False
     ):
+        bad_sections, bad_attr_names, bad_attr_values = verify_corosync_section(
+            corosync_conf_facade.config
+        )
+        if bad_sections or bad_attr_names or bad_attr_values:
+            raise LibraryError(
+                reports.corosync_config_cannot_save_invalid_names_values(
+                    bad_sections, bad_attr_names, bad_attr_values
+                )
+            )
         corosync_conf_data = corosync_conf_facade.config.export()
         if self.is_corosync_conf_live:
+            node_name_list, report_list = get_existing_nodes_names(
+                corosync_conf_facade,
+                # Pcs is unable to communicate with nodes missing names. It
+                # cannot send new corosync.conf to them. That might break the
+                # cluster. Hence we error out.
+                error_on_missing_name=True
+            )
+            self.report_processor.process_list(report_list)
+
             self._push_corosync_conf_live(
                 self.get_node_target_factory().get_target_list(
-                    corosync_conf_facade.get_nodes_names(),
+                    node_name_list,
                     skip_non_existing=skip_offline_nodes,
                 ),
                 corosync_conf_data,
@@ -288,22 +305,28 @@ class LibraryEnvironment:
         self, target_list, corosync_conf_data, need_stopped_cluster,
         need_qdevice_reload, skip_offline_nodes
     ):
+        # Check if the cluster is stopped when needed
         if need_stopped_cluster:
             com_cmd = CheckCorosyncOffline(
                 self.report_processor, skip_offline_nodes
             )
             com_cmd.set_targets(target_list)
             run_and_raise(self.get_node_communicator(), com_cmd)
+        # Distribute corosync.conf
         com_cmd = DistributeCorosyncConf(
             self.report_processor, corosync_conf_data, skip_offline_nodes
         )
         com_cmd.set_targets(target_list)
         run_and_raise(self.get_node_communicator(), com_cmd)
-        if is_service_running(self.cmd_runner(), "corosync"):
-            reload_corosync_config(self.cmd_runner())
-            self.report_processor.process(
-                reports.corosync_config_reloaded()
-            )
+        # Reload corosync
+        if not need_stopped_cluster:
+            # If cluster must be stopped then we cannot reload corosync because
+            # the cluster is stopped. If it is not stopped, we do not even get
+            # here.
+            com_cmd = ReloadCorosyncConf(self.report_processor)
+            com_cmd.set_targets(target_list)
+            run_and_raise(self.get_node_communicator(), com_cmd)
+        # Reload qdevice if needed
         if need_qdevice_reload:
             self.report_processor.process(
                 reports.qdevice_client_reload_started()
@@ -323,7 +346,7 @@ class LibraryEnvironment:
     def is_corosync_conf_live(self):
         return self._corosync_conf_data is None
 
-    def cmd_runner(self):
+    def cmd_runner(self) -> CommandRunner:
         runner_env = {
             # make sure to get output of external processes in English and ASCII
             "LC_ALL": "C",
@@ -356,12 +379,15 @@ class LibraryEnvironment:
     def communicator_factory(self):
         return self._communicator_factory
 
-    def get_node_communicator(self, request_timeout=None):
+    def get_node_communicator(
+        self,
+        request_timeout: Optional[int] = None,
+    ) -> Communicator:
         return self.communicator_factory.get_communicator(
             request_timeout=request_timeout
         )
 
-    def get_node_target_factory(self):
+    def get_node_target_factory(self) -> NodeTargetLibFactory:
         return NodeTargetLibFactory(
             self.__get_known_hosts(), self.report_processor
         )
@@ -382,10 +408,7 @@ class LibraryEnvironment:
                 self._known_hosts = {}
         return self._known_hosts
 
-    @property
-    def booth(self):
-        return self._booth
-
-    @property
-    def pacemaker(self):
-        return self._pacemaker
+    def get_booth_env(self, name):
+        if self.__loaded_booth_env is None:
+            self.__loaded_booth_env = BoothEnv(name, self._booth_files_data)
+        return self.__loaded_booth_env
