@@ -1,24 +1,16 @@
-from __future__ import (
-    absolute_import,
-    division,
-    print_function,
-)
+import base64
 from collections import namedtuple
+
 from pcs.lib import reports
 from pcs.lib.errors import LibraryError
-import base64
 
 def create_pcmk_remote_actions(action_list):
-    return dict([
-        (
-            "pacemaker_remote {0}".format(action),
-            service_cmd_format(
-                "pacemaker_remote",
-                action
-            )
+    return {
+        "pacemaker_remote {0}".format(action): service_cmd_format(
+            "pacemaker_remote", action
         )
         for action in action_list
-    ])
+    }
 
 def pcmk_authkey_format(authkey_content):
     """
@@ -52,6 +44,29 @@ def corosync_authkey_file(authkey_content):
         "corosync authkey": corosync_authkey_format(authkey_content)
     }
 
+def corosync_conf_format(corosync_conf_content):
+    return {
+        "type": "corosync_conf",
+        "data": corosync_conf_content,
+    }
+
+def corosync_conf_file(corosync_conf_content):
+    return {
+        "corosync.conf": corosync_conf_format(corosync_conf_content)
+    }
+
+def pcs_settings_conf_format(content):
+    return {
+        "data": content,
+        "type": "pcs_settings_conf",
+        "rewrite_existing": True,
+    }
+
+def pcs_settings_conf_file(content):
+    return {
+        "pcs_settings.conf": pcs_settings_conf_format(content)
+    }
+
 def service_cmd_format(service, command):
     """
     Return a dict usable in the communication with a remote/run_action
@@ -118,13 +133,10 @@ def response_items_to_result(response_items, expected_keys, node_label):
         ):
             raise LibraryError(reports.invalid_response_format(node_label))
 
-    return dict([
-        (
-            file_key,
-            Result(raw_result["code"], raw_result["message"])
-        )
+    return {
+        file_key: Result(raw_result["code"], raw_result["message"])
         for file_key, raw_result in response_items.items()
-    ])
+    }
 
 
 def response_to_result(
