@@ -186,7 +186,7 @@ def get_local_sbd_config():
         ))
 
 
-def get_sbd_service_name():
+def get_sbd_service_name() -> str:
     return "sbd" if external.is_systemctl() else "sbd_helper"
 
 
@@ -349,5 +349,4 @@ def test_watchdog(cmd_runner, watchdog=None):
         if "Multiple watchdog devices discovered" in std_out:
             raise LibraryError(reports.sbd_watchdog_test_multiple_devices())
         raise LibraryError(reports.sbd_watchdog_test_error(std_out))
-    else:
-        raise LibraryError(reports.sbd_watchdog_test_failed())
+    raise LibraryError(reports.sbd_watchdog_test_failed())
