@@ -1,50 +1,13 @@
-import sys
 import json
 
-from pcs import (
-    usage,
-    utils,
-)
+from pcs import utils
 from pcs.cli.common.errors import (
     CmdLineInputError,
     ERR_NODE_LIST_AND_ALL_MUTUALLY_EXCLUSIVE,
 )
 from pcs.cli.common.parse_args import prepare_options
-from pcs.lib.errors import LibraryError
 import pcs.lib.pacemaker.live as lib_pacemaker
 
-def node_cmd(lib, argv, modifiers):
-    if not argv:
-        usage.node()
-        sys.exit(1)
-
-    sub_cmd, argv_next = argv[0], argv[1:]
-
-    try:
-        if sub_cmd == "help":
-            usage.node([" ".join(argv_next)] if argv_next else [])
-        elif sub_cmd == "maintenance":
-            node_maintenance_cmd(lib, argv_next, modifiers, True)
-        elif sub_cmd == "unmaintenance":
-            node_maintenance_cmd(lib, argv_next, modifiers, False)
-        elif sub_cmd == "standby":
-            node_standby_cmd(lib, argv_next, modifiers, True)
-        elif sub_cmd == "unstandby":
-            node_standby_cmd(lib, argv_next, modifiers, False)
-        elif sub_cmd == "attribute":
-            node_attribute_cmd(lib, argv_next, modifiers)
-        elif sub_cmd == "utilization":
-            node_utilization_cmd(lib, argv_next, modifiers)
-        # pcs-to-pcsd use only
-        elif sub_cmd == "pacemaker-status":
-            node_pacemaker_status(lib, argv_next, modifiers)
-        else:
-            raise CmdLineInputError()
-    except LibraryError as e:
-        utils.process_library_reports(e.args)
-    except CmdLineInputError as e:
-        utils.exit_on_cmdline_input_errror(e, "node", sub_cmd)
-
 def node_attribute_cmd(lib, argv, modifiers):
     """
     Options:
@@ -124,6 +87,15 @@ def set_node_utilization(node, argv):
     Commandline options:
       * -f - CIB file
     """
+    nvpair_dict = prepare_options(argv)
+    if not nvpair_dict:
+        return
+    only_removing = True
+    for value in nvpair_dict.values():
+        if value != "":
+            only_removing = False
+            break
+
     cib = utils.get_cib_dom()
     node_el = utils.dom_get_node(cib, node)
     if node_el is None:
@@ -141,6 +113,10 @@ def set_node_utilization(node, argv):
         if not nodes_section_list:
             utils.err("Unable to get nodes section of cib")
 
+        if only_removing:
+            # Do not create new node if we are only removing values from it.
+            return
+
         dom = nodes_section_list[0].ownerDocument
         node_el = dom.createElement("node")
         node_el.setAttribute("id", node_attrs.id)
@@ -148,7 +124,7 @@ def set_node_utilization(node, argv):
         node_el.setAttribute("uname", node_attrs.name)
         nodes_section_list[0].appendChild(node_el)
 
-    utils.dom_update_utilization(node_el, prepare_options(argv), "nodes-")
+    utils.dom_update_utilization(node_el, nvpair_dict, "nodes-")
     utils.replace_cib_configuration(cib)
 
 def print_node_utilization(filter_node=None, filter_name=None):
