@@ -1,69 +1,7 @@
 import sys
 
-from pcs import (
-    usage,
-    utils,
-)
 from pcs.cli.common.errors import CmdLineInputError
-from pcs.lib.errors import LibraryError
 
-def qdevice_cmd(lib, argv, modifiers):
-    # pylint: disable=too-many-branches
-    if not argv:
-        usage.qdevice()
-        sys.exit(1)
-
-    sub_cmd, argv_next = argv[0], argv[1:]
-    try:
-        if sub_cmd == "help":
-            usage.qdevice([" ".join(argv_next)] if argv_next else [])
-        elif sub_cmd == "status":
-            qdevice_status_cmd(lib, argv_next, modifiers)
-        elif sub_cmd == "setup":
-            qdevice_setup_cmd(lib, argv_next, modifiers)
-        elif sub_cmd == "destroy":
-            qdevice_destroy_cmd(lib, argv_next, modifiers)
-        elif sub_cmd == "start":
-            qdevice_start_cmd(lib, argv_next, modifiers)
-        elif sub_cmd == "stop":
-            qdevice_stop_cmd(lib, argv_next, modifiers)
-        elif sub_cmd == "kill":
-            qdevice_kill_cmd(lib, argv_next, modifiers)
-        elif sub_cmd == "enable":
-            qdevice_enable_cmd(lib, argv_next, modifiers)
-        elif sub_cmd == "disable":
-            qdevice_disable_cmd(lib, argv_next, modifiers)
-        # following commands are internal use only, called from pcsd
-        elif sub_cmd == "sign-net-cert-request":
-            qdevice_sign_net_cert_request_cmd(lib, argv_next, modifiers)
-        elif sub_cmd == "net-client":
-            qdevice_net_client_cmd(lib, argv_next, modifiers)
-        else:
-            raise CmdLineInputError()
-    except LibraryError as e:
-        utils.process_library_reports(e.args)
-    except CmdLineInputError as e:
-        utils.exit_on_cmdline_input_errror(e, "qdevice", sub_cmd)
-
-# this is internal use only, called from pcsd
-def qdevice_net_client_cmd(lib, argv, modifiers):
-    if not argv:
-        utils.err("invalid command")
-
-    sub_cmd, argv_next = argv[0], argv[1:]
-    try:
-        if sub_cmd == "setup":
-            qdevice_net_client_setup_cmd(lib, argv_next, modifiers)
-        elif sub_cmd == "import-certificate":
-            qdevice_net_client_import_certificate_cmd(lib, argv_next, modifiers)
-        elif sub_cmd == "destroy":
-            qdevice_net_client_destroy(lib, argv_next, modifiers)
-        else:
-            raise CmdLineInputError("invalid command")
-    except LibraryError as e:
-        utils.process_library_reports(e.args)
-    except CmdLineInputError as e:
-        utils.err(e.message)
 
 def qdevice_status_cmd(lib, argv, modifiers):
     """
