@@ -1,81 +1,12 @@
 from pcs import (
     stonith,
-    usage,
     utils,
 )
 from pcs.cli.common import parse_args
-from pcs.cli.common.console_report import indent
 from pcs.cli.common.errors import CmdLineInputError
-from pcs.lib.errors import LibraryError
-
-def quorum_cmd(lib, argv, modifiers):
-    if not argv:
-        sub_cmd, argv_next = "config", []
-    else:
-        sub_cmd, argv_next = argv[0], argv[1:]
-
-    try:
-        if sub_cmd == "help":
-            usage.quorum([" ".join(argv_next)] if argv_next else [])
-        elif sub_cmd == "config":
-            quorum_config_cmd(lib, argv_next, modifiers)
-        elif sub_cmd == "expected-votes":
-            quorum_expected_votes_cmd(lib, argv_next, modifiers)
-        elif sub_cmd == "status":
-            quorum_status_cmd(lib, argv_next, modifiers)
-        elif sub_cmd == "device":
-            quorum_device_cmd(lib, argv_next, modifiers)
-        elif sub_cmd == "unblock":
-            # TODO switch to new architecture
-            quorum_unblock_cmd(lib, argv_next, modifiers)
-        elif sub_cmd == "update":
-            quorum_update_cmd(lib, argv_next, modifiers)
-        else:
-            raise CmdLineInputError()
-    except LibraryError as e:
-        utils.process_library_reports(e.args)
-    except CmdLineInputError as e:
-        utils.exit_on_cmdline_input_errror(e, "quorum", sub_cmd)
-
-def quorum_device_cmd(lib, argv, modifiers):
-    if not argv:
-        raise CmdLineInputError()
-
-    sub_cmd, argv_next = argv[0], argv[1:]
-    try:
-        if sub_cmd == "add":
-            quorum_device_add_cmd(lib, argv_next, modifiers)
-        elif sub_cmd == "heuristics":
-            quorum_device_heuristics_cmd(lib, argv_next, modifiers)
-        elif sub_cmd in {"delete", "remove"}:
-            quorum_device_remove_cmd(lib, argv_next, modifiers)
-        elif sub_cmd == "status":
-            quorum_device_status_cmd(lib, argv_next, modifiers)
-        elif sub_cmd == "update":
-            quorum_device_update_cmd(lib, argv_next, modifiers)
-        else:
-            sub_cmd = ""
-            raise CmdLineInputError()
-    except CmdLineInputError as e:
-        utils.exit_on_cmdline_input_errror(
-            e, "quorum", "device {0}".format(sub_cmd)
-        )
-
-def quorum_device_heuristics_cmd(lib, argv, modifiers):
-    if not argv:
-        raise CmdLineInputError()
-
-    sub_cmd, argv_next = argv[0], argv[1:]
-    try:
-        if sub_cmd in {"delete", "remove"}:
-            quorum_device_heuristics_remove_cmd(lib, argv_next, modifiers)
-        else:
-            sub_cmd = ""
-            raise CmdLineInputError()
-    except CmdLineInputError as e:
-        utils.exit_on_cmdline_input_errror(
-            e, "quorum", "device heuristics {0}".format(sub_cmd)
-        )
+from pcs.cli.common.reports import process_library_reports
+from pcs.common.tools import indent
+from pcs.lib.node import get_existing_nodes_names
 
 
 def quorum_config_cmd(lib, argv, modifiers):
@@ -325,14 +256,16 @@ def quorum_unblock_cmd(lib, argv, modifiers):
     if output.split("=")[-1].strip() != "1":
         utils.err("cluster is not waiting for nodes to establish quorum")
 
-    unjoined_nodes = (
-        set(utils.get_corosync_conf_facade().get_nodes_names())
-        -
-        set(utils.getCorosyncActiveNodes())
+    all_nodes, report_list = get_existing_nodes_names(
+        utils.get_corosync_conf_facade()
     )
+    if report_list:
+        process_library_reports(report_list)
+
+    unjoined_nodes = set(all_nodes) - set(utils.getCorosyncActiveNodes())
     if not unjoined_nodes:
         utils.err("no unjoined nodes found")
-    if modifiers.get("--force"):
+    if not modifiers.get("--force"):
         answer = utils.get_terminal_input(
             (
                 "WARNING: If node(s) {nodes} are not powered off or they do"
