@@ -20,11 +20,14 @@ corosync_qdevice_net_client_certs_dir = os.path.join(
 )
 corosync_qdevice_net_client_ca_file_name = "qnetd-cacert.crt"
 corosync_authkey_file = os.path.join(corosync_conf_dir, "authkey")
-corosync_authkey_bytes = 384
+# Must be set to 256 for corosync to work in FIPS environment.
+corosync_authkey_bytes = 256
 corosync_log_file = "/var/log/cluster/corosync.log"
 pacemaker_authkey_file = "/etc/pacemaker/authkey"
-pacemaker_authkey_bytes = 384
+# Using the same value as for corosync. Higher values MAY work in FIPS.
+pacemaker_authkey_bytes = 256
 booth_authkey_file_mode = 0o600
+# Booth does not support keys longer than 64 bytes.
 booth_authkey_bytes = 64
 cluster_conf_file = "/etc/cluster/cluster.conf"
 fence_agent_binaries = "/usr/sbin/"
@@ -32,16 +35,21 @@ pacemaker_schedulerd = "/usr/libexec/pacemaker/pacemaker-schedulerd"
 pacemaker_controld = "/usr/libexec/pacemaker/pacemaker-controld"
 pacemaker_based = "/usr/libexec/pacemaker/pacemaker-based"
 pacemaker_fenced = "/usr/libexec/pacemaker/pacemaker-fenced"
-pcs_version = "0.10.2"
+pcs_version = "0.10.4"
 crm_report = os.path.join(pacemaker_binaries, "crm_report")
+crm_rule = os.path.join(pacemaker_binaries, "crm_rule")
 crm_verify = os.path.join(pacemaker_binaries, "crm_verify")
 cibadmin = os.path.join(pacemaker_binaries, "cibadmin")
 crm_mon_schema = '/usr/share/pacemaker/crm_mon.rng'
 agent_metadata_schema = "/usr/share/resource-agents/ra-api-1.dtd"
-pcsd_cert_location = "/var/lib/pcsd/pcsd.crt"
-pcsd_key_location = "/var/lib/pcsd/pcsd.key"
-pcsd_users_conf_location = "/var/lib/pcsd/pcs_users.conf"
-pcsd_settings_conf_location = "/var/lib/pcsd/pcs_settings.conf"
+pcsd_var_location = "/var/lib/pcsd/"
+pcsd_cert_location = os.path.join(pcsd_var_location, "pcsd.crt")
+pcsd_key_location = os.path.join(pcsd_var_location, "pcsd.key")
+pcsd_known_hosts_location = os.path.join(pcsd_var_location, "known-hosts")
+pcsd_users_conf_location = os.path.join(pcsd_var_location, "pcs_users.conf")
+pcsd_settings_conf_location = os.path.join(
+    pcsd_var_location, "pcs_settings.conf"
+)
 pcsd_exec_location = "/usr/lib/pcsd/"
 pcsd_log_location = "/var/log/pcsd/pcsd.log"
 pcsd_default_port = 2224
