@@ -2,21 +2,27 @@ import os
 
 from lxml import etree
 
-from pcs_test.tools.command_env.mock_runner import Call as RunnerCall
+from pcs_test.tools.command_env.mock_runner import (
+    Call as RunnerCall,
+    CheckStdinEqualXml,
+)
 from pcs_test.tools.fixture import complete_state_resources
+from pcs_test.tools.fixture_cib import modify_cib
 from pcs_test.tools.misc import get_test_resource as rc
 from pcs_test.tools.xml import etree_to_str
 
-# pylint: disable=invalid-name, redefined-builtin, unused-argument, too-many-arguments, too-many-locals, too-many-boolean-expressions
+# pylint: disable=too-many-arguments
 
 DEFAULT_WAIT_TIMEOUT = 10
 WAIT_TIMEOUT_EXPIRED_RETURNCODE = 124
 AGENT_FILENAME_MAP = {
     "ocf:heartbeat:Dummy": "resource_agent_ocf_heartbeat_dummy.xml",
     "ocf:pacemaker:remote": "resource_agent_ocf_pacemaker_remote.xml",
+    "ocf:heartbeat:IPaddr2": "resource_agent_ocf_heartbeat_ipaddr2.xml",
+    "ocf:pacemaker:booth-site": "resource_agent_ocf_pacemaker_booth-site.xml",
 }
 
-def fixture_state_resources_xml(
+def _fixture_state_resources_xml(
     resource_id="A", resource_agent="ocf::heartbeat:Dummy", role="Started",
     failed="false", node_name="node1"
 ):
@@ -41,11 +47,17 @@ def fixture_state_resources_xml(
         )
     )
 
-def fixture_state_node_xml(
+def _fixture_state_node_xml(
     id, name, type="member", online=True, standby=False, standby_onfail=False,
     maintenance=False, pending=False, unclean=False, shutdown=False,
     expected_up=True, is_dc=False, resources_running=0
 ):
+    # This function uses a "clever" way of defaulting an input **dict containing
+    # attributes of an xml element.
+    # pylint: disable=unused-argument
+    # pylint: disable=invalid-name
+    # pylint: disable=redefined-builtin
+    # pylint: disable=too-many-locals
     attrs = locals()
     xml_attrs = []
     for attr_name, attr_value in attrs.items():
@@ -147,6 +159,7 @@ class PcmkShortcuts():
         string stderr -- crm_mon's stderr
         int returncode -- crm_mon's returncode
         """
+        # pylint: disable=too-many-boolean-expressions
         if (
             (resources or raw_resources is not None or nodes)
             and
@@ -173,10 +186,11 @@ class PcmkShortcuts():
             )
             return
 
-        state = etree.fromstring(open(rc(filename)).read())
+        with open(rc(filename)) as a_file:
+            state = etree.fromstring(a_file.read())
 
         if raw_resources is not None:
-            resources = fixture_state_resources_xml(**raw_resources)
+            resources = _fixture_state_resources_xml(**raw_resources)
         if resources:
             state.append(complete_state_resources(etree.fromstring(resources)))
 
@@ -184,7 +198,7 @@ class PcmkShortcuts():
             nodes_element = state.find("./nodes")
             for node in nodes:
                 nodes_element.append(
-                    etree.fromstring(fixture_state_node_xml(**node))
+                    etree.fromstring(_fixture_state_node_xml(**node))
                 )
 
         # set correct number of nodes and resources into the status
@@ -208,6 +222,63 @@ class PcmkShortcuts():
             )
         )
 
+    def load_state_plaintext(
+        self, name="runner.pcmk.load_state_plaintext",
+        inactive=True, verbose=False, fence_history=False,
+        stdout="", stderr="", returncode=0,
+    ):
+        """
+        Create a call for loading plaintext pacemaker status
+
+        str name -- key of the call
+        bool incative -- pass --inactive flag to crm_mon
+        bool verbose -- pass flags for increased verbosity to crm_mon
+        bool fence_history -- pass the flag for getting fence history to crm_mon
+        str stdout -- crm_mon's stdout
+        str stderr -- crm_mon's stderr
+        int returncode -- crm_mon's returncode
+        """
+        flags = ["--one-shot"]
+        if inactive:
+            flags.append("--inactive")
+        if verbose:
+            flags.extend([
+                "--show-detail", "--show-node-attributes", "--failcounts"
+            ])
+            if fence_history:
+                flags.append("--fence-history=3")
+        self.__calls.place(
+            name,
+            RunnerCall(
+                "crm_mon {}".format(" ".join(flags)),
+                stdout=stdout,
+                stderr=stderr,
+                returncode=returncode,
+            ),
+        )
+
+    def load_ticket_state_plaintext(
+        self, name="runner.pcmk.load_ticket_state_plaintext",
+        stdout="", stderr="", returncode=0,
+    ):
+        """
+        Create a call for loading plaintext tickets status
+
+        str name -- key of the call
+        str stdout -- crm_ticket's stdout
+        str stderr -- crm_ticket's stderr
+        int returncode -- crm_ticket's returncode
+        """
+        self.__calls.place(
+            name,
+            RunnerCall(
+                "crm_ticket --details",
+                stdout=stdout,
+                stderr=stderr,
+                returncode=returncode,
+            ),
+        )
+
     def load_agent(
         self,
         name="runner.pcmk.load_agent",
@@ -261,14 +332,15 @@ class PcmkShortcuts():
             )
             return
 
-        self.__calls.place(
-            name,
-            RunnerCall(
-                "crm_resource --show-metadata {0}".format(agent_name),
-                stdout=open(rc(agent_metadata_filename)).read()
-            ),
-            instead=instead,
-        )
+        with open(rc(agent_metadata_filename)) as a_file:
+            self.__calls.place(
+                name,
+                RunnerCall(
+                    "crm_resource --show-metadata {0}".format(agent_name),
+                    stdout=a_file.read()
+                ),
+                instead=instead,
+            )
 
     def load_fenced_metadata(
         self,
@@ -291,14 +363,14 @@ class PcmkShortcuts():
         string before -- the key of a call before which this new call is to be
             placed
         """
+        if stdout is None:
+            with open(rc("fenced_metadata.xml")) as a_file:
+                stdout = a_file.read()
         self.__calls.place(
             name,
             RunnerCall(
                 "/usr/libexec/pacemaker/pacemaker-fenced metadata",
-                stdout=(
-                    stdout if stdout is not None
-                    else open(rc("fenced_metadata.xml")).read()
-                ),
+                stdout=stdout,
                 stderr=stderr,
                 returncode=returncode
             ),
@@ -412,6 +484,8 @@ class PcmkShortcuts():
         string stderr -- crm_resource's stderr
         int returncode -- crm_resource's returncode
         """
+        # arguments are used via locals()
+        # pylint: disable=unused-argument
         all_args = locals()
         del all_args["self"]
         all_args["action"] = "--move"
@@ -446,6 +520,8 @@ class PcmkShortcuts():
         string stderr -- crm_resource's stderr
         int returncode -- crm_resource's returncode
         """
+        # arguments are used via locals()
+        # pylint: disable=unused-argument
         all_args = locals()
         del all_args["self"]
         all_args["action"] = "--ban"
@@ -480,6 +556,8 @@ class PcmkShortcuts():
         string stderr -- crm_resource's stderr
         int returncode -- crm_resource's returncode
         """
+        # arguments are used via locals()
+        # pylint: disable=unused-argument
         all_args = locals()
         del all_args["self"]
         all_args["action"] = "--clear"
@@ -591,7 +669,7 @@ class PcmkShortcuts():
             name,
             RunnerCall(
                 "crm_verify{0} {1}".format(
-                    " -V" if verbose else "",
+                    " -V -V" if verbose else "",
                     "--xml-file {0}".format(cib_tempfile) if cib_tempfile
                         else "--live-check"
                 ),
@@ -612,3 +690,52 @@ class PcmkShortcuts():
                 returncode=returncode,
             ),
         )
+
+    def simulate_cib(
+        self, new_cib_filepath, transitions_filepath,
+        cib_modifiers=None, cib_load_name="runner.cib.load",
+        stdout="", stderr="", returncode=0,
+        name="runner.pcmk.simulate_cib",
+        **modifier_shortcuts
+    ):
+        """
+        Create a call for simulating effects of cib changes
+
+        string new_cib_filepath -- a temp file for storing a new cib
+        string transitions_filepath -- a temp file for storing transitions
+        list of callable modifiers -- every callable takes etree.Element and
+            returns new etree.Element with desired modification
+        string cib_load_name -- key of a call from whose stdout the cib is taken
+        string stdout -- pacemaker's stdout
+        string stderr -- pacemaker's stderr
+        int returncode -- pacemaker's returncode
+        string name -- key of the call
+        dict modifier_shortcuts -- a new modifier is generated from each
+            modifier shortcut.
+            As key there can be keys of MODIFIER_GENERATORS.
+            Value is passed into appropriate generator from MODIFIER_GENERATORS.
+            For details see pcs_test.tools.fixture_cib (mainly the variable
+            MODIFIER_GENERATORS - please refer it when you are adding params
+            here)
+        """
+        cib_xml = modify_cib(
+            self.__calls.get(cib_load_name).stdout,
+            cib_modifiers,
+            **modifier_shortcuts
+        )
+        cmd = [
+            "crm_simulate", "--simulate",
+            "--save-output", new_cib_filepath,
+            "--save-graph", transitions_filepath,
+            "--xml-pipe",
+        ]
+        self.__calls.place(
+            name,
+            RunnerCall(
+                " ".join(cmd),
+                stdout=stdout,
+                stderr=stderr,
+                returncode=returncode,
+                check_stdin=CheckStdinEqualXml(cib_xml),
+            ),
+        )
