@@ -58,7 +58,7 @@ def fixture_master_xml(name, all_ops=True, meta_dict=None):
             [f'<meta_attributes id="{name}-master-meta_attributes">']
             +
             [
-                f'<nvpair id="{name}-master-meta_attributes-{key}"                   name="{key}" value="{val}"/>'
+                f'<nvpair id="{name}-master-meta_attributes-{key}" name="{key}" value="{val}"/>'
                 for key, val in meta_dict.items()
             ]
             +
@@ -223,6 +223,9 @@ MODIFIER_GENERATORS = {
     "replace": replace_all,
     "append": append_all,
     "resources": lambda xml: replace_all({"./configuration/resources": xml}),
+    "constraints": lambda xml: replace_all(
+        {"./configuration/constraints": xml}
+    ),
     "optional_in_conf": lambda xml: put_or_replace("./configuration", xml),
     #common modifier `put_or_replace` makes not sense - see explanation inside
     #this function - all occurences should be satisfied by `optional_in_conf`
