@@ -40,7 +40,7 @@ def get_pcs_internal_path()
   end
 end
 
-PCS_VERSION = '0.10.2'
+PCS_VERSION = '0.10.4'
 # unique instance signature, allows detection of dameon restarts
 COROSYNC = COROSYNC_BINARIES + "corosync"
 ISSYSTEMCTL = is_systemctl
@@ -48,7 +48,7 @@ COROSYNC_CMAPCTL = COROSYNC_BINARIES + "corosync-cmapctl"
 COROSYNC_QUORUMTOOL = COROSYNC_BINARIES + "corosync-quorumtool"
 
 if not defined? $cur_node_name
-  $cur_node_name = `hostname`.chomp
+  $cur_node_name = `/bin/hostname`.chomp
 end
 
 def configure_logger(log_device)
