@@ -1023,16 +1023,11 @@ def pcs_auth(auth_user, nodes)
   sync_successful, sync_responses = Cfgsync::save_sync_new_known_hosts(
     new_hosts, [], cluster_nodes, $cluster_name
   )
-  sync_responses.each { |node, response|
-    if response['status'] != 'ok'
-      sync_failed_nodes << node
-    else
-      node_result = response['result'][Cfgsync::PcsdKnownHosts.name]
-      if not ['accepted', 'rejected'].include?(node_result)
-        sync_failed_nodes << node
-      end
-    end
-  }
+  # sync_notauthorized_nodes is not used because clients extract that info from
+  # auth_responses themselves
+  sync_notauthorized_nodes, sync_failed_nodes = (
+    Cfgsync::get_failed_nodes_from_sync_responses(sync_responses)
+  )
   return auth_responses, sync_successful, sync_failed_nodes, sync_responses
 end
 
@@ -1445,7 +1440,7 @@ def cluster_status_from_nodes(auth_user, cluster_nodes, cluster_name)
 end
 
 def get_node_uptime()
-  uptime = `cat /proc/uptime`.chomp.split(' ')[0].split('.')[0].to_i
+  uptime = `/bin/cat /proc/uptime`.chomp.split(' ')[0].split('.')[0].to_i
   mm, ss = uptime.divmod(60)
   hh, mm = mm.divmod(60)
   dd, hh = hh.divmod(24)
@@ -1720,7 +1715,10 @@ end
 
 def is_service_running?(service)
   if ISSYSTEMCTL
-    cmd = ['systemctl', 'status', "#{service}.service"]
+    # --lines=0 disables listing last N lines from journal related to the
+    # service. The lines may contain non-ASCII characters which cause various
+    # encoding/decoding errors. We are not interested in the lines anyway.
+    cmd = ['systemctl', 'status', '--lines=0', "#{service}.service"]
   else
     cmd = ['service', service, 'status']
   end
@@ -1728,35 +1726,72 @@ def is_service_running?(service)
   return (retcode == 0)
 end
 
-def is_service_installed?(service)
-  unless ISSYSTEMCTL
-    stdout, _, retcode = run_cmd(PCSAuth.getSuperuserAuth(), 'chkconfig')
-    if retcode != 0
+class ServiceInstalledChecker
+  def initialize()
+    @list_unit_files_output = self.load_unit_files_list()
+  end
+
+  def is_installed?(service)
+    if @list_unit_files_output.nil?
       return nil
     end
-    stdout.each { |line|
-      if line.split(' ')[0] == service
+
+    @list_unit_files_output.each { |line|
+      if self.contains_line_service?(line, service)
         return true
       end
     }
     return false
   end
 
-  # currently we are not using systemd instances (service_name@instance) in pcsd
-  # for proper implementation of is_service_installed see
-  # pcs/lib/external.py:is_service_installed
-  stdout, _, retcode = run_cmd(
-    PCSAuth.getSuperuserAuth(), 'systemctl', 'list-unit-files', '--full'
-  )
-  if retcode != 0
-    return nil
-  end
-  stdout.each { |line|
-    if line.strip().start_with?("#{service}.service")
-      return true
+  protected
+  def load_unit_files_list()
+    stdout, _, retcode = self.run_command()
+    if retcode != 0
+      return nil
     end
-  }
-  return false
+    return stdout
+  end
+end
+
+class ServiceInstalledCheckerSystemctl < ServiceInstalledChecker
+  protected
+  def run_command
+    # currently we are not using systemd instances (service_name@instance) in pcsd
+    # for proper implementation of is_service_installed see
+    # pcs/lib/external.py:is_service_installed
+    return run_cmd(
+      PCSAuth.getSuperuserAuth(), 'systemctl', 'list-unit-files', '--full'
+    )
+  end
+
+  def contains_line_service?(line, service)
+    return line.strip().start_with?("#{service}.service")
+  end
+end
+
+class ServiceInstalledCheckerChkconfig < ServiceInstalledChecker
+  protected
+  def run_command
+    return run_cmd(PCSAuth.getSuperuserAuth(), 'chkconfig')
+  end
+
+  def contains_line_service?(line, service)
+    return line.split(' ')[0] == service
+  end
+end
+
+def get_service_installed_checker
+  if ISSYSTEMCTL
+    return ServiceInstalledCheckerSystemctl.new
+  else
+    return ServiceInstalledCheckerChkconfig.new
+  end
+end
+
+
+def is_service_installed?(service)
+  return get_service_installed_checker().is_installed?(service)
 end
 
 def enable_service(service)
@@ -1888,9 +1923,9 @@ def get_alerts(auth_user)
   end
 end
 
-def get_service_info(service_name)
+def get_service_info(service_name, service_checker)
   return {
-    :installed => is_service_installed?(service_name),
+    :installed => service_checker.is_installed?(service_name),
     :enabled => is_service_enabled?(service_name),
     :running => is_service_running?(service_name),
     :version => nil,
