@@ -37,7 +37,7 @@ function menu_show(item,show) {
 // and load the default item on the specified page if item is set
 function select_menu(menu, item, initial) {
   if (menu == "NODES") {
-    Pcs.set('cur_page',"nodes")
+    Pcs.set('cur_page',"nodes");
     if (item)
       Pcs.nodesController.load_node($('[nodeID='+item+']'));
     menu_show("node", true);
@@ -86,13 +86,6 @@ function select_menu(menu, item, initial) {
   } else {
     menu_show("acls", false);
   }
-
-  if (menu == "WIZARDS") {
-    Pcs.set('cur_page',"wizards");
-    menu_show("wizards", true);
-  } else {
-    menu_show("wizards", false);
-  }
 }
 
 function create_group() {
@@ -169,45 +162,6 @@ function create_group() {
   });
 }
 
-function add_node_dialog() {
-  var buttonOpts = [
-    {
-      text: "Add Node",
-      id: "add_node_submit_btn",
-      click: function() {
-        $("#add_node_submit_btn").button("option", "disabled", true);
-        checkAddingNode();
-      }
-    },
-    {
-      text: "Cancel",
-      click: function() {
-        $(this).dialog("close");
-      }
-    }
-  ];
-
-  buttonOpts["Cancel"] = function() {
-    $(this).dialog("close");
-  };
-
-  // If you hit enter it triggers the first button: Add Node
-  $('#add_node').keypress(function(e) {
-    if (e.keyCode == $.ui.keyCode.ENTER && !$("#add_node_submit_btn").button("option", "disabled")) {
-        $("#add_node_submit_btn").trigger("click");
-      return false;
-    }
-  });
-
-  $('#add_node').dialog({
-    title: 'Add Node',
-    modal:true,
-    resizable: false,
-    width: 'auto',
-    buttons: buttonOpts
-  });
-}
-
 function add_sbd_device_textbox() {
   var max_count = 3;
   var device_inputs = $("#add_node_selector .add_node_sbd_device");
@@ -223,7 +177,7 @@ function add_sbd_device_textbox() {
           </a> \
         </td> \
       </tr>'
-    )
+    );
   }
 }
 
@@ -231,61 +185,6 @@ function sbd_device_remove_textbox(obj) {
   $(obj).parents(".add_node_sbd_device").remove();
 }
 
-function checkAddingNode(){
-  var nodeName = $("#add_node").children("form").find("[name='new_nodename']").val().trim();
-  if (nodeName == "") {
-    $("#add_node_submit_btn").button("option", "disabled", false);
-    return false;
-  }
-  var port = $("#add_node_selector input.port").val().trim();
-  var data = {};
-  data["nodes"] = nodeName;
-  data["port-" + nodeName] = port;
-
-  ajax_wrapper({
-    type: 'GET',
-    url: '/manage/check_pcsd_status',
-    data: data,
-    timeout: pcs_timeout,
-    success: function (data) {
-      var mydata = jQuery.parseJSON(data);
-      if (mydata[nodeName] == "Unable to authenticate") {
-        var nodes_to_auth = {};
-        nodes_to_auth[nodeName] = port;
-        auth_nodes_dialog(nodes_to_auth, function(){$("#add_node_submit_btn").trigger("click");});
-        $("#add_node_submit_btn").button("option", "disabled", false);
-      } else if (mydata[nodeName] == "Offline") {
-        alert("Unable to contact node '" + nodeName + "'");
-        $("#add_node_submit_btn").button("option", "disabled", false);
-      } else {
-        create_node($("#add_node").children("form"));
-      }
-    },
-    error: function (XMLHttpRequest, textStatus, errorThrown) {
-      alert("ERROR: Unable to contact server");
-      $("#add_node_submit_btn").button("option", "disabled", false);
-    }
-  });
-}
-
-function create_node(form) {
-  var dataString = $(form).serialize();
-  ajax_wrapper({
-    type: "POST",
-    url: get_cluster_remote_url() + "add_node_to_cluster",
-    data: dataString,
-    success: function(returnValue) {
-      $("#add_node_submit_btn").button("option", "disabled", false);
-      $('#add_node').dialog('close');
-      Pcs.update();
-    },
-    error: function(error) {
-      alert(error.responseText);
-      $("#add_node_submit_btn").button("option", "disabled", false);
-    }
-  });
-}
-
 function create_resource_error_processing(error_message, form, update, stonith) {
   var message = (
     "Unable to " + (update ? "update " : "add ") + name + "\n" + error_message
@@ -296,7 +195,7 @@ function create_resource_error_processing(error_message, form, update, stonith)
   else {
     message = message.replace(', use --force to override', '');
     if (confirm(message + "\n\nDo you want to force the operation?")) {
-      create_resource(form, update, stonith, true)
+      create_resource(form, update, stonith, true);
     }
   }
 }
@@ -373,14 +272,20 @@ function load_resource_form(agent_name, stonith) {
   }, stonith);
 }
 
-function verify_remove(remove_func, forceable, checklist_id, dialog_id, label, ok_text, title, remove_id) {
-  var remove_id_list = new Array();
-  if (remove_id) {
+function verify_remove(
+  remove_func, forceable, checklist_id, dialog_id, label, ok_text, title,
+  remove_id, extraDialogOptions
+) {
+  var remove_id_list;
+
+  if (Array.isArray(remove_id)) {
+    remove_id_list = remove_id;
+  }else if (remove_id) {
     remove_id_list = [remove_id];
-  }
-  else {
+  } else {
     remove_id_list = get_checked_ids_from_nodelist(checklist_id);
   }
+
   if (remove_id_list.length < 1) {
     alert("You must select at least one " + label + " to remove.");
     return;
@@ -396,7 +301,7 @@ function verify_remove(remove_func, forceable, checklist_id, dialog_id, label, o
         }
         $("#verify_remove_submit_btn").button("option", "disabled", true);
         if (forceable) {
-          force = $("#" + dialog_id + " :checked").length > 0
+          force = $("#" + dialog_id + " :checked").length > 0;
           remove_func(remove_id_list, force);
         }
         else {
@@ -422,12 +327,12 @@ function verify_remove(remove_func, forceable, checklist_id, dialog_id, label, o
   });
   name_list += "</ul>";
   $("#" + dialog_id + " .name_list").html(name_list);
-  $("#" + dialog_id).dialog({
-    title: title,
-    modal: true,
-    resizable: false,
-    buttons: buttonOpts
-  });
+  dialogOptions = extraDialogOptions || {};
+  dialogOptions.title = title;
+  dialogOptions.modal = true;
+  dialogOptions.resizable = false,
+  dialogOptions.buttons = buttonOpts;
+  $("#" + dialog_id).dialog(dialogOptions);
 }
 
 function verify_remove_clusters(cluster_id) {
@@ -437,6 +342,12 @@ function verify_remove_clusters(cluster_id) {
   );
 }
 
+/*
+ * TODO Remove - dead code
+ * This function was handling the old node remove dialog used in pcs-0.9 for
+ * CMAN and Corosync 2 clusters. We keep it here for now until the dialog
+ * overhaul is done.
+*/
 function verify_remove_nodes(node_id) {
   verify_remove(
     remove_nodes, false, "node_list", "dialog_verify_remove_nodes",
@@ -466,7 +377,7 @@ function verify_remove_acl_roles(role_id) {
 }
 
 function get_checked_ids_from_nodelist(nodelist_id) {
-  var ids = new Array()
+  var ids = new Array();
   $("#" + nodelist_id + " .node_list_check :checked").each(function (index, element) {
     if($(element).parent().parent().attr("nodeID")) {
       ids.push($(element).parent().parent().attr("nodeID"));
@@ -510,7 +421,7 @@ function setStatus(item, status, message) {
   }
 
   if (typeof message !== 'undefined')
-    item.html(message)
+    item.html(message);
 }
 
 // Set the node in the node list blue or red depending on
@@ -635,9 +546,9 @@ function resource_stonith_cleanup_refresh(
   if (resource == null) {
     return;
   }
-  data = {"resource": resource}
+  data = {"resource": resource};
   if (full_refresh) {
-    data["full"] = 1
+    data["full"] = 1;
   }
   fade_in_out(action_button_id);
   ajax_wrapper({
@@ -661,7 +572,7 @@ function cleanup_resource() {
     "#resource_cleanup_link",
     "resource_cleanup",
     "cleanup"
-  )
+  );
 }
 
 function cleanup_stonith() {
@@ -670,7 +581,7 @@ function cleanup_stonith() {
     "#stonith_cleanup_link",
     "resource_cleanup",
     "cleanup"
-  )
+  );
 }
 
 function refresh_resource(refresh_supported) {
@@ -684,7 +595,7 @@ function refresh_resource(refresh_supported) {
      * (yet);
      * moreover the full refresh flag only works for refresh not for cleanup */
     refresh_supported ? true : false
-  )
+  );
 }
 
 function refresh_stonith(refresh_supported) {
@@ -698,7 +609,7 @@ function refresh_stonith(refresh_supported) {
      * (yet);
      * moreover the full refresh flag only works for refresh not for cleanup */
     refresh_supported ? true : false
-  )
+  );
 }
 
 function checkExistingNode() {
@@ -706,19 +617,13 @@ function checkExistingNode() {
   $('input[name="node-name"]').each(function(i,e) {
     node = e.value;
   });
-  var port = $("#add_existing_cluster_form input.port").val().trim();
-  var data = {};
-  data["nodes"] = node;
-  data["port-" + node] = port;
-
   ajax_wrapper({
     type: 'GET',
-    url: '/manage/check_pcsd_status',
-    data: data,
+    url: '/manage/check_auth_against_nodes',
+    data: {"node_list": [node]},
     timeout: pcs_timeout,
     success: function (data) {
-      mydata = jQuery.parseJSON(data);
-      update_existing_cluster_dialog(mydata);
+      update_existing_cluster_dialog(jQuery.parseJSON(data));
 
     },
     error: function (XMLHttpRequest, textStatus, errorThrown) {
@@ -727,61 +632,69 @@ function checkExistingNode() {
   });
 }
 
-function get_node_ports_object_from_create_new_cluster_dialog() {
-  var nodes = {}
-  $("#create_new_cluster_form tr.new_node").each(function(i, e) {
-    var node = $(e).find(".node").val().trim();
-    var port = $(e).find(".port").val().trim();
-    nodes[node] = port;
-  });
-  return nodes;
-}
-
-function checkClusterNodes() {
-  var nodes = get_node_ports_object_from_create_new_cluster_dialog();
-  var node_list_str = Object.keys(nodes).join(",");
-  var request_data = {
-    "nodes": node_list_str,
-  };
-  $.each(nodes, function(node, port) {
-    if (port) {
-      request_data["port-" + node] = port;
-    }
-  });
-
-  ajax_wrapper({
-    type: 'GET',
-    url: '/manage/check_pcsd_status',
-    data: request_data,
-    timeout: pcs_timeout,
-    success: function (data) {
-      mydata = jQuery.parseJSON(data);
-      ajax_wrapper({
-        type: 'GET',
-        url: '/manage/get_nodes_sw_versions',
-        data: {"nodes": node_list_str},
-        timeout: pcs_timeout,
-        success: function(data) {
-          versions = jQuery.parseJSON(data);
-          update_create_cluster_dialog(mydata, versions);
-        },
-        error: function (XMLHttpRequest, textStatus, errorThrown) {
-          alert("ERROR: Unable to contact server");
-        }
-      });
-    },
-    error: function (XMLHttpRequest, textStatus, errorThrown) {
-      alert("ERROR: Unable to contact server");
-    }
-  });
-}
-
 function auth_nodes(dialog) {
   $("#auth_failed_error_msg").hide();
+  var form = dialog.find("#auth_nodes_form");
+  var nodes = {};
+  var custom_addr_port = form.find('input:checkbox[name=custom_addr_port]').prop('checked');
+  form.find('#auth_nodes_list input').each(function() {
+    var input = $(this);
+    if(input.attr('name').startsWith('pass-')) {
+      var nodename = input.attr('name').slice('pass-'.length);
+      if(!nodes[nodename]) {
+        nodes[nodename] = {"password": "", "addr": "", "port": ""};
+      }
+      nodes[nodename]['password'] = input.val();
+    }
+    else if(custom_addr_port && input.attr('name').startsWith('port-')) {
+      var nodename = input.attr('name').slice('port-'.length);
+      if(!nodes[nodename]) {
+        nodes[nodename] = {"password": "", "addr": "", "port": ""};
+      }
+      nodes[nodename]['port'] = input.val().trim();
+      if(nodes[nodename]['port'].length < 1) {
+        nodes[nodename]['port'] = input.prop('placeholder');
+      }
+    }
+    else if(custom_addr_port && input.attr('name').startsWith('addr-')) {
+      var nodename = input.attr('name').slice('addr-'.length);
+      if(!nodes[nodename]) {
+        nodes[nodename] = {"password": "", "addr": "", "port": ""};
+      }
+      nodes[nodename]['addr'] = input.val().trim();
+      if(nodes[nodename]['addr'].length < 1) {
+        nodes[nodename]['addr'] = nodename;
+      }
+    }
+  });
+  if(form.find('input:checkbox[name=all]').prop('checked')) {
+    var passwd_for_all = form.find('input[name=pass-all]').val();
+    $.each(nodes, function(node_name, node_data) {
+      node_data.password = passwd_for_all;
+    });
+  }
+
+  var nodes_for_request = {};
+  $.each(nodes, function(node_name, node_data) {
+    nodes_for_request[node_name] = {
+      password: node_data['password'],
+      dest_list: [
+        {
+          addr: node_data['addr'],
+          port: node_data['port']
+        }
+      ]
+    };
+  });
+
   ajax_wrapper({
     type: 'POST',
     url: '/manage/auth_gui_against_nodes',
-    data: dialog.find("#auth_nodes_form").serialize(),
+    data: {
+      data_json: JSON.stringify({
+        nodes: nodes_for_request,
+      })
+    },
     timeout: pcs_timeout,
     success: function (data) {
       mydata = jQuery.parseJSON(data);
@@ -796,6 +709,13 @@ function auth_nodes(dialog) {
 function auth_nodes_dialog_update(dialog_obj, data) {
   var unauth_nodes = [];
   var node;
+  var plaintext_error = "";
+  var cannot_save_new_hosts = false;
+
+  if (data['plaintext_error']) {
+    plaintext_error += "\n\n" + data['plaintext_error'];
+  }
+
   if (data['node_auth_error']) {
     for (node in data['node_auth_error']) {
       if (data['node_auth_error'][node] != 0) {
@@ -804,6 +724,28 @@ function auth_nodes_dialog_update(dialog_obj, data) {
     }
   }
 
+  if (data['local_cluster_node_auth_error']) {
+    for (node in data['local_cluster_node_auth_error']) {
+      if (data['local_cluster_node_auth_error'][node] != 0) {
+        unauth_nodes.push(node);
+        cannot_save_new_hosts = true;
+      }
+    }
+  }
+  if (cannot_save_new_hosts) {
+    plaintext_error += (
+      "\n\n"
+      +
+      "Unable to save new cluster settings as the local cluster nodes are not "
+      +
+      "authenticated. Please, authenticate them as well."
+    );
+  }
+
+  if (plaintext_error.length > 0) {
+    alert($.trim(plaintext_error));
+  }
+
   var callback_one = dialog_obj.dialog("option", "callback_success_one_");
   var callback = dialog_obj.dialog("option", "callback_success_");
   if (unauth_nodes.length == 0) {
@@ -821,17 +763,34 @@ function auth_nodes_dialog_update(dialog_obj, data) {
     dialog_obj.find("#auth_failed_error_msg").show();
   }
 
-  if (unauth_nodes.length == 1) {
-    dialog_obj.find("#same_pass").hide();
-    dialog_obj.find('#auth_nodes_list').find('input:password').each(
-      function(){$(this).show()}
-    );
-  }
+  // add rows for local cluster nodes which are not authenticated
+  $.each(unauth_nodes, function(i, node) {
+    if (
+      dialog_obj.find("input:password[name='pass-" + node + "']").length == 0
+    ) {
+      auth_nodes_dialog_add_node_row(dialog_obj, node);
+    }
+  });
+  // hide advanced settings for new rows if appropriate
+  dialog_obj.find("input:checkbox[name='custom_addr_port']").each(
+    function(){
+      if ($(this).is(':checked')) {
+        $('#auth_nodes_form').find('.addr_port').show();
+      } else {
+        $('#auth_nodes_form').find('.addr_port').hide();
+      }
+    }
+  );
+  auth_nodes_dialog_toggle_same_pass(dialog_obj, unauth_nodes.length);
 
   var one_success = false;
-  dialog_obj.find("input:password[name$=-pass]").each(function() {
+  dialog_obj.find("input:password[name^=pass-]").each(function() {
     node = $(this).attr("name");
-    node = node.substring(0, node.length - 5);
+    if (node == "pass-all") {
+      // do not remove / modify the row common to all nodes
+      return;
+    }
+    node = node.substring("pass-".length);
     if (unauth_nodes.indexOf(node) == -1) {
       $(this).parent().parent().remove();
       one_success = true;
@@ -849,10 +808,9 @@ function auth_nodes_dialog_update(dialog_obj, data) {
   return unauth_nodes;
 }
 
-function auth_nodes_dialog(unauth_nodes, callback_success, callback_success_one, ports_required) {
+function auth_nodes_dialog(unauth_nodes, callback_success, callback_success_one) {
   callback_success = typeof callback_success !== 'undefined' ? callback_success : null;
   callback_success_one = typeof callback_success_one !== 'undefined' ? callback_success_one : null;
-  ports_required = typeof ports_required !== 'undefined' ? ports_required : false;
 
   var buttonsOpts = [
     {
@@ -874,8 +832,10 @@ function auth_nodes_dialog(unauth_nodes, callback_success, callback_success_one,
       }
     }
   ];
-  var dialog_obj = $("#auth_nodes").dialog({title: 'Authentification of nodes',
-    modal: true, resizable: false,
+  var dialog_obj = $("#auth_nodes").dialog({
+    title: 'Authentication of nodes',
+    modal: true,
+    resizable: false,
     width: 'auto',
     buttons: buttonsOpts,
     callback_success_: callback_success,
@@ -891,7 +851,7 @@ function auth_nodes_dialog(unauth_nodes, callback_success, callback_success_one,
       return false;
     }
   });
-  unauth_nodes_count = Object.keys(unauth_nodes).length;
+  unauth_nodes_count = unauth_nodes.length;
 
   if (unauth_nodes_count == 0) {
     if (callback_success !== null) {
@@ -900,28 +860,40 @@ function auth_nodes_dialog(unauth_nodes, callback_success, callback_success_one,
     return;
   }
 
+  auth_nodes_dialog_toggle_same_pass(dialog_obj, unauth_nodes_count);
+
+  dialog_obj.find("input:checkbox[name=custom_addr_port]").prop("checked", false);
+  dialog_obj.find('#auth_nodes_list').empty();
+  dialog_obj.find('#auth_nodes_list').append('<tr><td>Node</td><td class="password">Password</td><td class="addr_port">Address</td><td class="addr_port">Port</td></tr>');
+  $.each(unauth_nodes, function(i, node) {
+    auth_nodes_dialog_add_node_row(dialog_obj, node)
+  });
+  dialog_obj.find(".addr_port").hide();
+
+}
+
+function auth_nodes_dialog_toggle_same_pass(dialog_obj, unauth_nodes_count) {
   if (unauth_nodes_count == 1) {
     dialog_obj.find("#same_pass").hide();
+    dialog_obj.find('#auth_nodes_list').find('input:password').each(
+      function(){$(this).show()}
+    );
   } else {
     dialog_obj.find("#same_pass").show();
     dialog_obj.find("input:checkbox[name=all]").prop("checked", false);
     dialog_obj.find("#pass_for_all").val("");
     dialog_obj.find("#pass_for_all").hide();
   }
+}
 
-  dialog_obj.find('#auth_nodes_list').empty();
-  $.each(unauth_nodes, function(node, port) {
-    var html = "<tr><td>" + htmlEncode(node) + "</td><td>";
-    if (ports_required) {
-      html += ':<input type="text" size="5" name="port-' + htmlEncode(node) + '" placeholder="2224" />';
-    } else {
-      html += '<input type="hidden" name="port-' + htmlEncode(node) + '" value="' + htmlEncode(port) + '" />';
-    }
-    html += '</td><td><input type="password" name="' + htmlEncode(node) + '-pass"></td></tr>';
-
+function auth_nodes_dialog_add_node_row(dialog_obj, node) {
+    var html = "<tr>";
+    html += "<td>" + htmlEncode(node) + "</td>";
+    html += '<td class="password"><input type="password" name="pass-' + htmlEncode(node) + '"></td>';
+    html += '<td class="addr_port"><input type="text" name="addr-' + htmlEncode(node) + '"></td>';
+    html += '<td class="addr_port">:<input type="text" size="5" name="port-' + htmlEncode(node) + '" placeholder="2224" /></td>';
+    html += "</tr>";
     dialog_obj.find('#auth_nodes_list').append(html);
-  });
-
 }
 
 function add_existing_dialog() {
@@ -959,8 +931,8 @@ function add_existing_dialog() {
 }
 
 function update_existing_cluster_dialog(data) {
-  for (var i in data) {
-    if (data[i] == "Online") {
+  for (var node in data) {
+    if (data[node] == "Online") {
       ajax_wrapper({
         type: "POST",
         url: "/manage/existingcluster",
@@ -979,10 +951,15 @@ function update_existing_cluster_dialog(data) {
         }
       });
       return;
-    } else if (data[i] == "Unable to authenticate") {
-      var nodes_to_auth = {};
-      nodes_to_auth[i] = $("#add_existing_cluster_form input.port").val().trim();
-      auth_nodes_dialog(nodes_to_auth, function() {$("#add_existing_submit_btn").trigger("click");});
+    } else if (
+      (data[node] == "Unable to authenticate")
+      ||
+      (data[node] == "Offline")
+    ) {
+      if(data[node] == "Offline") {
+        alert("Unable to contact node '" + node + "'");
+      }
+      auth_nodes_dialog([node], function() {$("#add_existing_submit_btn").trigger("click")});
       $("#add_existing_submit_btn").button("option", "disabled", false);
       return;
     }
@@ -996,266 +973,6 @@ function update_existing_cluster_dialog(data) {
   $("#add_existing_submit_btn").button("option", "disabled", false);
 }
 
-function update_create_cluster_dialog(nodes, version_info) {
-  var keys = [];
-  for (var i in nodes) {
-    if (nodes.hasOwnProperty(i)) {
-      keys.push(i);
-    }
-  }
-
-  var cant_connect_nodes = 0;
-  var cant_auth_nodes = [];
-  var good_nodes = 0;
-  var addr1_match = 1;
-  var ring0_nodes = [];
-  var ring1_nodes = [];
-  var cman_nodes = [];
-  var noncman_nodes = [];
-  var rhel_versions = [];
-  var versions_check_ok = 1;
-  var cluster_name = $('input[name^="clustername"]').val()
-  var transport = $("#create_new_cluster select[name='config-transport']").val()
-
-    $('#create_new_cluster input[name^="node-"]').each(function() {
-      if ($(this).val() == "") {
-        $(this).parent().prev().css("background-color", "");
-        return;
-      }
-      for (var i = 0; i < keys.length; i++) {
-        if ($(this).val() == keys[i]) {
-          if (nodes[keys[i]] != "Online") {
-            if (nodes[keys[i]] == "Unable to authenticate") {
-              cant_auth_nodes.push(keys[i]);
-            } else {
-              $(this).parent().prev().css("background-color", "red");
-              cant_connect_nodes++;
-            }
-          } else {
-            $(this).parent().prev().css("background-color", "");
-            good_nodes++;
-          }
-        }
-      }
-    });
-
-    if (cant_auth_nodes.length > 0) {
-      var ports = get_node_ports_object_from_create_new_cluster_dialog();
-      var unauth_nodes_with_ports = {};
-      $.each(cant_auth_nodes, function(i, node) {
-        unauth_nodes_with_ports[node] = ports[node];
-      });
-      auth_nodes_dialog(unauth_nodes_with_ports, function(){$("#create_cluster_submit_btn").trigger("click")});
-      $("#create_cluster_submit_btn").button("option", "disabled", false);
-      return;
-    }
-
-  if (transport == "udpu") {
-    $('#create_new_cluster input[name^="node-"]').each(function() {
-      if ($(this).val().trim() != "") {
-        ring0_nodes.push($(this).attr("name"));
-      }
-    });
-    $('#create_new_cluster input[name^="ring1-node-"]').each(function() {
-      if ($(this).val().trim() != "") {
-        ring1_nodes.push($(this).attr("name").substr("ring1-".length));
-      }
-    });
-    if (ring1_nodes.length > 0) {
-      if (ring0_nodes.length != ring1_nodes.length) {
-        addr1_match = 0
-      }
-      else {
-        for (var i = 0; i < ring0_nodes.length; i++) {
-          if (ring0_nodes[i] != ring1_nodes[i]) {
-            addr1_match = 0;
-            break;
-          }
-        }
-      }
-    }
-  }
-
-  if(version_info) {
-    $.each(version_info, function(node, versions) {
-      if(! versions["pcs"]) {
-        // we do not have valid info for this node
-        return;
-      }
-      if(versions["cman"]) {
-        cman_nodes.push(node);
-      }
-      else {
-        noncman_nodes.push(node);
-      }
-      if(versions["rhel"]) {
-        if($.inArray(versions["rhel"].join("."), rhel_versions) == -1) {
-          rhel_versions.push(versions["rhel"].join("."))
-        }
-      }
-    });
-  }
-
-  if (cant_connect_nodes != 0) {
-    $("#unable_to_connect_error_msg").show();
-  } else {
-    $("#unable_to_connect_error_msg").hide();
-  }
-
-  if (good_nodes == 0 && cant_connect_nodes == 0) {
-    $("#at_least_one_node_error_msg").show();
-  } else {
-    $("#at_least_one_node_error_msg").hide();
-  }
-
-  if (cluster_name == "") {
-    $("#bad_cluster_name_error_msg").show();
-  } else {
-    $("#bad_cluster_name_error_msg").hide();
-  }
-
-  if (addr1_match == 0) {
-    $("#addr0_addr1_mismatch_error_msg").show();
-  }
-  else {
-    $("#addr0_addr1_mismatch_error_msg").hide();
-  }
-  if(versions) {
-    if(cman_nodes.length > 0 && transport == "udpu") {
-      if(noncman_nodes.length < 1 && ring1_nodes.length < 1) {
-        transport = "udp";
-        $("#create_new_cluster select[name='config-transport']").val(transport);
-        create_cluster_display_rrp(transport);
-      }
-      else {
-        versions_check_ok = 0;
-        $("#cman_udpu_transport_error_msg").show();
-      }
-    }
-    else {
-      $("#cman_udpu_transport_error_msg").hide();
-    }
-
-    if(cman_nodes.length > 1 && noncman_nodes.length > 1) {
-      versions_check_ok = 0;
-      $("#cman_mismatch_error_msg").show();
-    }
-    else {
-      $("#cman_mismatch_error_msg").hide();
-    }
-
-    if(rhel_versions.length > 1) {
-      versions_check_ok = 0;
-      $("#rhel_version_mismatch_error_msg").show();
-    }
-    else {
-      $("#rhel_version_mismatch_error_msg").hide();
-    }
-  }
-  else {
-    $("#cman_udpu_transport_error_msg").hide();
-    $("#cman_mismatch_error_msg").hide();
-    $("#rhel_version_mismatch_error_msg").hide();
-  }
-
-  if (good_nodes != 0 && cant_connect_nodes == 0 && cant_auth_nodes.length == 0 && cluster_name != "" && addr1_match == 1 && versions_check_ok == 1) {
-    ajax_wrapper({
-      type: "POST",
-      url: "/manage/newcluster",
-      timeout: 60*1000,
-      data: $('#create_new_cluster_form').serialize(),
-      success: function(data) {
-        if (data) {
-          alert("Operation Successful!\n\nWarnings:\n" + data);
-        }
-        $("#create_new_cluster.ui-dialog-content").each(function(key, item) {$(item).dialog("destroy")});
-        Pcs.update();
-      },
-      error: function (xhr, status, error) {
-        var err_msg = "";
-        if ((status == "timeout") || ($.trim(error) == "timeout")) {
-          err_msg = (
-            "Operation takes longer to complete than expected. " +
-            "It may continue running in the background. Later, you can try " +
-            "to add this cluster as existing one."
-          );
-        } else {
-          err_msg = xhr.responseText;
-        }
-        alert(err_msg);
-        $("#create_cluster_submit_btn").button("option", "disabled", false);
-      }
-    });
-  } else {
-    $("#create_cluster_submit_btn").button("option", "disabled", false);
-  }
-
-}
-
-function create_cluster_dialog() {
-  var buttonOpts = [{
-    text: "Create Cluster",
-    id: "create_cluster_submit_btn",
-    click: function() {
-      $("#create_new_cluster").find("table.err_msg_table").find("span[id$=_error_msg]").hide();
-      $("#create_cluster_submit_btn").button("option", "disabled", true);
-      checkClusterNodes();
-    }
-  },
-  {
-    text: "Cancel",
-    id: "create_cluster_cancel_btn",
-    click: function() {
-      $(this).dialog("close");
-    }
-  }]
-
-  $("#create_new_cluster").dialog({title: 'Create Cluster',
-    modal: false, resizable: false,
-    width: 'auto',
-    buttons: buttonOpts
-  });
-}
-
-function create_cluster_add_nodes() {
-  node_list = $("#create_new_cluster_form tr").has("input[name^='node-']");;
-  var ring1_node_list = $("#create_new_cluster_form tr").has(
-    "input[name^='ring1-node-']"
-  );
-  cur_num_nodes = node_list.length;
-
-  first_node = node_list.eq(0);
-  new_node = first_node.clone();
-  $("input[name=node-1]",new_node).attr("name", "node-"+(cur_num_nodes+1));
-  $("input[name=port-1]",new_node).attr("name", "port-"+(cur_num_nodes+1));
-  $("input",new_node).val("");
-  $("td", new_node).first().text("Node " + (cur_num_nodes+1)+ ":");
-  new_node.insertAfter(node_list.last());
-
-  var ring1_first_node = ring1_node_list.eq(0);
-  var ring1_new_node = ring1_first_node.clone();
-  $("input", ring1_new_node).attr("name", "ring1-node-" + (cur_num_nodes + 1));
-  $("input", ring1_new_node).val("");
-  $("td", ring1_new_node).first().text(
-    "Node " + (cur_num_nodes+1) + " (Ring 1):"
-  );
-  ring1_new_node.insertAfter(ring1_node_list.last());
-
-  if (node_list.length == 7)
-    $("#create_new_cluster_form tr").has("input[name^='node-']").last().next().remove();
-}
-
-function create_cluster_display_rrp(transport) {
-  if(transport == 'udp') {
-    $('#rrp_udp_transport').show();
-    $('#rrp_udpu_transport').hide();
-  }
-  else {
-    $('#rrp_udp_transport').hide();
-    $('#rrp_udpu_transport').show();
-  };
-}
-
 function show_hide_constraints(element) {
   //$(element).parent().siblings().each (function(index,element) {
   $(element).parent().nextUntil(".stop").toggle();
@@ -1348,12 +1065,21 @@ function remove_cluster(ids) {
       Pcs.update();
     },
     error: function (xhr, status, error) {
-      alert("Unable to remove cluster: " + res + " ("+error+")");
+      alert(
+        "Unable to remove cluster "
+        + ajax_simple_error(xhr, status, error)
+      );
       $("#dialog_verify_remove_clusters.ui-dialog-content").each(function(key, item) {$(item).dialog("destroy")});
     }
   });
 }
 
+/*
+ * TODO Remove - dead code
+ * This function was handling the old node remove dialog used in pcs-0.9 for
+ * CMAN and Corosync 2 clusters. We keep it here for now until the dialog
+ * overhaul is done.
+*/
 function remove_nodes(ids, force) {
   var data = {};
   for (var i=0; i<ids.length; i++) {
@@ -1455,7 +1181,7 @@ function remove_resource(ids, force) {
       }
       alert(message);
       $("#dialog_verify_remove_resources.ui-dialog-content").each(
-        function(key, item) { $(item).dialog("destroy"); }
+        function(key, item) { $(item).dialog("destroy") }
       );
       $("#dialog_verify_remove_resources input[name=force]").attr("checked", false);
       Pcs.update();
@@ -1655,7 +1381,9 @@ function add_constraint_prepare_data(parent_id, constraint_type){
 
 function add_constraint(parent_id, c_type, force) {
   var data = add_constraint_prepare_data(parent_id, c_type);
-  data["disable_autocorrect"] = true;
+  if (!Pcs.pcmk_constraint_no_autocorrect) {
+    data["disable_autocorrect"] = true;
+  }
   data["res_id"] = Pcs.resourcesContainer.cur_resource.get('id');
   data["node_id"] = $(parent_id + " input[name='node_id']").val();
   data["c_type"] = c_type;
@@ -1716,9 +1444,11 @@ function add_constraint_set_get_options(parent_id, constraint_type){
 function add_constraint_set(parent_id, c_type, force) {
   var data = {
     resources: [],
-    disable_autocorrect: true,
     options: {},
   };
+  if (!Pcs.pcmk_constraint_no_autocorrect) {
+    data["disable_autocorrect"] = true;
+  }
   $(parent_id + " input[name='resource_ids[]']").each(function(index, element) {
     var resources = element.value.trim();
     if (resources.length > 0) {
@@ -1733,7 +1463,7 @@ function add_constraint_set(parent_id, c_type, force) {
   if (data['resources'].length < 1) {
     return;
   }
-  fade_in_out($(parent_id))
+  fade_in_out($(parent_id));
 
   ajax_wrapper({
     type: "POST",
@@ -1826,7 +1556,7 @@ function remove_constraint_rule(id) {
 }
 
 function add_acl_role(form) {
-  var data = {}
+  var data = {};
   data["name"] = $(form).find("input[name='name']").val().trim();
   data["description"] = $(form).find("input[name='description']").val().trim();
   ajax_wrapper({
@@ -1859,7 +1589,7 @@ function remove_acl_roles(ids) {
     timeout: pcs_timeout*3,
     success: function(data,textStatus) {
       $("#dialog_verify_remove_acl_roles.ui-dialog-content").each(
-        function(key, item) { $(item).dialog("destroy"); }
+        function(key, item) { $(item).dialog("destroy") }
       );
       Pcs.update();
     },
@@ -1869,7 +1599,7 @@ function remove_acl_roles(ids) {
         + ajax_simple_error(xhr, status, error)
       );
       $("#dialog_verify_remove_acl_roles.ui-dialog-content").each(
-        function(key, item) { $(item).dialog("destroy"); }
+        function(key, item) { $(item).dialog("destroy") }
       );
     }
   });
@@ -1885,13 +1615,13 @@ function add_acl_item(parent_id, item_type) {
       data["type"] = $(parent_id + " select[name='role_type']").val();
       data["xpath_id"] = $(parent_id + " select[name='role_xpath_id']").val();
       data["query_id"] = $(parent_id + " input[name='role_query_id']").val().trim();
-      item_label = "permission"
+      item_label = "permission";
       break;
     case "user":
     case "group":
       data["item"] = item_type;
       data["usergroup"] = $(parent_id + " input[name='role_assign_user']").val().trim();
-      item_label = item_type
+      item_label = item_type;
       break;
   }
   fade_in_out($(parent_id));
@@ -1921,15 +1651,15 @@ function remove_acl_item(id,item) {
     case "perm":
       data["item"] = "permission";
       data["acl_perm_id"] = id.attr("acl_perm_id");
-      item_label = "permission"
+      item_label = "permission";
       break;
     case "group":
     case "user":
       data["item"] = "usergroup";
       data["item_type"] = item;
-      data["usergroup_id"] = id.attr("usergroup_id")
-      data["role_id"] = id.attr("role_id")
-      item_label = "user / group"
+      data["usergroup_id"] = id.attr("usergroup_id");
+      data["role_id"] = id.attr("role_id");
+      item_label = "user / group";
       break;
   }
 
@@ -2020,7 +1750,7 @@ function checkBoxToggle(cb,nodes) {
   if (nodes) {
     cbs = $('#node_list table').find(".node_list_check input[type=checkbox]");
   } else {
-    cbs = $(cb).closest("tr").parent().find(".node_list_check input[type=checkbox]")
+    cbs = $(cb).closest("tr").parent().find(".node_list_check input[type=checkbox]");
   }
   if ($(cb).prop('checked'))
     cbs.prop('checked',true).change();
@@ -2028,22 +1758,6 @@ function checkBoxToggle(cb,nodes) {
     cbs.prop('checked',false).change();
 }
 
-function loadWizard(item) {
-  wizard_name = $(item).val();
-  data = {wizard: wizard_name};
-
-  $("#wizard_location").load(
-   get_cluster_remote_url() + 'get_wizard',
-   data);
-}
-
-function wizard_submit(form) {
-  data = $(form).serialize();
-  $("#wizard_location").load(
-    get_cluster_remote_url() + 'wizard_submit',
-    data);
-}
-
 function update_resource_type_options() {
   var cp = $("#resource_class_provider_selector").val();
   var target = $("#add_ra_type");
@@ -2250,7 +1964,7 @@ function get_resource_agent_metadata(agent, on_success, stonith) {
         + ajax_simple_error(xhr, status, error)
       );
     }
-  })
+  });
 }
 
 function update_instance_attributes(resource_id) {
@@ -2333,6 +2047,28 @@ function resource_master(resource_id) {
   });
 }
 
+function resource_promotable(resource_id) {
+  if (resource_id == null) {
+    return;
+  }
+  show_loading_screen();
+  ajax_wrapper({
+    type: 'POST',
+    url: get_cluster_remote_url() + 'resource_promotable',
+    data: {resource_id: resource_id},
+    timeout: pcs_timeout,
+    error: function (xhr, status, error) {
+      alert(
+        "Unable to create promotable resource "
+        + ajax_simple_error(xhr, status, error)
+      );
+    },
+    complete: function() {
+      Pcs.update();
+    }
+  });
+}
+
 function resource_clone(resource_id) {
   if (resource_id == null) {
     return;
@@ -2449,7 +2185,7 @@ function resource_change_group(resource_id, form) {
 }
 
 function ajax_simple_error(xhr, status, error) {
-  var message = "(" + $.trim(error) + ")"
+  var message = "(" + $.trim(error) + ")";
   if (
     $.trim(xhr.responseText).length > 0
     &&
@@ -2490,7 +2226,7 @@ function ajax_wrapper(options) {
     else {
       error_original(xhr, status, error);
     }
-  }
+  };
   // Do not run complete function if login dialog is open.
   // Once user is logged in again, the original complete function will be run
   // in repeated ajax call run by login dialog on success.
@@ -2501,7 +2237,7 @@ function ajax_wrapper(options) {
     else {
       complete_original(xhr, status);
     }
-  }
+  };
 
   // run ajax request or put it into a queue
   if (login_dialog_opened) {
@@ -2521,7 +2257,7 @@ function login_dialog(on_success) {
       id: ok_button_id,
       click: function() {
         var me = $(this);
-        var my_dialog = $(this).dialog()
+        var my_dialog = $(this).dialog();
         my_dialog.find("#login_form_denied").hide();
         $(ok_button_selector).button("option", "disabled", true);
         $.ajax({
@@ -2609,7 +2345,7 @@ function permissions_load_all() {
     if (call_count < 1) {
       hide_loading_screen();
     }
-  }
+  };
 
   $.each(cluster_list, function(index, cluster) {
     permissions_load_cluster(cluster, callback);
@@ -2921,7 +2657,7 @@ Ember.Handlebars.helper('selector-helper', function (content, value, place_holde
   $.each(content, function(_, opt){
     line = '<option value="' + opt["value"] + '"';
     if (value == opt["value"]) {
-      line += ' selected="selected"'
+      line += ' selected="selected"';
     }
     line += ">" + Handlebars.Utils.escapeExpression(opt["name"]) + "</option>";
     out += line + "\n";
@@ -2934,9 +2670,9 @@ Ember.Handlebars.helper('bool-to-icon', function(value, options) {
   if (typeof(value) == 'undefined' || value == null) {
     out += "questionmarkdark";
   } else if (value) {
-    out += "checkdark"
+    out += "checkdark";
   } else {
-    out += "Xdark"
+    out += "Xdark";
   }
   return new Handlebars.SafeString(out + '">&nbsp;</span>');
 });
@@ -3026,7 +2762,7 @@ function enable_sbd_dialog(node_list) {
           '/>' +
         '</td>' +
       '</tr>'
-    )
+    );
   });
 }
 
